package contexts;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.util.*;
import ContextExpressionLanguage.analysis.*;
import ContextExpressionLanguage.node.*;
import ContextExpressionLanguage.analysis.DepthFirstAdapter;


public class ContextExpressionInterpreter extends DepthFirstAdapter {
//  private static Properties props = new Properties();
  private Hashtable values = new Hashtable();
  private Vector argumentsList = new Vector();
  private Hashtable arguments  = new Hashtable();
  private Hashtable contexts   = new Hashtable();
  private boolean error        = false;
  private String errorMessage  = "";
  private boolean debugMode    = false;
  private Object resultValue   = null;

  //Mtodos privados
  private void setError(String message, int line, int col) {
    if (!error) {
      error = true;
      errorMessage = "Erro: "+message+" na linha "+line+", coluna "+col+".";
      if (debugMode) {
        System.out.println(errorMessage);
      }
    }
  }

  private void setIntValue(Node node, int value) {
    values.put(node, new Integer(value));
  }

  private int getIntValue(Node node) {
    return ((Integer) values.get(node)).intValue();
  }

  private void setBoolValue(Node node, boolean value) {
    values.put(node, new Boolean(value));
  }

  private boolean getBoolValue(Node node) {
    return ((Boolean) values.get(node)).booleanValue();
  }

  private void setStringValue(Node node, String value) {
    values.put(node, value);
  }

  private String getStringValue(Node node) {
    return ((String) values.get(node));
  }

  private boolean orShortCircuitEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    if (this.getBoolValue(left)) {
      return true;
    }
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica ||");
    }
    return this.getBoolValue(right);
  }

  private boolean orStrictEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica ||");
    }
    return this.getBoolValue(left) || this.getBoolValue(right);
  }

  private boolean andShortCircuitEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    if (!this.getBoolValue(left)) {
      return false;
    }
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica &&");
    }
    return this.getBoolValue(right);
  }

  private boolean andStrictEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica &&");
    }
    return this.getBoolValue(left) && this.getBoolValue(right);
  }

  private boolean eqEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica =");
    }
    Object valLeft  = values.get(left);
    Object valRight = values.get(right);
    if ((valLeft instanceof String) && (valRight instanceof String)) {
      return ((String) valLeft).equalsIgnoreCase((String) valRight);
    }
    else {
      return valLeft.equals(valRight);
    }
  }

  private boolean neqEvaluation(Node left, Node right)  throws ContextException {
    return !this.eqEvaluation(left, right);
  }

  private boolean ltEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica <");
    }
    return this.getIntValue(left) < this.getIntValue(right);
  }

  private boolean gtEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica >");
    }
    return this.getIntValue(left) > this.getIntValue(right);
  }

  private boolean lteqEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica <=");
    }
    return this.getIntValue(left) <= this.getIntValue(right);
  }

  private boolean gteqEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao lgica >=");
    }
    return this.getIntValue(left) >= this.getIntValue(right);
  }

  private boolean notEvaluation(Node node)  throws ContextException {
    node.apply(this);  // Evaluate the expression
    if (error) {
      throw new ContextException("falha na operao lgica (NOT) !");
    }
    return !this.getBoolValue(node);
  }

  private int plusEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao de adio");
    }
    return this.getIntValue(left) + this.getIntValue(right);
  }

  private int minusEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao de subtrao");
    }
    return this.getIntValue(left) - this.getIntValue(right);
  }

  private int multEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao de mutiplicao");
    }
    return this.getIntValue(left) * this.getIntValue(right);
  }

  private int divEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao de diviso");
    }
    return this.getIntValue(left) / this.getIntValue(right);
  }

  private int modEvaluation(Node left, Node right)  throws ContextException {
    left.apply(this);  // Evaluate the left expression
    right.apply(this); // Evaluate the right expression
    if (error) {
      throw new ContextException("falha na operao de mdulo");
    }
    return this.getIntValue(left) % this.getIntValue(right);
  }

/*
  private void loadContexts() throws java.lang.ClassNotFoundException,
                                     java.lang.IllegalAccessException,
                                     java.lang.InstantiationException  {
    StringTokenizer ctxs = new StringTokenizer(props.getProperty("contexts"), ";");
    while (ctxs.hasMoreTokens()) {
      StringTokenizer ctx = new StringTokenizer(ctxs.nextToken(), ",");
      contexts.put(ctx.nextToken(), Class.forName(ctx.nextToken()).newInstance());
    }
  }
*/

  //Mtodos pblicos
  public ContextExpressionInterpreter(boolean debug) /*throws Exception*/ {
    super();
//    try {
      this.debugMode = debug;
/*      props.load(new FileInputStream("maca_cs.properties"));
      this.loadContexts();
      if (debugMode) {
        System.out.println("Contextos carregados.");
      }  */
/*    }
    catch (Exception exc) {
      if (debugMode) {
        System.out.println("Error carregando contexto: "+exc);
      }
      throw exc;
    }  */
  }

  public boolean loadContext(String aContextName, Context aContext) {
    if (!this.contexts.containsKey(aContextName)) {
      this.contexts.put(aContextName, aContext);
      return true;
    }
    return false;
  }

  public void loadContext(Hashtable aContexts) {
    Hashtable newContexts = new Hashtable(aContexts);
    newContexts.putAll(this.contexts);
    this.contexts = newContexts;
  }

  public boolean getError() {
    return error;
  }

  public String errorMessage() {
    return errorMessage;
  }

  public Object getResultValue() {
    return this.resultValue;
  }

  public void resetError() {
    this.error = false;
    this.errorMessage = "";
    this.resultValue  = null;
  }

  public void caseStart(Start node) {
    inStart(node);
    try {
      node.getPExpression().apply(this);
    }
    catch (Exception e) {
      this.setError(e.toString(), 1, 1);
    }
    if (!error) {
      this.resultValue = this.values.get(node.getPExpression());
      if (debugMode) {
        System.out.println(node+" >>>>>>>>> "+this.values.get(node.getPExpression()));
      }
    }
    node.getEOF().apply(this);
    outStart(node);
  }

  public void caseALogicalExpressionExpression(ALogicalExpressionExpression node) {
    inALogicalExpressionExpression(node);
    if ((!error) && node.getLogicalExpression() != null) {
      node.getLogicalExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getLogicalExpression()));
      }
    }
    outALogicalExpressionExpression(node);
  }

  public void caseAExpressionApplicationExpression(AExpressionApplicationExpression node) {
    inAExpressionApplicationExpression(node);
    if ((!error) && node.getExpressionApplication() != null) {
      node.getExpressionApplication().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getExpressionApplication()));
      }
    }
    outAExpressionApplicationExpression(node);
  }

  public void caseAExpressionApplication(AExpressionApplication node) {
    inAExpressionApplication(node);
    if ((!error) && node.getArgumentList() != null) {
      node.getArgumentList().apply(this);
    }
    if ((!error) && node.getExpressionDeclaration() != null) {
      node.getExpressionDeclaration().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getExpressionDeclaration()));
      }
    }
    outAExpressionApplication(node);
  }

  public void caseAExpressionArgumentList(AExpressionArgumentList node) {
    inAExpressionArgumentList(node);
    if ((!error) && node.getExpression() != null) {
//      node.getExpression().apply(this);
//      this.argumentsList.add(this.values.get(node.getExpression()));
      this.argumentsList.add(node.getExpression());
      if (debugMode) {
        System.out.println("Exp================"+node.getExpression());
      }
    }
    outAExpressionArgumentList(node);
  }

  public void caseAArgumentListArgumentList(AArgumentListArgumentList node) {
    inAArgumentListArgumentList(node);
    if ((!error) && node.getArgumentList() != null) {
      node.getArgumentList().apply(this);
    }
    if ((!error) && node.getExpression() != null) {
//      node.getExpression().apply(this);
//      this.argumentsList.add(this.values.get(node.getExpression()));
      this.argumentsList.add(node.getExpression());
      if (debugMode) {
        System.out.println("ExpArgList================"+node.getExpression());
      }
    }
    outAArgumentListArgumentList(node);
  }

  public void caseAExpressionDeclaration(AExpressionDeclaration node) {
    inAExpressionDeclaration(node);
    if ((!error) && node.getExpressionHeader() != null) {
      node.getExpressionHeader().apply(this);
    }
    if ((!error) && node.getExpressionBody() != null) {
      node.getExpressionBody().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getExpressionBody()));
      }
    }
    outAExpressionDeclaration(node);
  }

  public void caseAFormalParameterFormalParameterList(AFormalParameterFormalParameterList node) {
    inAFormalParameterFormalParameterList(node);
    if ((!error) && node.getFormalParameter() != null) {
      node.getFormalParameter().apply(this);
      if (!error) {
        this.arguments.put(((AFormalParameter)node.getFormalParameter()).getIdentifier().getText(),
                           this.argumentsList.firstElement());
        this.argumentsList.remove(0);
        if (debugMode) {
          System.out.println(((AFormalParameter)node.getFormalParameter()).getIdentifier().getText()+"==============="+
                             this.arguments.get(((AFormalParameter)node.getFormalParameter()).getIdentifier().getText()));
        }
      }
    }
    outAFormalParameterFormalParameterList(node);
  }

  public void caseAFormalParameterListFormalParameterList(AFormalParameterListFormalParameterList node) {
    inAFormalParameterListFormalParameterList(node);
    if ((!error) && node.getFormalParameterList() != null) {
      node.getFormalParameterList().apply(this);
    }
    if ((!error) && node.getFormalParameter() != null) {
      node.getFormalParameter().apply(this);
      if (!error) {
        this.arguments.put(((AFormalParameter)node.getFormalParameter()).getIdentifier().getText(),
                           this.argumentsList.firstElement());
        this.argumentsList.remove(0);
        if (debugMode) {
          System.out.println(((AFormalParameter)node.getFormalParameter()).getIdentifier().getText()+"==============="+
                             this.arguments.get(((AFormalParameter)node.getFormalParameter()).getIdentifier().getText()));
        }
      }
    }
    outAFormalParameterListFormalParameterList(node);
  }

  public void caseAIdentifierFunctionApplication(AIdentifierFunctionApplication node) {
    inAIdentifierFunctionApplication(node);
    if ((!error) && node.getArgumentList() != null) {
      node.getArgumentList().apply(this);
    }
    if ((!error) && node.getIdentifier() != null) {
      Node expDcl = (Node) this.arguments.get(node.getIdentifier().getText());
      expDcl.apply(this);
      if (!error) {
        this.values.put(node, this.values.get(expDcl));
      }
    }
    outAIdentifierFunctionApplication(node);
  }

  public void caseAQualifiedIdentifierFunctionApplication(AQualifiedIdentifierFunctionApplication node) {
    int count = 0;
    inAQualifiedIdentifierFunctionApplication(node);
    if ((!error) && node.getArgumentList() != null) {
      count = this.argumentsList.size();
      node.getArgumentList().apply(this);
      count = this.argumentsList.size() - count;
    }
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      Context ctx = (Context) contexts.get(node.getLeft().getText());
      Vector args = new Vector();
      if (ctx != null) {
        for (int n=0; (!error) && (n < count) ; n++) {
          Node arg = (Node) this.argumentsList.get(0);
          this.argumentsList.remove(0);
          arg.apply(this);
          if (!error) {
            args.add(this.values.get(arg));
          }
        }
      }
      else {
        this.setError("contexto '"+node.getLeft().getText()+"' no definido.", node.getQualifier().getLine(), node.getQualifier().getPos());
      }
      if (!error) {
        this.values.put(node, ctx.functionApplication(node.getRight().getText(), args));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getRight().getLine(), node.getRight().getPos());
      }
    }
    outAQualifiedIdentifierFunctionApplication(node);
  }


  public void caseAExpressionBody(AExpressionBody node) {
    inAExpressionBody(node);
    if ((!error) && node.getLogicalExpression() != null) {
      node.getLogicalExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getLogicalExpression()));
      }
    }
    outAExpressionBody(node);
  }

  public void caseAAndExpressionLogicalExpression(AAndExpressionLogicalExpression node) {
    inAAndExpressionLogicalExpression(node);
    if ((!error) && node.getAndExpression() != null) {
      node.getAndExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getAndExpression()));
      }
    }
    outAAndExpressionLogicalExpression(node);
  }

  public void caseAEqualityExpressionAndExpression(AEqualityExpressionAndExpression node) {
    inAEqualityExpressionAndExpression(node);
    if ((!error) && node.getEqualityExpression() != null) {
      node.getEqualityExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getEqualityExpression()));
      }
    }
    outAEqualityExpressionAndExpression(node);
  }

  public void caseAAndAndExpression(AAndAndExpression node) {
    inAAndAndExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.andShortCircuitEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getAnd().getLine(), node.getAnd().getPos());
      }
    }
    outAAndAndExpression(node);
  }

  public void caseAOrLogicalExpression(AOrLogicalExpression node)  {
    inAOrLogicalExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.orShortCircuitEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getOr().getLine(), node.getOr().getPos());
      }
    }
    outAOrLogicalExpression(node);
  }

  public void caseARelationalExpressionEqualityExpression(ARelationalExpressionEqualityExpression node) {
    inARelationalExpressionEqualityExpression(node);
    if ((!error) && node.getRelationalExpression() != null) {
      node.getRelationalExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getRelationalExpression()));
      }
    }
    outARelationalExpressionEqualityExpression(node);
  }

  public void caseAEqEqualityExpression(AEqEqualityExpression node) {
    inAEqEqualityExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.eqEvaluation(node.getLeft(), node.getRight()));
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getEq().getLine(), node.getEq().getPos());
      }
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
    }
    outAEqEqualityExpression(node);
  }

  public void caseANeqEqualityExpression(ANeqEqualityExpression node) {
    inANeqEqualityExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.neqEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getNeq().getLine(), node.getNeq().getPos());
      }
    }
    outANeqEqualityExpression(node);
  }

  public void caseAAdditiveExpressionRelationalExpression(AAdditiveExpressionRelationalExpression node) {
    inAAdditiveExpressionRelationalExpression(node);
    if ((!error) && node.getAdditiveExpression() != null) {
      node.getAdditiveExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getAdditiveExpression()));
      }
    }
    outAAdditiveExpressionRelationalExpression(node);
  }

  public void caseAInRelationalExpression(AInRelationalExpression node) {
    inAInRelationalExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      node.getLeft().apply(this);
      node.getRight().apply(this);
      if ((!error) && node.getRight() instanceof AQualifiedSetIdentifierSetExpression) {
        Context ctx = (Context) contexts.get(((AQualifiedSetIdentifierSetExpression)node.getRight()).getLeft().getText());
        if (ctx != null) {
          this.setBoolValue(node, ctx.inEvaluation(this.values.get(node.getLeft()), ((AQualifiedSetIdentifierSetExpression)node.getRight()).getRight().getText()));
        }
        else {
          this.setError("contexto '"+((AQualifiedSetIdentifierSetExpression)node.getRight()).getLeft().getText()+"' no definido.", node.getIn().getLine(), node.getIn().getPos());
        }
      }
      else if ((!error) && node.getRight() instanceof ASetIdentifierSetExpression) {
        Node arg = (Node) this.arguments.get(((ASetIdentifierSetExpression)node.getRight()).getIdentifier().getText());
        StringTokenizer strTk = new StringTokenizer(arg.toString(), ".");
        String left = strTk.nextToken().trim();
        Context ctx = (Context) contexts.get(left);
        if (ctx != null) {
          String right = strTk.nextToken().trim();
          this.setBoolValue(node, ctx.inEvaluation(this.values.get(node.getLeft()), right));
        }
        else {
          this.setError("contexto '"+left+"' no definido.", node.getIn().getLine(), node.getIn().getPos());
        }
      }
      else if ((!error) && node.getRight() instanceof ASetFunctionApplicationSetExpression) {
        PFunctionApplication funcAp = ((ASetFunctionApplicationSetExpression) node.getRight()).getFunctionApplication();
        if (funcAp instanceof AQualifiedIdentifierFunctionApplication) {
          Context ctx = (Context) contexts.get(((AQualifiedIdentifierFunctionApplication)funcAp).getLeft().getText());
          if (ctx != null) {
            this.setBoolValue(node, ctx.inEvaluation(this.values.get(node.getLeft()), this.values.get(funcAp)));
          }
          else {
            this.setError("contexto '"+((AQualifiedIdentifierFunctionApplication)funcAp).getLeft().getText()+"' no definido.", node.getIn().getLine(), node.getIn().getPos());
          }
        }
        else if (funcAp instanceof AIdentifierFunctionApplication) {
          this.setError("contexto da funo '"+((AIdentifierFunctionApplication)funcAp).getIdentifier().getText()+"' no definido.", node.getIn().getLine(), node.getIn().getPos());
        }
        else {
          this.setError("Funo no definida no operador 'in'.", node.getIn().getLine(), node.getIn().getPos());
        }
      }
      else if (!error) {
        StringTokenizer setOperands = new StringTokenizer(this.getStringValue(node.getRight()), " ");
        int leftOperand    = this.getIntValue(node.getLeft());
        this.setBoolValue(node, (leftOperand >= Integer.parseInt(setOperands.nextToken())) &&
                                (leftOperand <= Integer.parseInt(setOperands.nextToken())));
      }
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getIn().getLine(), node.getIn().getPos());
      }
    }
    outAInRelationalExpression(node);
  }

  public void caseALteqRelationalExpression(ALteqRelationalExpression node) {
    inALteqRelationalExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.lteqEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getLteq().getLine(), node.getLteq().getPos());
      }
    }
    outALteqRelationalExpression(node);
  }

  public void caseAGteqRelationalExpression(AGteqRelationalExpression node) {
    inAGteqRelationalExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.gteqEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getGteq().getLine(), node.getGteq().getPos());
      }
    }
    outAGteqRelationalExpression(node);
  }

  public void caseALtRelationalExpression(ALtRelationalExpression node) {
    inALtRelationalExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.ltEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getLt().getLine(), node.getLt().getPos());
      }
    }
    outALtRelationalExpression(node);
  }

  public void caseAGtRelationalExpression(AGtRelationalExpression node) {
    inAGtRelationalExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setBoolValue(node, this.gtEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getGt().getLine(), node.getGt().getPos());
      }
    }
    outAGtRelationalExpression(node);
  }

  public void caseASetLiteralSetExpression(ASetLiteralSetExpression node) {
      inASetLiteralSetExpression(node);
      if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) {
        node.getLeft().apply(this);
        node.getRight().apply(this);
        if (!error) {
          this.values.put(node, values.get(node.getLeft()) + "  " + values.get(node.getRight()));
        }
      }
      outASetLiteralSetExpression(node);
  }

  public void caseAMultiplicativeExpressionAdditiveExpression(AMultiplicativeExpressionAdditiveExpression node) {
    inAMultiplicativeExpressionAdditiveExpression(node);
    if ((!error) && node.getMultiplicativeExpression() != null) {
      node.getMultiplicativeExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getMultiplicativeExpression()));
      }
    }
    outAMultiplicativeExpressionAdditiveExpression(node);
  }

  public void caseAPlusAdditiveExpression(APlusAdditiveExpression node) {
    inAPlusAdditiveExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null))   { try {
      this.setIntValue(node, this.plusEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getIntValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getPlus().getLine(), node.getPlus().getPos());
      }
    }
    outAPlusAdditiveExpression(node);
  }

  public void caseAMinusAdditiveExpression(AMinusAdditiveExpression node) {
    inAMinusAdditiveExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setIntValue(node, this.minusEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getIntValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getMinus().getLine(), node.getMinus().getPos());
      }
    }
    outAMinusAdditiveExpression(node);
  }

  public void caseAUnaryExpressionMultiplicativeExpression(AUnaryExpressionMultiplicativeExpression node) {
    inAUnaryExpressionMultiplicativeExpression(node);
    if ((!error) && node.getUnaryExpression() != null) {
      node.getUnaryExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getUnaryExpression()));
      }
    }
    outAUnaryExpressionMultiplicativeExpression(node);
  }

  public void caseAStarMultiplicativeExpression(AStarMultiplicativeExpression node) {
    inAStarMultiplicativeExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setIntValue(node, this.multEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getIntValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getStar().getLine(), node.getStar().getPos());
      }
    }
    outAStarMultiplicativeExpression(node);
  }

  public void caseADivMultiplicativeExpression(ADivMultiplicativeExpression node) {
    inADivMultiplicativeExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setIntValue(node, this.divEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getIntValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getDiv().getLine(), node.getDiv().getPos());
      }
    }
    outADivMultiplicativeExpression(node);
  }

  public void caseAModMultiplicativeExpression(AModMultiplicativeExpression node) {
    inAModMultiplicativeExpression(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      this.setIntValue(node, this.modEvaluation(node.getLeft(), node.getRight()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getIntValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getMod().getLine(), node.getMod().getPos());
      }
    }
    outAModMultiplicativeExpression(node);
  }


  public void caseAValueUnaryExpression(AValueUnaryExpression node) {
    inAValueUnaryExpression(node);
    if ((!error) && node.getValue() != null) {
      node.getValue().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getValue()));
      }
    }
    outAValueUnaryExpression(node);
  }

  public void caseANotUnaryExpression(ANotUnaryExpression node) {
    inANotUnaryExpression(node);
    if ((!error) && node.getUnaryExpression() != null) { try {
      this.setBoolValue(node, this.notEvaluation(node.getUnaryExpression()));
      if (debugMode) {
        System.out.println(node + " >>>>>>> " + this.getBoolValue(node));
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getNot().getLine(), node.getNot().getPos());
      }
    }
    outANotUnaryExpression(node);
  }

  public void caseATrueValue(ATrueValue node) {
    inATrueValue(node);
    if ((!error) && node.getTrue() != null) {
      this.setBoolValue(node, true);
    }
    outATrueValue(node);
  }

  public void caseAFalseValue(AFalseValue node) {
    inAFalseValue(node);
    if ((!error) && node.getFalse() != null) {
      this.setBoolValue(node, false);
    }
    outAFalseValue(node);
  }

  public void caseANumberValue(ANumberValue node) {
    inANumberValue(node);
    if ((!error) && node.getNumber() != null) { try {
      int value = Integer.parseInt(node.getNumber().getText()); // parse the value
      this.setIntValue(node, value);
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getNumber().getLine(), node.getNumber().getPos());
      }
    }
    outANumberValue(node);
  }

  public void caseATextValue(ATextValue node) {
    inATextValue(node);
    if ((!error) && node.getText() != null) {
      String txt = node.getText().getText();
      this.setStringValue(node, txt.substring(1, txt.length() - 1));
    }
    outATextValue(node);
  }

  public void caseAIdentifierValue(AIdentifierValue node) {
    inAIdentifierValue(node);
    if ((!error) && node.getIdentifier() != null) {
      Object arg = this.arguments.get(node.getIdentifier().getText());
      if (arg instanceof Node) {
        ((Node)arg).apply(this);
        if (!error) {
          this.arguments.put(node.getIdentifier().getText(), this.values.get(arg));
        }
      }
      if (!error) {
        this.values.put(node, this.arguments.get(node.getIdentifier().getText()));
      }
    }
    outAIdentifierValue(node);
  }

  public void caseAQualifiedIdentifierValue(AQualifiedIdentifierValue node) {
    inAQualifiedIdentifierValue(node);
    if ((!error) && (node.getLeft() != null) && (node.getRight() != null)) { try {
      Context ctx = (Context) contexts.get(node.getLeft().getText());
      if (ctx != null) {
        this.values.put(node, ctx.getValue(node.getRight().getText()));
      }
      else {
        this.setError("contexto '"+node.getLeft().getText()+"' no definido.", node.getQualifier().getLine(), node.getQualifier().getPos());
      }
      }
      catch(Exception e) {
        this.setError(e.toString(), node.getQualifier().getLine(), node.getQualifier().getPos());
      }
    }
    outAQualifiedIdentifierValue(node);
  }

  public void caseAFunctionApplicationValue(AFunctionApplicationValue node) {
    inAFunctionApplicationValue(node);
    if ((!error) && node.getFunctionApplication() != null) {
      node.getFunctionApplication().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getFunctionApplication()));
      }
    }
    outAFunctionApplicationValue(node);
  }

  public void caseAExpressionDeclarationValue(AExpressionDeclarationValue node) {
    inAExpressionDeclarationValue(node);
    if ((!error) && node.getExpressionDeclaration() != null) {
      node.getExpressionDeclaration().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getExpressionDeclaration()));
      }
    }
    outAExpressionDeclarationValue(node);
  }

  public void caseAExpressionValue(AExpressionValue node) {
    inAExpressionValue(node);
    if ((!error) && node.getExpression() != null) {
      node.getExpression().apply(this);
      if (!error) {
        this.values.put(node, this.values.get(node.getExpression()));
      }
    }
    outAExpressionValue(node);
  }

}
