package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
*/

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Random;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

public class Authorization {
  // Constantes
  public static final String ACTIVE   = "Ativada";
  public static final String INACTIVE = "Inativada";
  public static final String UNDEFINED = "?";
  // Variveis de instncia privadas
  private DirContextHolder ctx;
  private Attributes attrs;
  private String _authoRoot;
  private String _authoRDN;
  private boolean newAutho = false;
  private String _status = UNDEFINED;

  // Classes internas estticas

  // Mtodos privados definidos manualmente
  private String getNewAuthorizationID() throws NamingException {
    Random rdm               = new Random();
    NamingEnumeration answer = null;
    SearchControls      ctls = new SearchControls();
    String             newID = "invlida";
    int                    n = 0;
    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {""});
    n     = rdm.nextInt();
    newID = String.valueOf( n < 0 ? n*-1 : n);
    answer = ctx.value.search(this._authoRoot,"cn="+newID, ctls);
    while (answer.hasMore()) {
      newID = String.valueOf(rdm.nextInt());
      answer = ctx.value.search(this._authoRoot,"cn="+newID, ctls);
    }
    return newID;
  }

  private void newAuthorizationSetup(Group roleGroup, Group resGroup) throws NamingException {
    attrs = new BasicAttributes(true);
    Attribute oc = new BasicAttribute("objectclass");
    oc.add("top");
    oc.add("incorauthorization");
    attrs.put(oc);
    _authoRDN = this.getNewAuthorizationID();
    attrs.put("cn", _authoRDN);
    _authoRDN = "cn="+_authoRDN;
    attrs.put("incorresourcedn", resGroup.getGroupDN());
    Attribute attr = resGroup.getGroupAtributes().get("incorresourcetype");
    if (attr != null) {
      attrs.put("incorresourcetype", attr.get());
    }
    attrs.put("incorroledn", roleGroup.getGroupDN());
    attrs.put("incorauthorizationtype", "weak");
    attrs.put("incorprivilegetype", "-");
  }

  public Vector getConflictingAuthorizations() throws NamingException {
    Vector conflictingAuthos = new Vector();
    SearchControls               ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    NamingEnumeration  answer =
        ctx.value.search(System.getProperty("maca_ca.ldap.authorizationsRDN")+","+
                         System.getProperty("maca_ca.ldap.rootDN"),
                         "(&(incorresourcedn="+this.getResourceName()+")"+
                           "(&(incorprivilege="+this.getPrivilege()+")"+
                             "(&(incorauthorizationtype="+this.getAuthorizationType()+")"+
                               "(!(incorprivilegetype="+this.getPrivilegeType()+")))))", ctls);
    if (!answer.hasMoreElements())
      return conflictingAuthos;
    Hashtable childRoles = new Hashtable();
    Enumeration enum = this.getRole().getChildPathDNs(false);
    while (enum.hasMoreElements()) {
      String roledn = (String) enum.nextElement();
      childRoles.put(roledn, roledn);
    }
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      Authorization autho = new Authorization(this.ctx, this._authoRoot,
                                              sr.getName(),
                                              sr.getAttributes());
      if ((!this.getRoleDN().equalsIgnoreCase(autho.getRoleDN())) &&
           !childRoles.containsKey(autho.getRoleDN())) {
        if (autho.isValid()) {
          autho.setStatus(ACTIVE);
        }
        conflictingAuthos.add(autho);
      }
    }
    return conflictingAuthos;
  }

  public String toString() {
    try {
      return "Autorizao["+attrs.get("cn")+"]: <"+getRoleName()+", "+getResourceName()+", "+getPrivilegeType()+", "+getPrivilege()+", "+(isStrong()?"strong":"weak")+">";
    }
    catch (NamingException exc) {
      System.out.println(exc);
    }
    return "";
  }

  /**
   * Retorna a lista dos papis que herdam esta autorizao
   */
  public MutableTreeNode getInheritorRoles() throws Exception {
    return getInheritorRoles(this.getRole());
  }

  private MutableTreeNode getInheritorRoles(Role role) throws Exception {
    MutableTreeNode roleNode = new DefaultMutableTreeNode(role);
    Enumeration enum = role.children();
    while (enum.hasMoreElements()) {
      Role child = (Role)enum.nextElement();
      if (!child.hasEquivalentAuthorization(this)) {
          roleNode.insert(this.getInheritorRoles(child),
                           roleNode.getChildCount());
      }
    }
    return roleNode;
  }

  public Role getRole() throws NamingException {
    String roleName = this.getRoleDN();
    if (!roleName.equals("")) {
      roleName = roleName.substring(0, roleName.toLowerCase().indexOf(",cn="+System.getProperty("maca_ca.roleName")));
    }
    return new Role(ctx, System.getProperty("maca_ca.ldap.roleRDN")+","+
                         System.getProperty("maca_ca.ldap.groupsRDN")+","+
                         System.getProperty("maca_ca.ldap.rootDN"),
                         roleName);
  }

  public Group getResource() throws NamingException {
    String resName = this.getResourceName();
    if (!resName.equals("")) {
      int index = resName.toLowerCase().indexOf(","+System.getProperty("maca_ca.ldap.resourcesRDN"));
      if (index >= 0) {
        resName = resName.substring(0, index);
      }
      else {
        resName = "";
      }
    }
    return new Group(ctx, System.getProperty("maca_ca.ldap.resourcesRDN")+","+System.getProperty("maca_ca.ldap.rootDN"),
                     resName);
  }

  public boolean authorizationRoleHasEquivalentAuthorization() throws NamingException { //Verifica se o papel desta autorizao
                                                                                        //j tem uma autorizao com mesmo nome e recurso
    return this.getRole().hasEquivalentAuthorization(this);
  }

  protected boolean overridesAStrongAuthorization() throws NamingException {  //Verifica se tenta sobrepor uma authorizaao forte
    Role role = this.getRole();
    if (role != null) {
      Enumeration enum = role.getParentAuthorizations(true,
                                                      this.ldapResourceDNToResourceName(false),
                                                      this.getPrivilege());
      if (enum.hasMoreElements()) {
        return true;
      }
    }
    return false;
  }

  protected boolean isOverridedByAStrongAuthorization(Role aRole) throws NamingException {  //Verifica se  sobreposta por uma autorizaao FORTE
    if (this.isStrong() && (aRole != null)) {
      Enumeration childrenRolesEnum = aRole.children();
      while (childrenRolesEnum.hasMoreElements()) {
        Role role = (Role) childrenRolesEnum.nextElement();
        Enumeration authoEnum = role.getAuthorizations(true,
                                                       this.getPrivilege(),
                                                       this.ldapResourceDNToResourceName(false));
        if (authoEnum.hasMoreElements()) {
            return true;
        }
        if (this.isOverridedByAStrongAuthorization(role)) { //Verifica nos filhos dos filhos
          return true;
        }
      }
    }
    return false;
  }


  public void saveAuthorizationAtributes() throws Exception {
    if (!this.authorizationRoleHasEquivalentAuthorization()) { //Previne conflitos estticos (fraco e forte) no papel da autorizao
      if ((this.isWeak() || !this.overridesAStrongAuthorization())) {
        if (!this.isOverridedByAStrongAuthorization(this.getRole())) {
          if (!this.newAutho) {
            ctx.value.modifyAttributes(this.getAuthorizationDN(), DirContext.REPLACE_ATTRIBUTE, attrs);
          }
          else {
            this.addNewAuthorization();
          }
        }
        else {
          throw new Exception("Proibido REVOGAR autorizaoes FORTES descendentes.");
        }
      }
      else {
        throw new Exception("Proibido REVOGAR autorizaoes FORTES herdadas.");
      }
    }
    else {
      throw new Exception("Papel selecionado j possui esta autorizao.");
    }
  }

  public void addNewAuthorization() throws NamingException {
    if ((ctx != null) && (newAutho)) {
      ctx.value.createSubcontext(this.getAuthorizationDN(), attrs);
      attrs = ctx.value.getAttributes(this.getAuthorizationDN());
      newAutho = false;
    }
  }

  public void refreshAttributes() throws NamingException {
    if ((ctx != null) && (!newAutho)) {
      attrs = ctx.value.getAttributes(this.getAuthorizationDN());
    }
  }

  static private String removeSpaces(String str) {
    String strWithOutSpace = "";
    for (int n = 0; n < str.length(); n++) {
      char ch = str.charAt(n);
      strWithOutSpace = strWithOutSpace + (ch != ' ' ? String.valueOf(ch) : "");
    }
    return strWithOutSpace;
  }

  // Mtodos protected definidos manualmente
  protected String getAuthorizationDN() {
    String authoRDN;
    authoRDN = (_authoRDN=="")?_authoRDN:_authoRDN+",";
    return authoRDN+_authoRoot;
  }

  protected String resourceNameToLDAPresourceDN(String resName) {
    StringTokenizer strTk = new StringTokenizer(resName, ";");
    String resDN = "";
    while (strTk.hasMoreTokens()) {
      resDN = resDN + "cn="+strTk.nextToken()+",";
    }
    return resDN+System.getProperty("maca_ca.ldap.resourcesRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
  }

  public String resourceNameToLDAPresourceDN(String resName, boolean reverse) {
    return this.resourceNameToLDAPresourceRDN(resName, reverse) + ","+System.getProperty("maca_ca.ldap.resourcesRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
  }

  public String resourceNameToLDAPresourceRDN(String resName, boolean reverse) {
    StringTokenizer strTk = new StringTokenizer(resName, ";");
    String resDN = "";
    while (strTk.hasMoreTokens()) {
      if (reverse) {
        resDN = "cn="+strTk.nextToken()+(resDN.equalsIgnoreCase("")?"":",")+resDN;
      }
      else {
        resDN = resDN + "cn="+strTk.nextToken()+(strTk.hasMoreTokens()?",":"");
      }
    }
    return resDN;
  }

  public String ldapResourceDNToResourceName(boolean reverse) throws NamingException {
    StringTokenizer strTk = new StringTokenizer(this.getResourceName(), ",");
    String resName = "";
    while (strTk.hasMoreTokens()) {
      String nameComp = strTk.nextToken();
      if (nameComp.equalsIgnoreCase("cn=aplicacoes")) {
        if (reverse) {
          resName = nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (resName.equalsIgnoreCase("")?"":";") + resName;
        }
        else {
          resName = resName + nameComp.substring(nameComp.indexOf("=")+1, nameComp.length());
        }
        break;
      }
      else {
        if (reverse) {
          resName = nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (resName.equalsIgnoreCase("")?"":";") + resName;
        }
        else {
          resName = resName + nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (strTk.hasMoreTokens()?";":"");
        }
      }
    }
    return resName;
  }

  // Mtodos pblicos definidos manualmente
  public Attributes getAuthorizationAtributes() {
    return attrs;
  }

  public void removeAuthorization() throws NamingException {
    if (!newAutho) {
      ctx.value.destroySubcontext(this.getAuthorizationDN());
    }
  }

  public Enumeration getAccessModes() throws NamingException {
    Vector acsModesStr = new Vector();
    Attribute    resDN = attrs.get("incorResourceDN");
    if ((resDN != null) && (ctx != null)) {
      String resDNStr = (String) resDN.get();
      Attributes resAttrs = ctx.value.getAttributes(resDNStr);
      if (resAttrs != null) {
        Attribute acsModesAttr = resAttrs.get("incorresourceaccessmode");
        if (acsModesAttr != null) {
          Enumeration acsModesEnum = acsModesAttr.getAll();
          while (acsModesEnum.hasMoreElements()) {
            String acsModeStr = (String) acsModesEnum.nextElement();
            acsModesStr.add(acsModeStr);
          }
        }
      }
    }
    ObjectEnumeration enumStr = new ObjectEnumeration(acsModesStr);
    return enumStr;
  }

  public Enumeration getAccessModes(String resDNStr) throws NamingException {
    Vector acsModesStr = new Vector();
    if (ctx != null) {
      Attributes resAttrs = ctx.value.getAttributes(resDNStr);
      if (resAttrs != null) {
        Attribute acsModesAttr = resAttrs.get("incorresourceaccessmode");
        if (acsModesAttr != null) {
          Enumeration acsModesEnum = acsModesAttr.getAll();
          while (acsModesEnum.hasMoreElements()) {
            String acsModeStr = (String) acsModesEnum.nextElement();
            acsModesStr.add(acsModeStr);
          }
        }
      }
    }
    ObjectEnumeration enumStr = new ObjectEnumeration(acsModesStr);
    return enumStr;
  }

  public String getAuthorizationRDN(String authoRoot, Group role, Group resource) throws NamingException {
    SearchControls      ctls = new SearchControls();
    NamingEnumeration answer = null;
    ctls.setReturningAttributes(new String[] {"cn", "incorresourcedn"});
    answer = ctx.value.search(this.getAuthorizationDN(), "(incorroledn="+role.getGroupDN()+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr = (SearchResult) answer.next();
      Attributes attrs = sr.getAttributes();
      Attribute  attr  = attrs.get("incorresourcedn");
      if (attr != null) {
        String resDN = (String) attr.get();
        resDN = removeSpaces(resDN);
        if (resDN.equalsIgnoreCase(removeSpaces(resource.getGroupDN()))) {
          Attribute cnAttr = attrs.get("cn");
          if (cnAttr != null) {
            String res = (String) cnAttr.get();
            return  res;
          }
        }
      }
    }
    return "";
  }

  private boolean isAuthorization(String cnStr) throws NamingException {
    NamingEnumeration answer = null;
    SearchControls      ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {""});
    answer = ctx.value.search(this.getAuthorizationDN(), cnStr, ctls);
    return answer.hasMore();
  }

  public boolean isWeak() throws NamingException {
    Attribute attr = attrs.get("incorauthorizationtype");
    return (attr != null && ("weak".equalsIgnoreCase((String) attr.get())));
  }

  public boolean isStrong() throws NamingException {
    return !this.isWeak();
  }

  public boolean isNew() {
    return this.newAutho;
  }

  public String getAuthorizationType() throws NamingException {
    Attribute attr = attrs.get("incorauthorizationtype");
    if (attr != null) {
      return (String) attr.get();
    }
    else {
      return "";
    }
  }

  public String getPrivilegeType() throws NamingException {
    Attribute attr = attrs.get("incorprivilegetype");
    if (attr != null) {
      return (String) attr.get();
    }
    else {
      return "";
    }
  }

  public boolean isPositive() throws NamingException {
    Attribute attr = attrs.get("incorprivilegetype");
    return (attr != null && ("+".equalsIgnoreCase((String) attr.get())));
  }

  public boolean isNegative() throws NamingException {
    Attribute attr = attrs.get("incorprivilegetype");
    return (attr != null && ("-".equalsIgnoreCase((String) attr.get())));
  }

  protected boolean isValid() throws NamingException {
    Role role = this.getRole();
    Enumeration enum = role.getParentAuthorizations(true, this.ldapResourceDNToResourceName(false), this.getPrivilege());
    if (enum.hasMoreElements()) return false;
    return true;
  }


  public String getPrivilege() throws NamingException {
    try {
      Attribute attr = attrs.get("incorprivilege");
      if (attr != null) {
        return (String) attr.get();
      }
      else {
        return "";
      }
    }
    catch (java.lang.NullPointerException exc) {
      return "";
    }
  }

  public String getResourceName() throws NamingException {
    try {
      Attribute attr = attrs.get("incorresourcedn");
      if (attr != null) {
        return (String) attr.get();
      }
      else {
        return "";
      }
    }
    catch (java.lang.NullPointerException exc) {
      return "";
    }
  }

  public String getResourceType() throws NamingException {
    try {
      Attribute attr = attrs.get("incorresourcetype");
      if (attr != null) {
        return (String) attr.get();
      }
      else {
        return "";
      }
    }
    catch (java.lang.NullPointerException exc) {
      return "";
    }
  }

  public String getRoleDN() throws NamingException {
    try {
      Attribute attr = attrs.get("incorroledn");
      if (attr != null) {
        return (String) attr.get();
      }
      else {
        return "";
      }
    }
    catch (java.lang.NullPointerException exc) {
      return "";
    }
  }

  public String getRoleName() throws NamingException {
    StringTokenizer strTk = new StringTokenizer(this.getRoleDN(), ",", false);
    String roleName = strTk.nextToken();
    return ((roleName.length() > 3)?roleName.substring(3, roleName.length()):roleName);
  }

  public Authorization(DirContextHolder aContext, String authoRoot, String authoRDN) throws NamingException {

    this.ctx            = aContext;
    this._authoRoot     = authoRoot;
    this._authoRDN      = authoRDN;
    this.attrs          = ctx.value.getAttributes(this.getAuthorizationDN());
    this.newAutho       = false;
  }

  public String getStatus() throws NamingException {
    return this._status;
  }

  public boolean setStatus(String aNewStatus) throws NamingException {
    if (aNewStatus.equalsIgnoreCase(INACTIVE) ||
        aNewStatus.equalsIgnoreCase(ACTIVE) ||
        aNewStatus.equalsIgnoreCase(UNDEFINED)) {
      this._status = aNewStatus;
      return true;
    }
    return false;
  }

  public Authorization(DirContextHolder aContext, String authoRoot, String authoRDN, Attributes authoAttrs) {

    this.ctx            = aContext;
    this._authoRoot     = authoRoot;
    this._authoRDN      = authoRDN;
    this.attrs          = authoAttrs;
    newAutho            = false;
  }

  public Authorization(DirContextHolder aContext, String authoRoot, Group roleGroup, Group resGroup) throws NamingException {

    this.ctx            = aContext;
    this._authoRoot     = authoRoot;
    this._authoRDN      = "";
    this.newAutho       = true;
    this.newAuthorizationSetup(roleGroup, resGroup);
  }

}