package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.util.*;

class BooleanCache {
  static int   queueLenth = 10;
  private Hashtable queue = new Hashtable(queueLenth);

  class BooleanNode {
    private Boolean bool;
    private int weight = 0;
    private String key = "";

    public BooleanNode(String aKey, boolean aValue) {
      weight = 1;
      key   = aKey;
      bool   = new Boolean(aValue);
    }

    public String getKey() {
      return key;
    }

    public int getWeight() {
      return weight;
    }

    public boolean booleanValue() {
      weight = weight + 1;
      return bool.booleanValue();
    }
  }

  private boolean isFull() {
    return this.queue.size() >= queueLenth;
  }

  private void init(int aQueueLenth) {
    this.queueLenth = aQueueLenth;
    this.queue = new Hashtable(queueLenth);
  }

  public BooleanCache(int aQueueLenth) {
    this.init(aQueueLenth);
  }

  public BooleanCache() {
    int aQueueLenth = 10;
    try {
      aQueueLenth = Integer.parseInt(System.getProperty("maca_ca.user.cache.length"));
    }
    catch (NumberFormatException exc) {
    }
    this.init(aQueueLenth);
  }

  public boolean isEmpty() {
    return this.queue.isEmpty();
  }

  public synchronized void resetCache() throws Exception {
    this.queue.clear();
  }

  public synchronized void put(String str, boolean aBoolean) throws Exception {
    if (this.isFull()) {
      Enumeration enum = this.queue.elements();
      int min = Integer.MAX_VALUE;
      BooleanNode minNode = null;
      while (enum.hasMoreElements()) {
        BooleanNode node = (BooleanNode) enum.nextElement();
        if (min >= node.getWeight()) {
          min     = node.getWeight();
          minNode = node;
        }
      }
      if (minNode != null) {
        this.queue.remove(minNode.getKey());
      }
    }
    this.queue.put(str, new BooleanNode(str, aBoolean));
  }

  public synchronized boolean get(String str) throws Exception {
    if (this.isEmpty()) {
      throw new NotFoudInCacheException("Cache vazio");
    }
    BooleanNode result = (BooleanNode) this.queue.get(str);
    if (result == null) {
      throw new NotFoudInCacheException("Recurso '"+str+"' no encontrador no cache");
    }
    else {
      return result.booleanValue();
    }
  }







}