package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

 import java.io.*;
 import java.util.*;

 public abstract class LoadProperties {
      private static String roleName           = "papeis";
      private static String resourcesName      = "resources";
      private static String authorizationsName = "authorizations";
      private static String orgChartName       = "organograma";
      private static String peopleName         = "people";

      static {
        System.out.println("   MACA 3.2.2 - Middleware de Autenticao e Controle de Acesso");
        System.out.println("   Copyright (c) 2000-2005 Fundao Zerbini/Instituto do Corao - HC.FMUSP");
      }

      static public void loadPropertiesFile(LogWriter log, String fileName) {
        Properties props = new Properties();
        try {
          File file = new File(fileName);
          InputStream st = LoadProperties.class.getResourceAsStream("/"+file.getName());
          if (st != null) {
            props.load(st);
            st.close();
            if (log != null) log.println(java.util.logging.Level.INFO, "maca_cs",
                                         "maca_ca.LoadProperties", "loadPropertiesFile",
                                         "Recurso /"+file.getName()+" carregado");
            System.out.println("Recurso /"+file.getName()+" carregado");
          }
          try {
            st = new FileInputStream(fileName);
            Properties fileProps = new Properties();
            fileProps.load(st);
            props.putAll(fileProps);
            st.close();
            if (log != null) log.println(java.util.logging.Level.INFO, "maca_cs",
                                         "maca_ca.LoadProperties", "loadPropertiesFile",
                                         "Arquivo ./"+fileName+" carregado");
            System.out.println("Arquivo "+fileName+" carregado");
          }
          catch (java.io.FileNotFoundException exc) {
          }
          System.getProperties().putAll(props);
        }
        catch (Exception exc) {
          if (log != null)
             log.println(java.util.logging.Level.SEVERE, "maca_cs",
                         "maca_ca.LoadProperties", "loadPropertiesFile",
                         "Falha carregando "+fileName+": "+exc.getMessage());
          System.out.println("Falha carregando "+fileName+": "+exc.getMessage());
        }
      }

    static public synchronized void init(LogWriter log)  throws Exception {
      System.setProperty("maca_ca.roleName", roleName);
      System.setProperty("maca_ca.resourcesName", resourcesName);
      System.setProperty("maca_ca.authorizationsName", authorizationsName);
      System.setProperty("maca_ca.orgChartName", orgChartName);
      System.setProperty("maca_ca.peopleName", peopleName);

      System.setProperty("maca_ca.user.schema", "");
      System.setProperty("maca_ca.user.cache.length", "10");
      System.setProperty("maca_ca.mail", "");


      System.setProperty("maca_ca.log.LogFileCreationFrequency", "D");
      System.setProperty("maca_ca.log.LogFileMaxLength", "0");
      System.setProperty("maca_ca.log.LogMaxDirSize", "250");
      System.setProperty("maca_ca.log.LogFileMaxLengthFreq", "05");
      System.setProperty("maca_ca.log.LogMaxDirSizeFreq", "60");

      //Especifica o mdulo de gerao de senha para utilizar
      //O valor default  o 'Simple Password Factory'
      System.setProperty("maca_ca.passwordPolicy.passwordFactory", "maca_ca.SimplePasswordFactory");
      //Indica o tipo de algoritmo de hash usado para armazenar senhas
      System.setProperty("maca_ca.passwordPolicy.passwordStorageScheme", "SHA");
      //Indica se a sintaxe da senha ser verificada
      System.setProperty("maca_ca.passwordPolicy.passwordCheckSyntax", "false");
      //Nmero mnimo de caracteres de uma senha
      System.setProperty("maca_ca.passwordPolicy.passwordMinLength", "6");
      //Indica se a senha expira aps decorrido um dado nmero de segundos
      System.setProperty("maca_ca.passwordPolicy.passwordExp", "false");
      //Numero de segundos aps o qual a senha expira - 100 dias
      System.setProperty("maca_ca.passwordPolicy.passwordMaxAge", "8640000");
      //Tempo em segundos antes do venvimento da senha do usurio, aps o qual
      //o usurio ser alertado de que sua senha ir expirar - 10 dias
      System.setProperty("maca_ca.passwordPolicy.passwordWarning", "86400");
      //Indica se o histrico das ltimas senhas deve ser mantido
      System.setProperty("maca_ca.passwordPolicy.passwordKeepHistory", "false");
      //Quantidade senhas antigas mantidas no histrico
      System.setProperty("maca_ca.passwordPolicy.passwordInHistory", "6");
      //Indica se o usurio pode mudar a sua senha
      System.setProperty("maca_ca.passwordPolicy.passwordChange", "true");
      //Indica que o usurio deve modificar sua senha na primeira autenticao
      //ou quando a senha  resetada pelo administrador
      System.setProperty("maca_ca.passwordPolicy.passwordMustChange", "false");
      //Interstcio em segundos antes do qual o usurio no poder mudar a senha
      System.setProperty("maca_ca.passwordPolicy.passwordMinAge", "0");

      System.setProperty("maca_ca.ldap.product", "openldap");
      System.setProperty("maca_ca.ldap.server", "127.0.0.1");
      System.setProperty("maca_ca.ldap.ssl", "false");
      System.setProperty("maca_ca.ldap.ssl.console", "true");
      System.setProperty("maca_ca.ldap.ssl.keystore", "maca_ca_cacerts");
      System.setProperty("maca_ca.ldap.ssl.keystorepwd", "");
      System.setProperty("maca_ca.ldap.ssl.factory.socket", "maca_ca.ssl.AutomaticSocketFactory");
      System.setProperty("maca_ca.ldap.rootDN", "dc=root");
      System.setProperty("maca_ca.ldap.groupsRDN", "ou=groups");
      System.setProperty("maca_ca.ldap.peopleRDN", "ou="+peopleName);
      System.setProperty("maca_ca.ldap.roleRDN", "cn="+roleName);
      System.setProperty("maca_ca.ldap.orgChartRDN", "cn="+orgChartName);
      System.setProperty("maca_ca.ldap.resourcesRDN", "ou="+resourcesName);
      System.setProperty("maca_ca.ldap.authorizationsRDN", "ou="+authorizationsName);
      System.setProperty("maca_ca.ldap.groupObjectClass", "incorgroup");
      System.setProperty("maca_ca.ldap.groupMemberAttribute", "member");

      System.setProperty("maca_ca.ldap.searchableFields", "uid, givenname, cn, sn, mail, employeenumber");
      System.setProperty("maca_ca.ldap.returningFields", "uid, givenname, cn, sn, mail, employeenumber");
      System.setProperty("maca_ca.ldap.filterFields", "");

      System.setProperty("maca_ca.maca_cs.url", "http://127.0.0.1");

      loadPropertiesFile(log, "./maca_ca.properties");
    }
}


/*
      //Indica que a conta do usurio ficar bloqueda aps um determinado
      //nmero de tentativas mal sucedidas de autenticaes
      System.setProperty("maca_ca.passwordPolicy.passwordLockout", "");
      //Nmero consecutivo de tentativas mal sucedidas de autenticao
      //antes de bloquear a conta do usurio
      System.setProperty("maca_ca.passwordPolicy.passwordMaxFailure", "");
      //Indica se uma conta bloqueada poder ser desbloqueada aps uma
      //durao em segundos pr-determinada
      System.setProperty("maca_ca.passwordPolicy.passwordUnlock", "");
      //Durao em segundos pr-determinada para desbloqueio de uma conta
      //bloqueada por tentativas mal sucedidas de autenticao
      System.setProperty("maca_ca.passwordPolicy.passwordLockoutDuration", "");
      //Nmero de segundos aps o qual o contador de falhas de autenticao ser
      //resetado
      System.setProperty("maca_ca.passwordPolicy.passwordResetFailureCount", "");
*/

