package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta, Luiz Kobayashi
 * @version 3.2.1
 */

import java.io.*;
import java.text.*;
import java.util.*;

public class LogWriter {
  private PrintWriter              log = null;
  private static DateFormat         df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS z", new Locale("pt", "BR"));
  private static DateFormat        df2 = new SimpleDateFormat("yyyyMMddHHmmss", new Locale("pt", "BR"));
  private File                _logFile = new File("");

  private static java.util.Timer timer = new java.util.Timer(true);
  private String             Frequency = new String();
  private Calendar           aCalendar = Calendar.getInstance();

  public LogWriter(String logFileName) throws java.io.FileNotFoundException {
    this(logFileName, null);
  }

  public LogWriter(String logFileName, java.util.Timer aTimer) throws java.io.FileNotFoundException {
    if (aTimer != null) {
      this.timer = aTimer;
    }
    long maxLen = 0;
    long maxDirSize = 0;
    long maxLenFreq = 0;
    long maxDirSizeFreq = 0;
    this._logFile     = new File(logFileName);
    this.log          = new PrintWriter(new BufferedOutputStream(new FileOutputStream(logFileName, true)), true);
    this.log.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
    this.log.println("<log>");
    Frequency = System.getProperty("maca_ca.log.LogFileCreationFrequency");
    maxLenFreq = Long.parseLong(System.getProperty("maca_ca.log.LogFileMaxLengthFreq"));
    maxDirSizeFreq = Long.parseLong(System.getProperty("maca_ca.log.LogMaxDirSizeFreq"));
    maxLen = Long.parseLong(System.getProperty("maca_ca.log.LogFileMaxLength"));
    maxDirSize = Long.parseLong(System.getProperty("maca_ca.log.LogMaxDirSize"));
    maxLenFreq = maxLenFreq*60000; //converto para milissegundos
    maxDirSizeFreq = maxDirSizeFreq*60000; //converto para milissegundos
    Date aDate = new Date();
    if ((maxDirSizeFreq>0)&&(maxDirSize>0)) {
      try{
          timer.scheduleAtFixedRate(new LogMaxDirSizeTask(), (60-aCalendar.get(aCalendar.SECOND))*1000, maxDirSizeFreq); // hora a hora
      }
      catch (Exception exc){
        exc.printStackTrace();
      }
    }

    if ((maxLenFreq>0)&&(maxLen>0)) {
      try {
        timer.scheduleAtFixedRate(new LogMaxLenTask(), (60-aCalendar.get(aCalendar.SECOND))*1000, maxLenFreq); // hora a hora
      }
      catch (Exception exc)
      {
        exc.printStackTrace();
      }
    }
    /*
    Pode ser:
    I: mInutely
    H: Hourly
    D: Daily
    W: Weekly
    M: Monthly
    Y: Yearly
    */
    if (
        Frequency.equalsIgnoreCase("I") ||
        Frequency.equalsIgnoreCase("H") ||
        Frequency.equalsIgnoreCase("D") ||
        Frequency.equalsIgnoreCase("W") ||
        Frequency.equalsIgnoreCase("M") ||
        Frequency.equalsIgnoreCase("Y")
        ){
      try {
      // verifica qual  a freqencia
        switch (Frequency.toUpperCase().charAt(0)) {
          case 'I':
//            System.out.println("Evento LogFileFrequencyTask - Case I ...");
//            System.out.println("Delay: "+ Integer.toString((60-aCalendar.get(aCalendar.SECOND))*1000)+" miliseconds");
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (60-aCalendar.get(aCalendar.SECOND))*1000 , 60000); // minuto a minuto
            break;
          case 'H':
//            System.out.println("Evento LogFileFrequencyTask - Case H ...");
//            System.out.println("Delay: "+ Integer.toString((60-aCalendar.get(aCalendar.MINUTE))*60000)+" miliseconds");
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (60-aCalendar.get(aCalendar.MINUTE))*60000, 3600000); // hora a hora
            break;
          case 'D':
//            System.out.println("Evento LogFileFrequencyTask - Case D ...");
//            System.out.println("Delay: "+ Integer.toString((24-aCalendar.get(aCalendar.HOUR_OF_DAY))*3600000)+" miliseconds");
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (24-aCalendar.get(aCalendar.HOUR_OF_DAY))*3600000, 86400000); // de 1 em 1 dia
            break;
          case 'W':
//            System.out.println("Evento LogFileFrequencyTask - Case W ...");
//            System.out.println("Delay: "+ Integer.toString((7-aCalendar.get(aCalendar.DAY_OF_WEEK))*86400000)+" miliseconds");
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (7-aCalendar.get(aCalendar.DAY_OF_WEEK))*86400000, 604800000); // de 7 em 7 dias
            break;
          case 'M':
//            System.out.println("Evento LogFileFrequencyTask - Case M ...");
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (24-aCalendar.get(aCalendar.HOUR_OF_DAY))*3600000, 86400000); // de 1 em 1 dia (seno estoura a varivel - mes em mes  impossivel)
            break;
          case 'Y':
//            System.out.println("Evento LogFileFrequencyTask - Case Y ...");
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (24-aCalendar.get(aCalendar.HOUR_OF_DAY))*3600000, 86400000); // de 1 em 1 dia (seno estoura a varivel)
            break;
          default:
            timer.scheduleAtFixedRate(new LogFileFrequencyTask(), (24-aCalendar.get(aCalendar.HOUR_OF_DAY))*3600000, 86400000); // de 1 em 1 dia
        }
      }
      catch (Exception exc) {
        exc.printStackTrace();
      }
    }
  }

  public LogWriter(PrintWriter aLog) {
    this.log = aLog;
  }

  public synchronized void println(java.util.logging.Level level, String logger, String classe, String method, String message, boolean useCdata) {
    if (useCdata)
      this.println(level, logger, classe, method, message);
    else
      this.log.println("<record>"+
                          "<date>"+
                             df.format(new Date())+
                          "</date>"+
                          "<logger><![CDATA["+
                             logger+
                          "]]></logger>"+
                          "<level>"+
                             level.getName()+
                          "</level>"+
                          "<class>"+
                              classe+
                          "</class>"+
                          "<method>"+
                              method+
                          "</method>"+
                          "<message>"+
                              message+
                          "</message>"+
                       "</record>");
  }

  public synchronized void println(java.util.logging.Level level, String logger, String classe, String method, String message) {
    this.log.println("<record>"+
                        "<date>"+
                           df.format(new Date())+
                        "</date>"+
                        "<logger><![CDATA["+
                           logger+
                        "]]></logger>"+
                        "<level>"+
                           level.getName()+
                        "</level>"+
                        "<class>"+
                            classe+
                        "</class>"+
                        "<method>"+
                            method+
                        "</method>"+
                        "<message><![CDATA["+
                            message+
                        "]]></message>"+
                     "</record>");
  }

  public synchronized void close() {
    this.log.println("</log>");
    this.log.close();
  }

  //Mtodos privados
  private synchronized void LogMaxDirSizeTaskImplement() {
    long maxDirSize = 0;
    Long MaxDirSize = new Long(0);
    Long DirSize = new Long(0);
    int numberFiles = 0;
    File oldDir = new File("");

    maxDirSize = Long.parseLong(System.getProperty("maca_ca.log.LogMaxDirSize"));
    maxDirSize = maxDirSize*1024*1024; // tamanho em bytes
    MaxDirSize = new Long(maxDirSize);
    DirSize = new Long(computeDirSize(_logFile));
    oldDir     = _logFile.getParentFile();
    numberFiles = oldDir.listFiles().length;
    if (numberFiles>0) {
      for (int i =0; i<numberFiles;i++){
        if (MaxDirSize.compareTo(DirSize) <= 0){
          deleteOldFiles();
        }
        DirSize = new Long(computeDirSize(_logFile));
      }
    }


    /*
    while (MaxDirSize.compareTo(DirSize) <= 0){
      deleteOldFiles();
      DirSize = new Long(computeDirSize(_logFile));
    }
    */
  }

  private synchronized void LogMaxLenTaskImplement() {
    // verifica o parametro de limite mximo
    Integer FileMaximumSize =  new Integer(Integer.parseInt(System.getProperty("maca_ca.log.LogFileMaxLength")));
    long fileMaximumSize = 0;
    fileMaximumSize = FileMaximumSize.longValue()*1024*1024; //valor em bytes
    // Se o limite mximo for no positivo, ento no tem limite mximo.
    if (fileMaximumSize>0){
      //System.out.println("Entrou no File Maximum Size.longValue ...");
      if (_logFile.length()>=fileMaximumSize){
        //System.out.println("Entrou no LogFile Length >= File Maximum Size ...");
        // se extrapolou o limite mximo, ento tem que renomear o arquivo e criar um arquivo novo.
        try {
          Date aDate = new Date();
          // String NewFileName = _logFile.toString().substring(0,_logFile.toString().length()-3)+"."+df2.format(aDate)+".old";
          String NewFileName = _logFile.toString()+"."+df2.format(aDate)+".old";
          File _newlogFile = new File(NewFileName);
          if (!_newlogFile.exists()) {
            //System.out.println("NewFileName = "+NewFileName);
            this.close();
            this._logFile.renameTo(_newlogFile);
            this.log = new PrintWriter(new BufferedOutputStream(new FileOutputStream(_logFile.toString(), true)), true);
            this._logFile.createNewFile();
            this.log.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
            this.log.println("<log>");
            //System.out.println("Criou arquivo "+_logFile.toString()+" ok");
          }
        }
        catch (Exception exc) {
          exc.printStackTrace();
        }
      }
        //System.out.println("Saiu do LogFile Length >= File Maximum Size ...");
    }
    //System.out.println("Saiu do File Maximum Size.longValue ...");
  }

  private synchronized void LogFileFrequencyTaskImplement(){
    Date aDate = new Date();
    switch (Frequency.toUpperCase().charAt(0)) {
      case 'M': // mes a mes
        aCalendar = Calendar.getInstance();
//        System.out.println("Evento LogFileFrequencyTaskImplement - Case M ...");
//        System.out.println("aCalendar.Month: " + String.valueOf(aCalendar.get(aCalendar.MONTH)));
//        System.out.println("aCalendar.Day of Month: " + String.valueOf(aCalendar.get(aCalendar.DAY_OF_MONTH)));
        if (
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.JANUARY) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.FEBRUARY) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 28)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.MARCH) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.APRIL) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 30)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.MAY) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.JUNE) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 30)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.JULY) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.AUGUST) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.SEPTEMBER) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 30)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.OCTOBER) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.NOVEMBER) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 30)) ||
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.DECEMBER) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31))
        ){
          try {
            // String NewFileName = _logFile.toString().substring(0,_logFile.toString().length()-3)+df2.format(aDate)+".old";
            String NewFileName = _logFile.toString()+"."+df2.format(aDate)+".old";
            File _newlogFile = new File(NewFileName);
            if (!_newlogFile.exists()) {
              //System.out.println("NewFileName = "+NewFileName);
              this.close();
              this._logFile.renameTo(_newlogFile);
              this.log = new PrintWriter(new BufferedOutputStream(new FileOutputStream(_logFile.toString(), true)), true);
              this._logFile.createNewFile();
              this.log.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
              this.log.println("<log>");
              //System.out.println("Criou arquivo "+_logFile.toString()+" ok");
            }
          }
          catch (Exception exc) {
            exc.printStackTrace();
        }
        }
        break;
      case 'Y': // ano a ano
//        System.out.println("Evento LogFileFrequencyTaskImplement - Case Y ...");
        aCalendar = Calendar.getInstance();
        if (
          ((aCalendar.get(aCalendar.MONTH) == aCalendar.DECEMBER) && (aCalendar.get(aCalendar.DAY_OF_MONTH) == 31))
        ){
          try {
            //String NewFileName = _logFile.toString().substring(0,_logFile.toString().length()-3)+df2.format(aDate)+".old";
            String NewFileName = _logFile.toString()+"."+df2.format(aDate)+".old";
            File _newlogFile = new File(NewFileName);
            if (!_newlogFile.exists()) {
              //System.out.println("NewFileName = "+NewFileName);
              this.close();
              this._logFile.renameTo(_newlogFile);
              this.log = new PrintWriter(new BufferedOutputStream(new FileOutputStream(_logFile.toString(), true)), true);
              this._logFile.createNewFile();
              this.log.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
              this.log.println("<log>");
              //System.out.println("Criou arquivo "+_logFile.toString()+" ok");
            }
          }
          catch (Exception exc) {
            exc.printStackTrace();
        }
        }
        break;
      default: // para logs de minuto a minuto, hora a hora, dia a dia e semana a semana
//        System.out.println("Evento LogFileFrequencyTaskImplement - Case Default ...");
        aCalendar = Calendar.getInstance();
        try {
          //String NewFileName = _logFile.toString().substring(0,_logFile.toString().length()-3)+df2.format(aDate)+".old";
          String NewFileName = _logFile.toString()+"."+df2.format(aDate)+".old";
          File _newlogFile = new File(NewFileName);
          if (!_newlogFile.exists()) {
//            System.out.println("NewFileName = "+NewFileName);
            this.close();
            this._logFile.renameTo(_newlogFile);
            this.log = new PrintWriter(new BufferedOutputStream(new FileOutputStream(_logFile.toString(), true)), true);
            this._logFile.createNewFile();
            this.log.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
            this.log.println("<log>");
//            System.out.println("Criou arquivo "+_logFile.toString()+" ok");
          }
        }
        catch (Exception exc) {
          exc.printStackTrace();
        }
    }
    //df.parse();
  }

  private synchronized long computeDirSize(File logFile){
    File oldDir = new File("");
    File oldFile = new File("");
    File[] oldFileList = null;
    long dirSize=0;
    oldDir     = logFile.getParentFile();
    oldFileList = new File[oldDir.listFiles().length];
    try {
      oldFileList = oldDir.listFiles();
      int numberFiles = oldDir.listFiles().length;
      // descubro qual  o tamanho do diretrio
      if (oldDir.listFiles().length>0){
        for (int auxCounter = 0; auxCounter<oldDir.listFiles().length; auxCounter++) {
          if (oldFileList[auxCounter].isFile()){
            // System.out.println("auxCounter:" + auxCounter+" File: "+oldFileList[auxCounter].getCanonicalFile());
            dirSize += oldFileList[auxCounter].length();
            }
          }
      }
//      System.out.println("dirSize = "+String.valueOf(dirSize));
    }
    catch (Exception exc){
      exc.printStackTrace();
    }
    return dirSize;
  }

  private synchronized void deleteOldFiles(){
    String fileName = new String (_logFile.getName());
    File oldDir = new File("");
    File oldFile = new File("");
    File[] oldFileList = null;
    long lastModified = 0; // mximo valor de long
    int deleteCounter = -1;
    //System.out.println("fileName: "+fileName);
    /*
    if (fileName.lastIndexOf('.')>0){
      fileName = fileName.substring(0,fileName.lastIndexOf('.'));
    }
    System.out.println("fileName: "+fileName);
    */
    oldDir     = _logFile.getParentFile();
    FilenameFilter fileFilter = new FileListFilter(fileName,"old");
    oldFileList = new File[oldDir.listFiles(fileFilter).length];
    try {
      oldFileList = oldDir.listFiles(fileFilter);
      int numberFiles = oldDir.listFiles(fileFilter).length;
      // System.out.println("Number of Files: "+(String.valueOf(numberFiles)));
      /*
      for (int gambi = 0; gambi < numberFiles; gambi++){
        System.out.println(oldFileList[gambi].getCanonicalPath());
      }
      */
      // descubro qual  o arquivo mais velho
      if (numberFiles>0){
        deleteCounter = 0;
        lastModified = oldFileList[0].lastModified();
        for (int auxCounter = 0; auxCounter<numberFiles; auxCounter++) {
          //System.out.println("oldFileList[auxCounter].lastModified():" + oldFileList[auxCounter].lastModified() + "\ndeleteCounter:" + deleteCounter+" auxCounter:" + auxCounter+" File: "+oldFileList[auxCounter].getCanonicalFile());
          if (oldFileList[auxCounter].isFile()){
            if (oldFileList[auxCounter].lastModified() < lastModified){
            deleteCounter = auxCounter;
            // System.out.println("deleteCounter: "+String.valueOf(deleteCounter));
            }
          }
        }
      }
      // agora que eu tenho qual  o arquivo mais velho, apago o arquivo mais velho
      if (deleteCounter>=0){
        // System.out.println("File to be Deleted = "+oldFileList[deleteCounter].getCanonicalFile());
        oldFileList[deleteCounter].getCanonicalFile().delete();
      }
    }
    catch (Exception exc){
      exc.printStackTrace();
    }
  }

  class LogMaxLenTask extends java.util.TimerTask {
    public void run() {
      try {
//        System.out.println("Evento LogMaxLenTask inicio...");
        LogMaxLenTaskImplement();
//        System.out.println("Evento LogMaxLenTask fim...");
      }
      catch (Exception exc) {
        exc.printStackTrace();
      }
    }
  }

  class LogFileFrequencyTask extends java.util.TimerTask {
    public void run() {
      try {
//        System.out.println("Evento LogFileFrequencyTask inicio...");
        LogFileFrequencyTaskImplement();
//        System.out.println("Evento LogFileFrequencyTask fim...");
      }
      catch (Exception exc) {
        exc.printStackTrace();
      }
    }
  }

  class LogMaxDirSizeTask extends java.util.TimerTask {
    public void run() {
      try {
//        System.out.println("Evento LogMaxDirSize inicio...");
        LogMaxDirSizeTaskImplement();
//        System.out.println("Evento LogMaxDirSize fim...");
      }
      catch (Exception exc) {
        exc.printStackTrace();
      }
    }
  }
}

   class FileListFilter implements java.io.FilenameFilter {
     private String name;
     private String extension;
     // Construtor
     public FileListFilter(String name, String extension)
     {
       this.name = name;
       this.extension = extension;
     }

     public boolean accept(File directory,String filename){
       boolean fileOK = true;

       // if there is a name filter specified, check the filename
       if (name != null)
         fileOK = filename.startsWith(name);

       // if there is an extension filter, check the file extension
       if (extension != null)
         fileOK &= filename.endsWith('.' + extension);
       return fileOK;
     }
   }
/*
      Curiosidade:
      Maximum Sizes on NTFS Volumes
      In theory, the maximum NTFS volume size is 2^64 clusters minus 1 cluster.
      However, the maximum NTFS volume size as implemented in Windows XP Professional is 2^32 clusters minus 1 cluster.
      For example, using 64-KB clusters, the maximum NTFS volume size is 256 terabytes minus 64 KB.
      Using the default cluster size of 4 KB, the maximum NTFS volume size is 16 terabytes minus 4 KB.
      Because partition tables on master boot record (MBR) disks only support partition sizes up to 2 terabytes, you must use dynamic volumes to create NTFS volumes over 2 terabytes.
      Windows XP Professional manages dynamic volumes in a special database instead of in the partition table, so dynamic volumes are not subject to the 2-terabyte physical limit imposed by the partition table.
      Therefore, dynamic NTFS volumes can be as large as the maximum volume size supported by NTFS.
      Itanium-based computers that use GUID partition table (GPT) disks also support NTFS volumes larger than 2 terabytes.

      Note

      If you use large numbers of files in an NTFS folder (300,000 or more), disable short-file name generation, especially if the first six characters of the long file names are similar.
      For more information, see "Optimizing NTFS Performance" later in this chapter.
      Maximum file size  Theory: 16 exabytes minus 1 KB (2^64 bytes minus 1 KB)
      Implementation: 16 terabytes minus 64 KB (2^44 bytes minus 64 KB)
      Maximum volume size Theory: 264 clusters minus 1 cluster
      Implementation: 256 terabytes minus 64 KB ( 2^32 clusters minus 1 cluster)
      Files per volume 4,294,967,295 (232 minus 1 file)

      Maximum Sizes on FAT32 Volumes
      A FAT32 volume must have a minimum of 65,527 clusters.
      Windows XP Professional can format FAT32 volumes up to 32 GB, but it can mount larger FAT32 volumes created by other operating systems.
      Maximum file size  4 GB minus 1 byte (2^32 bytes minus 1 byte)
      Maximum volume size 32 GB (implementation)
      Files per volume 4,177,920
      Maximum number of files and subfolders within a single folder 65,534 (The use of long file names can significantly reduce the number of available files and subfolders within a folder.)

      Maximum Sizes on FAT16 Volumes
      FAT16 supports a maximum of 65,524 clusters per volume.
      Maximum file size  4 GB minus 1 byte (2^32 bytes minus 1 byte)
      Maximum volume size 4 GB
      Files per volume Approximately 65,536 (2^16 files)
      Maximum number of files and folders within the root folder 512 (Long file names can reduce the number of available files and folders in the root folder.)

      Dados retirados de
      http://www.microsoft.com/technet/treeview/default.asp?url=/TechNet/prodtechnol/winxppro/reskit/prkc_fil_tdrn.asp
      */



