package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.util.*;
import java.lang.*;
import javax.naming.*;
import javax.swing.tree.*;
import javax.naming.directory.*;
import ContextExpressionLanguage.analysis.*;
import ContextExpressionLanguage.lexer.*;
import ContextExpressionLanguage.parser.*;
import ContextExpressionLanguage.node.*;

public class Role extends Group {

  //Variveis privadas
  private String _roleAuthoRoot = "";

  //Variveis protegidas

  private synchronized  Enumeration _getAuthorizations(String filter) throws NamingException {
    Vector     authos   = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
//    ctls.setReturningAttributes(new String[] {});


    Enumeration answer = ctx.value.search(this.getRoleAuthorizationRoot(), filter, ctls);
    while (answer.hasMoreElements()) {
      SearchResult sr  = (SearchResult) answer.nextElement();
      Authorization autho = new Authorization(ctx,
                                              this.getRoleAuthorizationRoot(),
                                              sr.getName(),
                                              sr.getAttributes());
      authos.add(autho);
    }
    return new ObjectEnumeration(authos);
  }

  private Hashtable _getParentResources(String filter) throws NamingException {
    Hashtable       res = new Hashtable();
    String parentName = this.getParentRDN();
    while (!parentName.equals("")) {
      Role role = new Role(ctx, _groupRoot, parentName);
      res.putAll(role._getResources(filter));
      parentName = role.getParentRDN();
    }
    return res;
  }


  private synchronized  Hashtable _getResources(String filter) throws NamingException {
    Hashtable       res = new Hashtable();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
//    ctls.setReturningAttributes(new String[] {});
    Enumeration answer = ctx.value.search(this.getRoleAuthorizationRoot(), filter, ctls);
    while (answer.hasMoreElements()) {
      SearchResult sr  = (SearchResult) answer.nextElement();
      Authorization autho = new Authorization(ctx,
                                              this.getRoleAuthorizationRoot(),
                                              sr.getName(),
                                              sr.getAttributes());
      res.put(autho.getPrivilege()+autho.getResourceName(), autho);
    }
    return res;
  }

  private void createRoleAuthorizationsRoot() throws NamingException {
     Attributes attrs = new BasicAttributes(true);
     Attribute oc = new BasicAttribute("objectclass");
     oc.add("top");
     oc.add("organizationalunit");
     attrs.put(oc);
     attrs.put("ou",this.getRoleAuthorizationRootRDN());
     System.out.println("Criando roleautho: "+this.getRoleAuthorizationRoot());
     ctx.value.createSubcontext(this.getRoleAuthorizationRoot(), attrs);
  }

  public Group newGroup(DirContextHolder aContext, String groupRoot, String groupRDN) throws NamingException {
    return new Role(aContext, groupRoot, groupRDN);
  }

  private String getRoleAuthorizationRootRDN() {
    StringTokenizer strTk = new StringTokenizer(this._groupRDN, ",");
    String roleName = "";
    while (strTk.hasMoreTokens()) {
      String str = strTk.nextToken();
      roleName = roleName + (roleName.equalsIgnoreCase("")?"":"|") + str.substring(str.indexOf("=")+1);
    }
    return roleName;
  }

  public String getRoleAuthorizationRoot() throws NamingException {
    if (this._roleAuthoRoot.equalsIgnoreCase("")) {
      String roleName = this.getRoleAuthorizationRootRDN();
      _roleAuthoRoot = "ou="+roleName+","+System.getProperty("maca_ca.ldap.authorizationsRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
      if (!this.newGroup) {
        try {
          ctx.value.getAttributes(_roleAuthoRoot, new String[]{"objectclass"});
        }
        catch (NameNotFoundException exc) {
          this.createRoleAuthorizationsRoot();
        }
      }
    }
    return this._roleAuthoRoot;
  }

  public Role(DirContextHolder aContext, String groupRoot, String groupRDN) throws NamingException {
    super(aContext, groupRoot, groupRDN);
  }

  public static Vector getRolesWithNames(DirContextHolder aContext, Vector rolesNames) throws NamingException {
    Vector roles = new Vector();
    for (int i = 0; i < rolesNames.size(); i++) {
      roles.add(new Role(aContext, System.getProperty("maca_ca.ldap.roleRDN")+","+
                                   System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                   System.getProperty("maca_ca.ldap.rootDN"),
                                   (String)rolesNames.get(i)));
    }
    return roles;
  }

  public void renameGroupName(String newGroupName) throws NamingException {
    super.renameGroupName(newGroupName);
    if ((this.ctx != null) && (!this.newGroup)){
      this.ctx.value.destroySubcontext(this.getRoleAuthorizationRoot());
      this._roleAuthoRoot = this.getRoleAuthorizationRoot();
      this.createRoleAuthorizationsRoot();
    }
  }
  /**
   *  Verifica se este papel
   *  j tem uma autorizao equivalente (com
   *  mesmo nome e recurso) diretamente definida.
   */
  public boolean hasEquivalentAuthorization(Authorization autho) throws NamingException {
    boolean result = false;
    if (autho == null)
      return result;
    Enumeration enum = this.getAuthorizations(autho.getPrivilege(), autho.ldapResourceDNToResourceName(false));
    while (enum.hasMoreElements()) {
      Authorization roleAutho = (Authorization) enum.nextElement();
      if (!roleAutho.getAuthorizationDN().equalsIgnoreCase(autho.getAuthorizationDN())) {
        result = true;
      }
    }
    return result;
  }



  public void setRoleAuthorizationStatus(Vector aParentAuthos, Vector aThisAuthos, String resFilter) throws NamingException {
    aParentAuthos.clear();
    aThisAuthos.clear();

    Hashtable parentAuthos = new Hashtable(500),
              thisAuthos   = new Hashtable(100);
    Vector parents         = this.getParentList();
    parents.add(0, this);
    for (int n = (parents.size() - 1); n >= 0; n--) {
      Role parentRole = (Role) parents.get(n);
      Enumeration enum = parentRole.getAuthorizations(true, resFilter);
      while ((enum != null) && enum.hasMoreElements()) {
        Authorization autho = (Authorization) enum.nextElement();
        autho.setStatus(Authorization.ACTIVE);
        if (!parentAuthos.containsKey(autho.getRoleDN() + autho.getResourceName() + autho.getPrivilege())) {
          parentAuthos.put(autho.getRoleDN() + autho.getResourceName() + autho.getPrivilege(), autho);
        }
        for (int k = (n - 1); k >= 0; k--) {
          Role childRole = (Role) parents.get(k);
          Enumeration childEnum = childRole.getAuthorizations(autho.getPrivilege(),
                                                              autho.ldapResourceDNToResourceName(false));
          while ((childEnum != null) && childEnum.hasMoreElements()) {
            Authorization childAutho = (Authorization) childEnum.nextElement();
            childAutho.setStatus(Authorization.INACTIVE);
            if (!parentAuthos.containsKey(childAutho.getRoleDN() + childAutho.getResourceName() + childAutho.getPrivilege())) {
              parentAuthos.put(childAutho.getRoleDN() + childAutho.getResourceName() + childAutho.getPrivilege(), childAutho);
            }
          }
        }
      }
    }
    for (int n = 0; n < parents.size(); n++) {
      Role parentRole = (Role) parents.get(n);
      Enumeration enum = parentRole.getAuthorizations(resFilter);
      while ((enum != null) && enum.hasMoreElements()) {
        Authorization autho = (Authorization) enum.nextElement();
        if (!parentAuthos.containsKey(autho.getRoleDN() + autho.getResourceName() + autho.getPrivilege())) {
          autho.setStatus(Authorization.ACTIVE);
          parentAuthos.put(autho.getRoleDN() + autho.getResourceName() + autho.getPrivilege(), autho);
        }
        for (int k = (n + 1); k < parents.size(); k++) {
          Role childRole = (Role) parents.get(k);
          Enumeration childEnum = childRole.getAuthorizations(false, autho.getPrivilege(),
                                                              autho.ldapResourceDNToResourceName(false));
          while ((childEnum != null) && childEnum.hasMoreElements()) {
            Authorization childAutho = (Authorization) childEnum.nextElement();
            childAutho.setStatus(Authorization.INACTIVE);
            if (!parentAuthos.containsKey(childAutho.getRoleDN() + childAutho.getResourceName() + childAutho.getPrivilege())) {
              parentAuthos.put(childAutho.getRoleDN() + childAutho.getResourceName() + childAutho.getPrivilege(), childAutho);
            }
          }
        }
      }
    }
    String thisRoleDN  = this.getGroupDN();
    Enumeration values = parentAuthos.elements();
    while (values.hasMoreElements()) {
      Authorization autho = (Authorization) values.nextElement();
      if (autho.getRoleDN().equalsIgnoreCase(thisRoleDN)) {
        aThisAuthos.add(autho);
      }
      else {
        aParentAuthos.add(autho);
      }
    }
  }

  protected Enumeration getAuthorizations(boolean strong, boolean positive, String resName) throws NamingException {
    return this._getAuthorizations("(&(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(incorauthorizationtype="+(strong?"strong":"weak")+"))(incorprivilegetype="+(positive?"+":"-")+"))");
  }

  protected Enumeration getAuthorizations(boolean strong) throws NamingException {
    return this._getAuthorizations("(incorauthorizationtype="+(strong?"strong":"weak")+")");
  }

  protected Enumeration getAuthorizations(boolean strong, String privilege, String resName, boolean dynamic) throws NamingException {
    return this._getAuthorizations("(&(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(&(incorprivilege="+privilege+")(incorauthorizationtype="+(strong?"strong":"weak")+")))"+(dynamic?"(!(|(incorprivilegetype=+)(incorprivilegetype=-)))":"")+")");
  }

  protected Enumeration getAuthorizations(boolean strong, boolean positive, String privilege, String resName) throws NamingException {
    return this._getAuthorizations("(&(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(&(incorprivilege="+privilege+")(incorauthorizationtype="+(strong?"strong":"weak")+")))(incorprivilegetype="+(positive?"+":"-")+"))");
  }

  protected Enumeration getAuthorizations(boolean strong, String resName) throws NamingException {
    return this._getAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(incorauthorizationtype="+(strong?"strong":"weak")+"))");
  }

  protected Enumeration getAuthorizations(String privilege, String resName) throws NamingException {
    return this._getAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")((incorprivilege="+privilege+")))");
  }

  protected Enumeration getAuthorizations(boolean strong, String privilege, String resName) throws NamingException {
    return this._getAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(&(incorprivilege="+privilege+")(incorauthorizationtype="+(strong?"strong":"weak")+")))");
  }

  protected Enumeration getAuthorizations(String resName, boolean positive) throws NamingException {
    return this._getAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(incorprivilegetype="+(positive?"+":"-")+"))");
  }

  protected Enumeration getAuthorizations(String resName) throws NamingException {
    return this._getAuthorizations("(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")");
  }

  protected Enumeration getAuthorizations() throws NamingException {
    return this._getAuthorizations("(cn=*)");
  }

  public Hashtable getResources() throws NamingException {
    return this._getResources("(cn=*)");
  }

  public Hashtable getResources(String resName, String privilege) throws NamingException {
    return this._getResources("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(incorprivilege="+privilege+"))");
  }

  public Hashtable getParentResources() throws NamingException {
    return this._getParentResources("(cn=*)");
  }

  public Hashtable getParentResources(String resName, String privilege) throws NamingException {
    return this._getParentResources("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(incorprivilege="+privilege+"))");
  }

  public Vector getValidAuthorizations(String resName) throws Exception {
    Hashtable resAndPriv = new Hashtable();
    Vector validAuthos = new Vector();
    Role role = this;
    while (role != null) {
      Enumeration enum = role.getAuthorizations(resName);
      while (enum.hasMoreElements()) {
        Authorization autho = (Authorization) enum.nextElement();
        if (autho.isValid()) {
          String resAndPrivKey = (autho.getResourceName()+autho.getPrivilege()).toLowerCase().trim();
          if (autho.isWeak()) {
            if (!resAndPriv.containsKey(resAndPrivKey)) {
              validAuthos.add(autho);
              resAndPriv.put(resAndPrivKey,"");
            }
          }
          else {
            validAuthos.add(autho);
            resAndPriv.put(resAndPrivKey,"");
          }
        }
      }
      role = (Role) role.getParent();
    }
    return validAuthos;
  }

  public Vector getConflictingAuthorizations(boolean strong, String resName) throws Exception {
    Vector conflictingAuthos = new Vector();
    Vector validAuthos = this.getValidAuthorizations(resName);
    for (int i = 0; i < validAuthos.size(); i++) {
      Authorization autho = (Authorization) validAuthos.get(i);
      Vector     cfAuthos = autho.getConflictingAuthorizations();
      for (int n = 0; n < cfAuthos.size(); n++) {
        Authorization cfAutho = (Authorization) cfAuthos.get(n);
        if (cfAutho.isStrong() == strong) {
          conflictingAuthos.add(cfAutho);
        }
      }
    }
    return conflictingAuthos;
  }

  public String getGroupType() {
    return System.getProperty("maca_ca.roleName");
  }

  protected Enumeration getParentAuthorizations(boolean strong, String resName) throws NamingException {
     return this._getParentAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")"+
                                            "(incorauthorizationtype="+(strong?"strong":"weak")+"))");
  }

  protected Enumeration getParentAuthorizations(boolean strong, boolean positive, String resName) throws NamingException {
     return this._getParentAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")"+
                                            "(&(incorprivilegetype="+(positive?"+":"-")+")(incorauthorizationtype="+(strong?"strong":"weak")+")))");
  }

  protected Enumeration getParentAuthorizations(boolean strong, boolean positive, String resName, String privilege) throws NamingException {
     return this._getParentAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")"+
                                            "(&(incorprivilegetype="+(positive?"+":"-")+")"+
                                              "(&(incorprivilege="+privilege+")(incorauthorizationtype="+(strong?"strong":"weak")+"))))");
  }

  protected Enumeration getParentAuthorizations(boolean strong, String resName, String privilege) throws NamingException {
     return this._getParentAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")"+
                                            "((&(incorprivilege="+privilege+")(incorauthorizationtype="+(strong?"strong":"weak")+"))))");
  }

  protected Enumeration getParentAuthorizations() throws NamingException {
    return this._getParentAuthorizations("cn=*");
  }

  protected Enumeration getParentAuthorizations(String resName, boolean positive) throws NamingException {
    return this._getParentAuthorizations("(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")(incorprivilegetype="+(positive?"+":"-")+"))");
  }

  protected Enumeration getParentAuthorizations(String resName) throws NamingException {
    return this._getParentAuthorizations("(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")");
  }

  private Enumeration _getParentAuthorizations(String filter) throws NamingException {
    Vector authos     = new Vector();
    String parentName = this.getParentRDN();
    while (!parentName.equals("")) {
      Role role = new Role(ctx, _groupRoot, parentName);
      Enumeration authoEnum = role._getAuthorizations(filter);
      while (authoEnum.hasMoreElements()) {
        authos.add(authoEnum.nextElement());
      }
      parentName = role.getParentRDN();
    }
    return new ObjectEnumeration(authos);
  }

  public boolean addMemberToGroup(String aUserDN) throws NamingException {
    if (this.getGroupRDN().equalsIgnoreCase(System.getProperty("maca_ca.ldap.roleRDN"))) {
      throw new NamingException("Proibida a atribuio do papel '"+System.getProperty("maca_ca.roleName")+"'");
    }
    return super.addMemberToGroup(aUserDN);
  };

  public TreeNode getParent() {
    TreeNode tn = null;
    try {
      if (getParentRDN() == "") {
        tn = null;
      }
      else {
        tn = new Role(ctx, _groupRoot, getParentRDN());
      }
    }
    catch (NamingException exc) {
      exc.printStackTrace();
    }
    return tn;
  }

  public Enumeration children() {
    NamingEnumeration answer = null;
    ObjectEnumeration  groups = null;
    Vector        groupsList = new Vector();

    SearchControls ctls = new SearchControls();

    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    try {
      answer = ctx.value.search(this.getGroupDN(), "(cn=*)", ctls);
      while (answer.hasMore()) {
        SearchResult sr = (SearchResult) answer.next();
        if (_groupRDN == "") {
          groupsList.add(new Role(ctx, _groupRoot, sr.getName()));
        }
        else {
          groupsList.add(new Role(ctx, _groupRoot, sr.getName()+","+_groupRDN));
        }
      }
    }
    catch (NamingException exc) {
      System.out.println(exc);
    }
    groups = new ObjectEnumeration(groupsList);
    return groups;
  }
}