package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.IOException;
import java.io.PushbackReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.security.MessageDigest;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.swing.table.TableModel;

import ContextExpressionLanguage.lexer.Lexer;
import ContextExpressionLanguage.node.Start;
import ContextExpressionLanguage.parser.Parser;

public class User {
  //Constantes
  static protected final String         LINE_END = System.getProperty("line.separator");
  static protected final SimpleDateFormat UTC_DF = new SimpleDateFormat("yyyyMMddHHmmss", new Locale("br", "BR"));
  static public    final PasswordFactory PWD_FAC = getPasswordFactory();

  // Variveis de instncia privadas
  private DirContextHolder ctx = null;
  private Attributes attrs;
  private String _userRoot;
  private String _userRDN;
  private boolean newUser = false;
  private LogWriter _log = null;
  private Hashtable activableRolesHash = null;
  private Vector    activableRolesVect = null;
  private Vector    activeRoles = new Vector();
  private contexts.ContextExpressionInterpreter inter = null;

  // Classes privadas estticas
  private static class AuthorizationResult {
    private boolean _hasResult = false;
    private boolean    _result = false;
    private Role         _role = null;

    public AuthorizationResult() {
    }

    public AuthorizationResult(boolean result, Role role) {
      _hasResult = true;
      _result    = result;
      _role      = role;
    }

    public boolean isTrue() {
      return _result;
    }

    public boolean isFalse() {
      return !_result;
    }

    public boolean isUndecided() {
      return !_hasResult;
    }

    public boolean result() {
      return _result;
    }

    public Role getRole() {
      return _role;
    }
  }



  // Mtodos privados estticos
  private static PasswordFactory getPasswordFactory() {
    String passwordFactoryStr = System.getProperty("maca_ca.passwordPolicy.passwordFactory");
    try {
      PasswordFactory fac = (PasswordFactory) Class.forName(passwordFactoryStr).newInstance();
      System.out.println("PasswordFactory ["+passwordFactoryStr+"] carregada.");
      return fac;
    }
    catch (Exception exc) {
      System.out.println("Falha carregando PasswordFactory["+passwordFactoryStr+"]: "+exc.toString());
      PasswordFactory fac = new maca_ca.SimplePasswordFactory();
      System.out.println("PasswordFactory default ["+passwordFactoryStr+"] carregada.");
      return fac;
    }
  }

  /**
   * Seleciona como disponvel para ativao apenas os papis mais
   * seniores na hierarquia de papis. Deve ser chamado na construo
   * do objeto da classe 'User'. Inicializa a varivel 'activableRoles'.
   */
  private void ActivableRolesSetup() throws NamingException {
    class GroupComparator implements Comparator {
      public int compare(java.lang.Object obj1, java.lang.Object obj2) {
         Group gp1 = (Group) obj1;
         Group gp2 = (Group) obj2;
         if (gp1.getLevel() < gp2.getLevel()) {
           return 1;
         }
         else if (gp1.getLevel() > gp2.getLevel()) {
           return -1;
         }
         return 0;
      }
    }
    Hashtable seniorRoles = new Hashtable();
    Vector roles = Role.getRolesWithNames(ctx, this.getRolesRDN());
    Collections.sort(roles, new GroupComparator());
    while (roles.size() > 0) {          //Seleciona apenas os papis seniores
      Role r = (Role) roles.remove(0);
      seniorRoles.put(r.getName().toLowerCase(), r);
      StringTokenizer strTk = new StringTokenizer(r._groupRDN, ",");
      strTk.nextToken();
      while (strTk.hasMoreTokens()) {
        String roleName = strTk.nextToken().toLowerCase();
        roleName = roleName.substring(3, roleName.length());
        roleRemoved:
        for (int n = 0; n < roles.size(); n++) {
          if (((Role)roles.get(n)).getName().equalsIgnoreCase(roleName)) {
            roles.remove(n);
            if (seniorRoles.containsKey(roleName)) {
              seniorRoles.remove(roleName);
            }
            break roleRemoved;
          }
        }
      }
    }
    this.activableRolesHash = seniorRoles;
    this.activableRolesVect = new Vector(seniorRoles.values());

  }

  private AuthorizationResult activableRolesHaveAuthorization(
      boolean strong, boolean positive,
      String priv, String resName) throws NamingException {
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"incorroledn"});
    Enumeration answer = ctx.value.search(System.getProperty("maca_ca.ldap.authorizationsRDN")+","+
                                          System.getProperty("maca_ca.ldap.rootDN"),
                                          "(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")"+
                                            "(&(incorprivilege="+priv+")"+
                                              "(&(incorauthorizationtype="+(strong?"strong":"weak")+")"+
                                                "(incorprivilegetype="+(positive?"+":"-")+"))))",
                                          ctls);
    Hashtable rolesdn = new Hashtable();
    while (answer.hasMoreElements()) {
      SearchResult sr = (SearchResult) answer.nextElement();
      String roledn = ((String) (sr).getAttributes().get("incorroledn").get()).trim().toLowerCase();
      rolesdn.put(roledn, sr);
    }
    if (!rolesdn.isEmpty()) { //Existem autorizaoes fortes para o recurso, no se sabe se para os papis
                              //atual ou seus acendentes, cuja verificao  feita dentro do if
      for (int i = 0; i < this.activableRolesVect.size(); i++) {
        Role role = (Role) this.activableRolesVect.get(i);
        while (role != null) {
          SearchResult sr = (SearchResult) rolesdn.get(role.getGroupDN().trim().toLowerCase());
          if (sr != null) {
            return new AuthorizationResult(positive, (Role) this.activableRolesVect.get(i));
          }
          role = (Role) role.getParent();
        }
      }
    }
    return new AuthorizationResult();
  }

  private AuthorizationResult activableRolesHaveAuthorization(
      boolean strong, String priv, String resName, String params,
      contexts.ContextExpressionInterpreter inter) throws Exception {
    AuthorizationResult  secondaryResult = new AuthorizationResult();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"incorroledn", "incorprivilegetype"});
    Enumeration answer = ctx.value.search(System.getProperty("maca_ca.ldap.authorizationsRDN")+","+
                                                             System.getProperty("maca_ca.ldap.rootDN"),
                                                             "(&(incorresourcedn="+this.resourceNameToLDAPresourceDN(resName)+")"+
                                                               "(&(incorprivilege="+priv+")"+
                                                                 "(&(incorauthorizationtype="+(strong?"strong":"weak")+")"+
                                                                   "(&(!(incorprivilegetype=+))(!(incorprivilegetype=-))))))",
                                                             ctls);
    Hashtable rolesdn = new Hashtable();
    while (answer.hasMoreElements()) {
      SearchResult sr = (SearchResult) answer.nextElement();
      String roledn = ((String) (sr).getAttributes().get("incorroledn").get()).trim().toLowerCase();
      String   rule = ((String) (sr).getAttributes().get("incorprivilegetype").get());
      rolesdn.put(roledn, rule);
    }
    if (!rolesdn.isEmpty()) { //Existem autorizaoes fortes com regras para o recurso, no se
                              //sabe se para o papel atual ou seus acendentes, cuja
                              //verificao  feita dentro do if
      for (int i = 0; i < this.activableRolesVect.size(); i++) {
        Role role = (Role) this.activableRolesVect.get(i);
        while (role != null) {
          String rule = (String) rolesdn.get(role.getGroupDN().trim().toLowerCase());
          if ((rule != null) && (inter != null)) {
            Parser p = new Parser(new Lexer(new PushbackReader(new StringReader(rule + params))));
            Start tree = p.parse();
            tree.apply(inter);
            if (inter.getError()) {
              String errorMessage = inter.errorMessage();
              inter.resetError();
              throw new Exception(errorMessage);
            }
            AuthorizationResult prioritaryresult =
                new AuthorizationResult(((inter.getResultValue() instanceof Boolean)?
                                        ((Boolean)inter.getResultValue()).booleanValue():
                                          false),
                                        (Role) this.activableRolesVect.get(i));
            inter.resetError();
            if (!prioritaryresult.isUndecided()) {
              if (strong) { // quando forte, retorna na primeira autorizao negativa
                if (prioritaryresult.isFalse()) {
                  return prioritaryresult;
                }
                else {
                  secondaryResult = prioritaryresult;
                }
              }
              else { // quando fraca, retorna na primeira autorizao positiva
                if (prioritaryresult.isTrue()) {
                  return prioritaryresult;
                }
                else {
                  secondaryResult = prioritaryresult;
                }
              }
            }
          }
          role = (Role) role.getParent();
        }
      }
    }
    return secondaryResult;
  }

  private void newUserSetup(String uidStr) throws  java.security.NoSuchAlgorithmException,
                                                   NamingException {
    attrs = new BasicAttributes(true);
    attrs.put(UserSchema.getDefaultObjectClassAttribute());
    String[] attrNames = UserSchema.getAttributesNames();
    for (int i = 0; i < attrNames.length; i++) {
      Attributes attrSchema = UserSchema.getAttributeSchema(attrNames[i]);
      if ((attrSchema != null) && (!"objectclass".equalsIgnoreCase(attrNames[i]))) {
        Attribute newAttr = new BasicAttribute((String) attrNames[i]);
        Attribute    attr = attrSchema.get("default");
        if (attr != null) {
          try {
            javax.naming.NamingEnumeration enum = attr.getAll();
            while (enum.hasMoreElements()) {
              Object value = enum.nextElement();
              if (value != null) {
                newAttr.add(value);
              }
            }
          }
          catch (javax.naming.NamingException exc) {}
        }
        else {
          newAttr.add("");
        }
        attrs.put(newAttr);
      }
    }
    attrs.put(new BasicAttribute("uid",uidStr.substring(uidStr.indexOf("=")+1,uidStr.length())));
  }

  private boolean isUser(String uidStr) throws NamingException {
    NamingEnumeration answer = null;
    SearchControls      ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    ctls.setReturningAttributes(new String[] {""});
    answer = ctx.value.search(this._userRoot, "("+uidStr+")", ctls);
    return answer.hasMore();
  }

  private boolean newPasswordIsSimilarToPersonelData(String newPassword) throws Exception {
    newPassword = newPassword.toLowerCase();
    Enumeration attrs = this.getUserAtributes().getAll();
    while (attrs.hasMoreElements()) {
      Attribute attr = (Attribute) attrs.nextElement();
      if ((attr != null) && !"objectclass".equalsIgnoreCase(attr.getID()) &&
                            !"userpassword".equalsIgnoreCase(attr.getID())) {
        Enumeration enum = attr.getAll();
        while (enum.hasMoreElements()) {
          Object val = enum.nextElement();
          if ((val != null) && (val instanceof String)) {
            String str = ((String) val).toLowerCase();
            if (str.indexOf(newPassword) > -1) {
              return true;
            }
          }
        }
      }
    }
    return false;
  }

  private void putPasswordInHistory(Attribute pwdAttr) throws NamingException {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordKeepHistory"))) {
      if ((pwdAttr != null) &&
          (pwdAttr.get() != null) &&
          (pwdAttr.getID().equalsIgnoreCase("userpassword"))) {
        Attributes attrs = ctx.value.getAttributes(this.getUserDN(), new String[] {"passwordhistory"});
        Attribute   attr = attrs.get("passwordhistory");
        if (attr == null) {
          attrs = new BasicAttributes(true);
          attrs.put("passwordhistory", pwdAttr.get());
          this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.ADD_ATTRIBUTE, attrs);
        }
        else {
          int max = 6; //mximo das 6 ltimas senhas no histrico - default
          try {
            max = Integer.parseInt(System.getProperty("maca_ca.passwordPolicy.passwordInHistory"));
            max = max <= 0 ? 1 : max;
          }
          catch (NumberFormatException exc) {
          }
          while ((attr.size() >= max) && (max >= 0)) {
            attr.remove(attr.size() - 1);
          }
          attr.add(pwdAttr.get());
          this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.REPLACE_ATTRIBUTE, attrs);
        }
      }
    }
  }

  private boolean isPasswordInHistory(String clearPwd) throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordKeepHistory"))) {
      Attributes attrs = ctx.value.getAttributes(this.getUserDN(), new String[] {"passwordhistory"});
      Attribute   attr = attrs.get("passwordhistory");
      if ((attr != null) && (attr.contains(this.getPasswordHashing(clearPwd)))) {
        throw new PasswordErrorException("No  permitido repetir as ltimas "+
                                         System.getProperty("maca_ca.passwordPolicy.passwordInHistory")
                                         +" senhas utilizadas");
      }
    }
    return false;
  }

  private boolean isAlowedToChangePassword() throws Exception {
    String ldapLogin = (String) ctx.value.getEnvironment().get(Context.SECURITY_PRINCIPAL);
  	if (this.isAccountLocked() && ldapLogin.equalsIgnoreCase(this.getUserDN())){
        throw new PasswordErrorException("Proibida a troca de senha: a conta do usurio est bloqueada");
  	}
    if (!"true".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordChange"))) {
      throw new PasswordErrorException("Proibida a troca de senha pelo usurio");
    }
    if (!"0".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordMinAge"))) {
     Date allowedChangeDate = this.getPasswordAttributeDate("passwordAllowChangeTime");
     if ((allowedChangeDate != null) && (allowedChangeDate.after(new Date()))) {
       DateFormat df = new SimpleDateFormat("dd/MM/yyyy 's' HH:mm:ss", new Locale("pt", "BR"));
       throw new PasswordErrorException("Senha s poder ser trocada a partir de "+df.format(allowedChangeDate));
     }
    }
    return true;
  }

  private boolean newPasswordIsValid(String newPassword) throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordCheckSyntax"))) {
      if ("".equalsIgnoreCase(newPassword)) {
        throw new PasswordErrorException("Senha no pode ser vazia");
      }
      final int minLenDefault = 6;
      int minLen = minLenDefault;
      try {
        minLen = Integer.parseInt(System.getProperty("maca_ca.passwordPolicy.passwordMinLength"));
        minLen = minLen < 0 ? minLenDefault : minLen;
      }
      catch (NumberFormatException exc) {
      }
      if (newPassword.length() < minLen) {
        throw new PasswordErrorException("A senha deve conter no mnimo "+minLen+" caracteres");
      }
      if (this.newPasswordIsSimilarToPersonelData(newPassword)) {
        throw new PasswordErrorException("Senha no pode ser similar ou igual a dados pessoais do usurio");
      }
    }
    return true;
  }

  public Date getPasswordAttributeDate(String dateAttrName) throws Exception {
    String  attr = this.getOperacionalAttribute(dateAttrName);
    Date date = null;
    if (!"".equalsIgnoreCase(attr)) {
      if (attr.endsWith("Z")) {
        attr = attr.substring(0, attr.length() - 1);
        date = UTC_DF.parse(attr);
        if (date != null) {
          date = new Date(date.getTime() + UTC_DF.getTimeZone().getRawOffset());
        }
      }
      else {
        date = UTC_DF.parse(attr);
      }
    }
    return date;
  }

  public synchronized Vector getRolesRDN() throws NamingException {
    NamingEnumeration      answer = null;
    Vector         rolesNamesList = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    answer = ctx.value.search(System.getProperty("maca_ca.ldap.roleRDN")+","+
                                                 System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                                 System.getProperty("maca_ca.ldap.rootDN"),
                                                 "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+this.getUserDN()+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      rolesNamesList.add(sr.getName());
    }
    return rolesNamesList;
  }

  private Vector getOrgUnitsRDN() throws NamingException {
    NamingEnumeration      answer = null;
    Vector      orgUnitsNamesList = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    answer = ctx.value.search(System.getProperty("maca_ca.ldap.orgChartRDN")+","+
                                                 System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                                 System.getProperty("maca_ca.ldap.rootDN"),
                                                 "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+this.getUserDN()+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      orgUnitsNamesList.add(sr.getName());
    }
    return orgUnitsNamesList;
  }

  public DirContextHolder getDirContextHolder() {
    return ctx;
  }

  private Vector getSameEmployeeNumberList() throws NamingException {
    Attribute attr = this.attrs.get("employeenumber");
    String employeenumber = "";
    if ((attr != null) && (attr.get() != null)) {
      employeenumber = (String) attr.get();
    }
    NamingEnumeration      answer = null;
    Vector         sameEmployeeNumberList = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"uid"});
    answer = ctx.value.search(System.getProperty("maca_ca.ldap.peopleRDN")+","+System.getProperty("maca_ca.ldap.rootDN"), "(&(employeenumber="+employeenumber+")(!(uid="+this.getUserID()+")))", ctls);
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      attr  = sr.getAttributes().get("uid");
      if ((attr != null) && (attr.get() != null)) sameEmployeeNumberList.add(attr.get());
    }
    return sameEmployeeNumberList;
  }

  private String resourceNameToLDAPresourceDN(String resName) {
    StringTokenizer strTk = new StringTokenizer(resName, ";");
    String resDN = "";
    while (strTk.hasMoreTokens()) {
      resDN = resDN + "cn="+strTk.nextToken()+",";
    }
    return resDN+System.getProperty("maca_ca.ldap.resourcesRDN")+","+System.getProperty("maca_ca.ldap.rootDN");
  }

  private String resourceNameToLDAPresourceRDN(String resName) {
    StringTokenizer strTk = new StringTokenizer(resName, ";");
    String resDN = "";
    while (strTk.hasMoreTokens()) {
      resDN = resDN + "cn="+strTk.nextToken() + (strTk.hasMoreTokens()?",":"");
    }
    return resDN;
  }

  private String ldapResourceRDNToResourceName(String ldapName) {
    StringTokenizer strTk = new StringTokenizer(ldapName, ",");
    String resName = "";
    while (strTk.hasMoreTokens()) {
      String nameComp = strTk.nextToken();
      resName = resName + nameComp.substring(nameComp.indexOf("=")+1, nameComp.length()) + (strTk.hasMoreTokens()?";":"");
    }
    return resName;
  }

  //Chamado sempre que o cadastro do usurio  salvo pela primeira vez atravs do
  //mtodo 'AddNewUser'
  private Attributes getAttributesToBeSaved(Attributes attrs) throws NamingException  {
    Attributes nonEmptyAttrs = (Attributes) attrs.clone();
    Attribute             oc = nonEmptyAttrs.get("objectclass");
    NamingEnumeration   enum = nonEmptyAttrs.getIDs();
    while (enum.hasMoreElements()) {
      String attrID = (String) enum.nextElement();
      Attribute attr = nonEmptyAttrs.get(attrID);
      if ((attr.size() <= 0) || attr.contains(null) || attr.contains("")) { //Se atributo for vazio, remove
        adjustObjectClassValuesToRemoveAttributte(oc, attrID);
        nonEmptyAttrs.remove(attrID);
      }
    }
    verifyObjectClassAndAttributesConsistency(oc, nonEmptyAttrs);
    return nonEmptyAttrs;
  }

  private void removeValueFromAttrIfFound(Attribute ocattr, String value) throws NamingException {
    NamingEnumeration ocValues = ocattr.getAll();
    while (ocValues.hasMoreElements()) {
      Object ocValue = ocValues.nextElement();
      if (value.equalsIgnoreCase((String) ocValue)) {
        ocattr.remove(ocValue);
      }
    }
  }

  private void adjustObjectClassValuesToRemoveAttributte(Attribute oc, String attrID) throws NamingException {
    String objectClassName = UserSchema.getAttributeParameter(attrID, "requiredby");
    if (!"".equalsIgnoreCase(objectClassName)) {
      removeValueFromAttrIfFound(oc, objectClassName);
    }
  }

  private void addValueToAttrIfNotFound(Attribute ocattr, String newvalue) throws NamingException {
    if (newvalue != null) {
      boolean found = false;
      Enumeration ocenum = ocattr.getAll();
      while (ocenum.hasMoreElements()) {
        String value = (String) ocenum.nextElement();
        if (value.equalsIgnoreCase(newvalue)) {
          found = true;
          break;
        }
      }
      if (!found) ocattr.add(newvalue);
    }
  }

  private void adjustObjectClassValuesToAddorModifyAttributte(Attribute oc, String attrID) throws NamingException {
    String objectClassName = UserSchema.getAttributeParameter(attrID, "requiredby");
    if (!"".equalsIgnoreCase(objectClassName)) {
      addValueToAttrIfNotFound(oc, objectClassName);
    }
  }

  private void verifyIfAllowsMultiValues(Attribute attr) throws NamingException {//Verifica se o attributo admite
                                                                                 //mltiplos valores
    if (attr.size() > 1 && !UserSchema.isMultivalue(attr.getID())) {
      String displayName = UserSchema.getAttributeParameter(attr.getID(), "displayName");
      if ("".equalsIgnoreCase(displayName)) displayName = attr.getID();
      throw new NamingException("O campo '"+displayName+"' no permite mltiplos valores");
    }
  }

  //Verifica a consistncia dos atributos em relao s classes de objetos do LDAP
  private void verifyObjectClassAndAttributesConsistency(Attribute oc, Vector mods) throws NamingException  {
    for (int i = 0; i < mods.size(); i++) {
      if (((ModificationItem) mods.get(i)).getModificationOp() == DirContext.REMOVE_ATTRIBUTE) {
        adjustObjectClassValuesToRemoveAttributte(oc, ((ModificationItem) mods.get(i)).getAttribute().getID());
      }
    }
    Hashtable originalAttrsNames = new Hashtable();
    Attributes attrs = new BasicAttributes(true);
    for (int i = 0; i < mods.size(); i++) {
      if ((((ModificationItem) mods.get(i)).getModificationOp() == DirContext.REPLACE_ATTRIBUTE) ||
          (((ModificationItem) mods.get(i)).getModificationOp() == DirContext.ADD_ATTRIBUTE)) {
        attrs.put(((ModificationItem) mods.get(i)).getAttribute());
        originalAttrsNames.put(((ModificationItem) mods.get(i)).getAttribute().getID().trim().toLowerCase(), ((ModificationItem) mods.get(i)).getAttribute().getID());
      }
    }
    verifyObjectClassAndAttributesConsistency(oc, attrs);
    for (int i = 0; i < mods.size(); i++) {  //Atualiza modificaes feitas nos atributos passados como parmetros
      String attrName = ((ModificationItem) mods.get(i)).getAttribute().getID().trim().toLowerCase();
      Attribute attr = attrs.get(attrName);
      if ((attr != null) && (attr.get() != null)) {
        mods.set(i, new ModificationItem(((ModificationItem) mods.get(i)).getModificationOp(), attr));
      }
    }
    Enumeration enum = attrs.getAll();
    while (enum.hasMoreElements()) { //Adiciona atributos default no presentes originalmente
      Attribute attr = (Attribute) enum.nextElement();
      if ((attr != null) && (!originalAttrsNames.containsKey(attr.getID().trim().toLowerCase()))) {
        mods.add(new ModificationItem(DirContext.ADD_ATTRIBUTE, attr));
      }
    }
  }

  //Verifica a consistncia dos atributos em relao s classes de objetos do LDAP
  private void verifyObjectClassAndAttributesConsistency(Attribute oc, Attributes attrs) throws NamingException  {
    try {
      Attribute[] defaultAttrs = UserSchema.getDefaultAttributes(); //Preenche valores default
      for (int i = 0; i < defaultAttrs.length; i++) {
        if (attrs.get(defaultAttrs[i].getID()) == null) {
          attrs.put(defaultAttrs[i]);
        }
      }

      String[] samevalueofAttrsName = UserSchema.getSameValueofAttributesName();
      for (int i = 0; i < samevalueofAttrsName.length; i++) {
        String samevalueofmask = UserSchema.getAttributeParameter(samevalueofAttrsName[i], "samevalueof");
        String attrName        = samevalueofmask;
        String  pattern = "";
        int  beginIndex = samevalueofmask.indexOf("'");
        if (beginIndex >= 0) {
          int endIndex = samevalueofmask.indexOf("'", beginIndex+1);
          if (endIndex > beginIndex) {
            pattern  = samevalueofmask.substring(beginIndex, endIndex+1);
            attrName = samevalueofmask.substring(beginIndex+1, endIndex);
          }
        }
        Attribute attr = attrs.get(attrName);
        if ((attr != null) && (attr.get() != null)) {
          NamingEnumeration enum = attr.getAll();
          Attribute newAttr = new BasicAttribute(samevalueofAttrsName[i]);
          while (enum.hasMoreElements()) {
            Object value = enum.nextElement();
            if (!"".equalsIgnoreCase(pattern) && (value instanceof String)) {
              value = samevalueofmask.replaceAll(pattern, (String) value);
            }
            newAttr.add(value);
          }
          attrs.put(newAttr);
        }
      }

      String[] requiredAttrsName = UserSchema.getRequiredAttributesName();
      for (int n = 0; n < requiredAttrsName.length; n++) { //Verifica os campos obrigatrios do template
                                                           //de usurios
        if (attrs.get(requiredAttrsName[n]) == null) {
          String displayName = UserSchema.getAttributeParameter(requiredAttrsName[n], "displayName");
          if ("".equalsIgnoreCase(displayName)) displayName = requiredAttrsName[n];
          throw new NamingException("O preenchimento do campo '"+displayName+"'  obrigatrio");
        }
      }
      Enumeration enum = attrs.getAll();
      while (enum.hasMoreElements()) {
        Attribute attr = (Attribute) enum.nextElement();
        if (attr != null) {
          verifyIfAllowsMultiValues(attr);
          adjustObjectClassValuesToAddorModifyAttributte(oc, attr.getID());
        }
      }
      enum = oc.getAll();
      while (enum.hasMoreElements()) {
        String ocName = (String) enum.nextElement();
        UserSchema.isObjectClassConsistent(ocName, attrs);
      }
      UserSchema.areAttributesConsistent(attrs);
      UserSchema.setCharCase(attrs);
      UserSchema.processReplacementRegularExpression(attrs);

      String[] lookupAttrsName = UserSchema.getLookupsAttributesName(true);
      for (int n = 0; n < lookupAttrsName.length; n++) {
        String  lookupName = UserSchema.getAttributeParameter(lookupAttrsName[n], "lookupname");
        String lookupField = UserSchema.getAttributeParameter(lookupAttrsName[n], "lookupupdatefield");
        if (!lookupField.equalsIgnoreCase("")) {
          Attribute attr = attrs.get(lookupAttrsName[n]);
          if ((attr != null) && (attr.get() != null)) {
            TableModel result = UserSchema.getLookup(lookupName, lookupField + "='" + attr.get() +"'");
            if (result.getRowCount() == 0) {
              String displayName = UserSchema.getAttributeParameter(lookupAttrsName[n], "displayName");
              if ("".equalsIgnoreCase(displayName)) displayName = lookupAttrsName[n];
              throw new NamingException("O valor do campo '"+displayName+"' (" + attr.get() +")  invlido");
            }
          }
        }
      }
    }
    catch (Exception exc) {
      throw new NamingException(exc.getMessage());
    }
  }

  //Chamado sempre que o  cadastro do usurio  salvo pelo mtodo 'SaveUserAttributes'
  private ModificationItem[] getAttributesToBeModified(Attributes attrs) throws NamingException  {
    Vector         modItems  = new Vector();
    Attributes currentAttrs  = ctx.value.getAttributes(this.getUserDN());
    Attribute oc             = attrs.get("objectclass");
    modItems.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, oc));
    Enumeration  enum = attrs.getIDs();
    while (enum.hasMoreElements()) {
      String attrID = (String) enum.nextElement();
      Attribute attr = attrs.get(attrID);
      if (attr != null) {
        if ((attr.size() > 0) && !attr.contains(null) && !attr.contains("")) {
          if (currentAttrs.get(attrID) == null) { // Adiciona novo atributo
            modItems.add(new ModificationItem(DirContext.ADD_ATTRIBUTE, attr));
          }
          else { // Modifica atributo existente
            modItems.add(new ModificationItem(DirContext.REPLACE_ATTRIBUTE, attr));
          }
        }
        else if (currentAttrs.get(attrID) != null) { // Remove atributo existente no cadastro
          modItems.add(new ModificationItem(DirContext.REMOVE_ATTRIBUTE, currentAttrs.get(attrID)));
        }
        else {  //Atributos vazios e que no existem no cadastro, logo remover de 'objectclass'
                //as classes que tm alguns destes atributos como obrigatrios
          adjustObjectClassValuesToRemoveAttributte(oc, attrID);
        }
      }
    }
    verifyObjectClassAndAttributesConsistency(oc, modItems);
    ModificationItem[] mods = new ModificationItem[modItems.size()];
    for (int n = 0; n < mods.length; n++) mods[n] = (ModificationItem) modItems.get(n);
    return mods;
  }

  private void reOpen() throws NamingException {
    Hashtable env = ctx.value.getEnvironment();
    ctx.value.close();
    ctx.value = new InitialDirContext(env);
    attrs = ctx.value.getAttributes(this.getUserDN(), UserSchema.getAttributesNames());
  }


  private Role getUserRole(String aRoleName) throws NamingException {
    Role role = null;
    if (!aRoleName.equalsIgnoreCase("")) {
      NamingEnumeration      answer = null;
      SearchControls ctls = new SearchControls();
      ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
      ctls.setReturningAttributes(new String[] {"cn"});
      answer = ctx.value.search(System.getProperty("maca_ca.ldap.roleRDN")+","+
                                System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                System.getProperty("maca_ca.ldap.rootDN"),
                                "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+this.getUserDN()+")", ctls);
      while (answer.hasMore()) {
        SearchResult sr  = (SearchResult) answer.next();
        Attributes attrs = sr.getAttributes();
        String      roleName = (String) attrs.get("cn").get();
        if (aRoleName.equalsIgnoreCase(roleName)) {
          role = new Role(ctx, System.getProperty("maca_ca.ldap.roleRDN")+","+
                               System.getProperty("maca_ca.ldap.groupsRDN")+","+
                               System.getProperty("maca_ca.ldap.rootDN"), sr.getName());
          break;
        }
      }
    }
    return role;
  }

  // Mtodos pblicos definidos manualmente
  public synchronized String getOperacionalAttribute(String attrName) throws NamingException {
    Attributes attrs = ctx.value.getAttributes(this.getUserDN(), new String[] {attrName});
    Attribute attr = attrs.get(attrName);
    if ((attr != null) && (attr.get() != null)) {
      return attr.get().toString();
    }
    return "";
  }
  
  public synchronized boolean haveAttribute(String attrName) throws NamingException {
    Attributes attrs = ctx.value.getAttributes(this.getUserDN(), new String[] {attrName});
    Attribute attr = attrs.get(attrName);
    return attr != null;
  }

  public synchronized boolean refresh() {
    return true;
  }

  public synchronized Date getPasswordWarningDate() throws Exception {
    final long pwdWarningDefault = 86400; //10 dias em segundos
    long pwdWarning = pwdWarningDefault;
    try {
      pwdWarning = Long.parseLong(System.getProperty("maca_ca.passwordPolicy.passwordWarning"));
      pwdWarning = pwdWarning < 0 ? pwdWarningDefault : pwdWarning;
    }
    catch (NumberFormatException exc) {
    }
    Date expDate = this.getPasswordAttributeDate("passwordExpirationTime");
    if (expDate != null) {
      return new Date(expDate.getTime() - pwdWarning * 1000); // segundos para milissegundos;
    }
    return null;
  }

  public synchronized Date getPasswordDefaultExpirationDate() throws Exception {
    long pwdExp = 8640000; //100 dias em segundos
    try {
      pwdExp = Long.parseLong(System.getProperty("maca_ca.passwordPolicy.passwordMaxAge"));
      pwdExp = pwdExp < 0 ? 8640000 : pwdExp;
    }
    catch (NumberFormatException exc) {
    }
    return new Date((new Date()).getTime() + pwdExp * 1000); // segundos para milissegundos;
  }

  public synchronized Date getPasswordDefaultMinAgeDate() throws Exception {
    final long pwdMinAgeDefault = 0;
    long pwdMinAge = pwdMinAgeDefault; //a partir de agora
    try {
      pwdMinAge = Long.parseLong(System.getProperty("maca_ca.passwordPolicy.passwordMinAge"));
      pwdMinAge = pwdMinAge < 0 ? pwdMinAgeDefault : pwdMinAge;
    }
    catch (NumberFormatException exc) {
    }
    return new Date((new Date()).getTime() + pwdMinAge * 1000); // segundos para milissegundos;
  }

  public synchronized boolean setPasswordMustChange(boolean change) throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordMustChange"))) {
      Attributes attrs = new BasicAttributes(true);
      attrs.put("passwordMustChange", change?"true":"false");
      String attr = this.getOperacionalAttribute("passwordMustChange");
      if ("".equalsIgnoreCase(attr)) {
      	this.addObjectClassValueIfNotFound("passwordobject");
        this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.ADD_ATTRIBUTE, attrs);
      }
      else {
        this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.REPLACE_ATTRIBUTE, attrs);
      }
      return true;
    }
    return false;
  }

  public synchronized boolean setPasswordExpirationWarned(boolean warned) throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordExp"))) {
      Attributes pwdExpWarnedAttrs = new BasicAttributes(true);
      pwdExpWarnedAttrs.put("passwordExpWarned", warned?"true":"false");
      String attr = this.getOperacionalAttribute("passwordExpWarned");
      if ("".equalsIgnoreCase(attr)) {
      	this.addObjectClassValueIfNotFound("passwordobject");
        this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.ADD_ATTRIBUTE, pwdExpWarnedAttrs);
      }
      else {
        this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.REPLACE_ATTRIBUTE, pwdExpWarnedAttrs);
      }
      if ((this._log != null)  && warned){
        this._log.println(java.util.logging.Level.WARNING, this.getUserID(),
                    "maca_ca.User", "setPasswordExpirationWarned",
                    "Usuario["+this.getUserID()+"][]: avisado de que a senha vai expirar em "+
                     UTC_DF.format(this.getPasswordAttributeDate("passwordExpirationTime")));
      }
      return true;
    }
    return false;
  }

  public synchronized boolean setPasswordExpirationDate()  throws Exception {
    return this.setPasswordExpirationDate(this.getPasswordDefaultExpirationDate());
  }

  public synchronized boolean setPasswordExpirationDate(Date expDate)  throws Exception {
    String pwdExp = System.getProperty("maca_ca.passwordPolicy.passwordExp");
    if (!"false".equalsIgnoreCase(pwdExp)) {
      if ((expDate != null) && (expDate.after(new Date()))) {
        Attributes attrs = new BasicAttributes(true);
        attrs.put("passwordExpirationTime", UTC_DF.format(new Date(expDate.getTime() - UTC_DF.getTimeZone().getRawOffset())) + "Z");
        String attr = this.getOperacionalAttribute("passwordExpirationTime");
        if ("".equalsIgnoreCase(attr)) {
          this.addObjectClassValueIfNotFound("passwordobject");
          this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.ADD_ATTRIBUTE, attrs);
        }
        else {
          this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.REPLACE_ATTRIBUTE, attrs);
        }
        this.setPasswordExpirationWarned(false);
        return true;
      }
    }
    return false;
  }

  public synchronized boolean setPasswordAllowedChangeDate()  throws Exception {
    return this.setPasswordAllowedChangeDate(this.getPasswordDefaultMinAgeDate());
  }

  public synchronized boolean setPasswordAllowedChangeDate(Date minAgeDate)  throws Exception {
    if (!"0".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordMinAge"))) {
      if ((minAgeDate != null) && (minAgeDate.after(new Date()))) {
        Attributes attrs = new BasicAttributes(true);
        attrs.put("passwordAllowChangeTime", UTC_DF.format(new Date(minAgeDate.getTime() - UTC_DF.getTimeZone().getRawOffset())) + "Z");
        String attr = this.getOperacionalAttribute("passwordAllowChangeTime");
        if ("".equalsIgnoreCase(attr)) {
          this.addObjectClassValueIfNotFound("passwordobject");
          this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.ADD_ATTRIBUTE, attrs);
        }
        else {
          this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.REPLACE_ATTRIBUTE, attrs);
        }
        return true;
      }
    }
    return false;
  }

  public synchronized boolean isToWarningPasswordExpiration() throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordExp"))) {
      Date warningDate = this.getPasswordWarningDate();
      if (warningDate != null) {
        return warningDate.before(new Date());
      }
    }
    return false;
  }

  public synchronized boolean passwordMustChange() throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordMustChange"))) {
      String attr = this.getOperacionalAttribute("passwordMustChange");
      if ("true".equalsIgnoreCase(attr)) {
        return true;
      }
    }
    return false;
  }

  public synchronized boolean passwordExpirationDateWarned() throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordExp"))) {
      String attr = this.getOperacionalAttribute("passwordExpWarned");
      if ("true".equalsIgnoreCase(attr)) {
        return true;
      }
    }
    return false;
  }

  public synchronized boolean isPasswordExpired() throws Exception {
    if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordExp"))) {
      Date expDate = this.getPasswordAttributeDate("passwordExpirationTime");
      if ((expDate != null) && (expDate.before(new Date()))) {
        return true;
      }
    }
    return false;
  }

  public synchronized String getActiveRolesName() {
    String result = "";
    for (int n = 0; n < activeRoles.size(); n++) {
      String roleName = (String) ((Role) activeRoles.get(n)).getName();
      result = result + roleName + (n < (activeRoles.size()-1) ? ";" : "");
    }
    return result;
  }

  public synchronized boolean setActiveRole(String aRoleName)  {
    try {
      Role role = (Role) activableRolesHash.get(aRoleName.toLowerCase());
      if ((role != null) && !this.activeRoles.contains(role)) {
        this.activeRoles.add(role);
        return true;
      }
    }
    catch (Exception exc) {
      if (this._log != null) {
        this._log.println(java.util.logging.Level.SEVERE, this.getUserID(),
                    "maca_ca.User", "setActiveRole",
                    exc.toString());
      }
      else {
        System.out.println(exc);
        exc.printStackTrace();
      }
    }
    return false;
  }

  public synchronized Vector getRolesNames() throws NamingException {
    NamingEnumeration      answer = null;
    Vector         rolesNamesList = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    answer = ctx.value.search(System.getProperty("maca_ca.ldap.roleRDN")+","+
                              System.getProperty("maca_ca.ldap.groupsRDN")+","+
                              System.getProperty("maca_ca.ldap.rootDN"),
                              "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+this.getUserDN()+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      Attributes attrs = sr.getAttributes();
      Attribute   attr = attrs.get("cn");
      rolesNamesList.add(attr.get());
    }
    return rolesNamesList;
  }

  public synchronized Hashtable getAssociatedResources(String resName, String privilege) {
    Hashtable resList = new Hashtable();
    try {
      try {
        Vector roles = this.activableRolesVect;
        for (int n = 0; n < roles.size(); n++) {
          Role r = (Role) roles.get(n);
          resList.putAll(r.getParentResources(resName, privilege));
          resList.putAll(r.getResources(resName, privilege));
        }
      }
      catch (javax.naming.ServiceUnavailableException exc) {
        this.reOpen();
      }
    }
    catch (NamingException exc) {
      exc.printStackTrace();
    }
    return resList;
  }

  public synchronized String[] getRoles() {
    String[] rolesList = {};
    Vector rolesVec = this.activableRolesVect;
    rolesList = new String[rolesVec.size()];
    for (int n = 0; n < rolesList.length; n++) {
      rolesList[n] = ((Role) rolesVec.get(n)).getName();
    }
    return rolesList;
  }

  public synchronized Vector getOrgNamesMembership() throws NamingException {
    NamingEnumeration      answer = null;
    Vector           orgNamesList = new Vector();
    SearchControls ctls = new SearchControls();
    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
    ctls.setReturningAttributes(new String[] {"cn"});
    answer = ctx.value.search(System.getProperty("maca_ca.ldap.orgChartRDN")+","+
                              System.getProperty("maca_ca.ldap.groupsRDN")+","+
                              System.getProperty("maca_ca.ldap.rootDN"),
                              "("+System.getProperty("maca_ca.ldap.groupMemberAttribute")+"="+this.getUserDN()+")", ctls);
    while (answer.hasMore()) {
      SearchResult sr  = (SearchResult) answer.next();
      Attributes attrs = sr.getAttributes();
      Attribute   attr = attrs.get("cn");
      orgNamesList.add(attr.get());
    }
    return orgNamesList;
  }

  public synchronized boolean hasPassword(String pwd) {
    boolean authenticated  = false;
    try {
      Hashtable env = (Hashtable) this.ctx.value.getEnvironment().clone();
      env.put(Context.SECURITY_CREDENTIALS, pwd);
      if (pwd.length() > 0) {
        DirContext ctx = new InitialDirContext(env);
        authenticated = true;
        ctx.close();
      }
      else {
        authenticated = false;
      }
    }
    catch (NamingException exc) {
     authenticated = false;
    }
    finally {
     return authenticated;
    }
  }

  public synchronized void changeLoginName(String newLoginName) throws NamingException {
    if ((newLoginName == null) || newLoginName.equalsIgnoreCase("")) {
      throw new NamingException("Novo nome do usurio no pode ser vazio!");
    }
    String oldUserDN = this.getUserDN();
    String newUserDN = "uid="+newLoginName+","+this._userRoot;
    ctx.value.rename(oldUserDN, newUserDN);
    Vector rolesRDNs = this.getRolesRDN();
    for (int n = 0; n < rolesRDNs.size(); n++) {
      Role role = new Role(this.ctx, System.getProperty("maca_ca.ldap.roleRDN")+","+
                                     System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                     System.getProperty("maca_ca.ldap.rootDN"), (String) rolesRDNs.get(n));
      if (role != null) {
        role.removeMemberFromGroup(oldUserDN);
        role.addMemberToGroup(newUserDN);
      }
    }
    Vector orgUnitsRDNs = this.getOrgUnitsRDN();
    for (int n = 0; n < orgUnitsRDNs.size(); n++) {
      Group grp = new Group(this.ctx, System.getProperty("maca_ca.ldap.orgChartRDN")+","+
                                      System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                      System.getProperty("maca_ca.ldap.rootDN"), (String) orgUnitsRDNs.get(n));
      if (grp != null) {
        grp.removeMemberFromGroup(oldUserDN);
        grp.addMemberToGroup(newUserDN);
      }
    }
    String ldapLogin = (String) ctx.value.getEnvironment().get(Context.SECURITY_PRINCIPAL);
    if (ldapLogin.equalsIgnoreCase(newUserDN)) {
      _userRDN = "uid="+newLoginName;
      ctx.value.addToEnvironment(Context.SECURITY_PRINCIPAL, newUserDN);
    }
    else {
      _userRDN = "uid="+newLoginName;
    }
    attrs = ctx.value.getAttributes(newUserDN, UserSchema.getAttributesNames());
  }

  public synchronized boolean setNewPassword(String actualPwd, String newPwd) throws PasswordErrorException {
    try {
      try {
        if (!newPwd.equalsIgnoreCase("") && this.isAlowedToChangePassword() &&
            this.hasPassword(actualPwd) && this.newPasswordIsValid(newPwd) &&
            !this.isPasswordInHistory(newPwd)) {
          Attribute oldPwdAttr = this.ctx.value.getAttributes(this.getUserDN(), new String[] {"userpassword"}).get("userpassword");

          this.changePassword(newPwd); //Modifica a senha

          this.putPasswordInHistory(oldPwdAttr);
          this.setPasswordMustChange(false);
          this.setPasswordAllowedChangeDate();
          this.setPasswordExpirationDate();
          return true;
        }
      }
      catch (javax.naming.ServiceUnavailableException exc) {
        // System.out.println("Reconectando ..."+exc.toString(true));
        this.reOpen();
        throw new PasswordErrorException(exc.getExplanation());
      }
    }
    catch (java.security.NoSuchAlgorithmException exc) {
      throw new PasswordErrorException(exc.getMessage());
     }
   catch (NamingException exc) {
     throw new PasswordErrorException(exc.getExplanation());
    }
    catch (Exception exc) {
      throw new PasswordErrorException(exc.getMessage());
     }
    return false;
  }

  private String getPasswordHashing(String clearPwd) throws  java.security.NoSuchAlgorithmException {
    String mdAlg = System.getProperty("maca_ca.passwordPolicy.passwordStorageScheme");
    String pwdHashing = clearPwd;
    if (!mdAlg.equalsIgnoreCase("Clear")) {
      MessageDigest md = MessageDigest.getInstance(mdAlg);
      md.update(pwdHashing.getBytes());
      pwdHashing = "{"+md.getAlgorithm()+"}"+ Conversion.byteArrayToBase64String(md.digest()).trim();
    }
    return pwdHashing;
  }


  private synchronized void changePassword(String newPassword) throws NamingException,
                                              java.security.NoSuchAlgorithmException {
    Attributes  pwdAttrs = new BasicAttributes(true);
    String       pwdHash = this.getPasswordHashing(newPassword);
    pwdAttrs.put("userpassword", pwdHash);
    this.ctx.value.modifyAttributes(this.getUserDN(),
                                    (this.haveAttribute("userpassword")?
                                      DirContext.REPLACE_ATTRIBUTE:
                                      DirContext.ADD_ATTRIBUTE),
                                    pwdAttrs);
    try { //Tenta remover o atributo 'passwordlocked', caso ele no exista,
    	  //no tem problema, a execuo deve prosseguir normalmente
      Attributes lockPwdAttrs = new BasicAttributes(true);
      lockPwdAttrs.put("passwordlocked", null);
      this.ctx.value.modifyAttributes(this.getUserDN(), DirContext.REMOVE_ATTRIBUTE, lockPwdAttrs);
    } catch (NamingException exc) {}
    String ldapLogin = (String) ctx.value.getEnvironment().get(Context.SECURITY_PRINCIPAL);
    if (ldapLogin.equalsIgnoreCase(this.getUserDN())) {
      ctx.value.addToEnvironment(Context.SECURITY_CREDENTIALS, newPassword);
      this.reOpen();
    }
    if (_log != null) {
      this._log.println(java.util.logging.Level.WARNING, this.getUserID() + " :: "+this.getActiveRolesName(),
                        "maca_ca.User", "changePassword",
                        "Senha alterada");
    }
  }
  
  private synchronized void addObjectClassValueIfNotFound(String ocValue)  throws Exception {
  	Attributes attrs = ctx.value.getAttributes(this.getUserDN(), new String[] {"objectclass"});
  	Attribute  ocAttr = attrs.get("objectclass");
  	if ((ocAttr != null) && (ocAttr.get() != null)) {
 		NamingEnumeration values = ocAttr.getAll();
  		boolean found = false;
  		while (values.hasMoreElements()) {
  		  Object value = values.next();
  		  if ((value != null) && value instanceof String) {
  		  	if (((String)value).equalsIgnoreCase(ocValue)) {
  		  		found = true;
  		  		break;
  		  	}
  		  }
  		} 
  		if (!found) {
  			ocAttr.add(ocValue);	
  			Attributes ocAttrs = new BasicAttributes(true);
  			ocAttrs.put(ocAttr);
  			this.ctx.value.modifyAttributes(this.getUserDN(),
  					DirContext.REPLACE_ATTRIBUTE, 
					ocAttrs);									
  		} 		
  	}
  }
  
  public synchronized void lockAccount() throws Exception {
  	Attributes attrs = ctx.value.getAttributes(this.getUserDN(), new String[] {"userpassword"});
  	Attribute  pwdAttr = attrs.get("userpassword");
  	if ((pwdAttr != null) && (pwdAttr.get() != null)) {
  		addObjectClassValueIfNotFound("passwordobject");	
  		ModificationItem[] mods = new ModificationItem[3];
  		mods[0] = new ModificationItem((this.haveAttribute("passwordlocked")?
  				                        DirContext.REPLACE_ATTRIBUTE: DirContext.ADD_ATTRIBUTE),
										new BasicAttribute("passwordlocked", pwdAttr.get()));  
  		mods[1] = new ModificationItem((this.haveAttribute("passwordlocktime")?
                                        DirContext.REPLACE_ATTRIBUTE: DirContext.ADD_ATTRIBUTE),
				                        new BasicAttribute("passwordlocktime", UTC_DF.format(new Date((new Date()).getTime() - UTC_DF.getTimeZone().getRawOffset())) + "Z"));  
  		mods[2] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, pwdAttr);  
  		this.ctx.value.modifyAttributes(this.getUserDN(), mods);
  	}
  	else {
  		throw new NamingException("A conta do usurio '"+this.getUserID()+"' j est bloqueada");
  	}
  }

  public synchronized void unLockAccount() throws Exception {
  	if (this.haveAttribute("userpassword")) {
  		throw new NamingException("A conta do usurio '"+this.getUserID()+"' j est desbloqueada");
  	}
  	Attributes lockPwdAttrs = ctx.value.getAttributes(this.getUserDN(), new String[] {"passwordlocked"});
  	Attribute  lockPwdAttr = lockPwdAttrs.get("passwordlocked");
  	if ((lockPwdAttr != null) && (lockPwdAttr.get() != null)) {
  		ModificationItem[] mods = new ModificationItem[3];
  		mods[0] = new ModificationItem((this.haveAttribute("userpassword")?
                                        DirContext.REPLACE_ATTRIBUTE: DirContext.ADD_ATTRIBUTE),
				                        new BasicAttribute("userpassword", lockPwdAttr.get()));  
  		mods[1] = new ModificationItem((this.haveAttribute("passwordunlocktime")?
                                        DirContext.REPLACE_ATTRIBUTE: DirContext.ADD_ATTRIBUTE),
                                        new BasicAttribute("passwordunlocktime", UTC_DF.format(new Date((new Date()).getTime() - UTC_DF.getTimeZone().getRawOffset())) + "Z"));  
  		mods[2] = new ModificationItem(DirContext.REMOVE_ATTRIBUTE, lockPwdAttr);  		
  		this.ctx.value.modifyAttributes(this.getUserDN(), mods);
  	}
  	else {
      throw new NamingException("A conta do usurio '"+this.getUserID()+"' est sem senha para desbloquear: defina uma nova senha");
    }
  }

  public synchronized boolean isAccountLocked() throws Exception {
  	return !this.haveAttribute("userpassword");
  }

  public synchronized Attributes getUserAtributes() {
    return (Attributes) attrs.clone();
  }

  public synchronized void setUserAttributes(Attributes newAttrs) throws NamingException {
    Attribute       ocAttr = UserSchema.getDefaultObjectClassAttribute();
    NamingEnumeration enum = newAttrs.getAll();
    while (enum.hasMoreElements()) {
      Attribute newattr = (Attribute) enum.nextElement();
      if (newattr.getID().equalsIgnoreCase(ocAttr.getID())) {
        this.attrs.put(ocAttr);
        NamingEnumeration newocenum = newattr.getAll();
        while (newocenum.hasMoreElements()) {
          addValueToAttrIfNotFound(ocAttr, (String) newocenum.nextElement());
        }
      }
      else if (newattr.getID().equalsIgnoreCase("userpassword")) {
        //No permite alterar a senha
      }
      else {
        attrs.put(newattr);
      }
    }
  }

  public synchronized void saveUserAtributes() throws Exception {
    Vector sameEmployeeNumberList = this.getSameEmployeeNumberList();
    if (sameEmployeeNumberList.isEmpty()) {
      if (!newUser) {
        ctx.value.modifyAttributes(this.getUserDN(),
                                   this.getAttributesToBeModified(this.attrs));
      }
      else {
        this.addNewUser();
      }
    }
    else {
      String uids = "";
      for (int n = 0; n < sameEmployeeNumberList.size(); n++) {
        if (n == 0) {
          uids = (String) sameEmployeeNumberList.get(n);
        }
        else {
          uids = (String) sameEmployeeNumberList.get(n) +", "+ uids;
        }
      }
      String employeeNumberDisplayName = "employeeNumber";
      Attributes employeeNumberSchemaAttr = UserSchema.getAttributeSchema("employeeNumber");
      if (employeeNumberSchemaAttr != null) {
        Attribute displayNameAttr = employeeNumberSchemaAttr.get("displayname");
        if ((displayNameAttr != null) && (displayNameAttr.get() != null)) {
          employeeNumberDisplayName = (String) displayNameAttr.get();
        }
      }
      throw new NamingException("erro - j h usurios cadastrados com mesmo valor para o campo '"+employeeNumberDisplayName+"': "+uids);
    }
  }

  private synchronized void addNewUser() throws Exception {
    if (newUser) {
      ctx.value.createSubcontext(this.getUserDN(), this.getAttributesToBeSaved(attrs));
      attrs = ctx.value.getAttributes(this.getUserDN(), UserSchema.getAttributesNames());
      try {
        String firstPwd = new String(PWD_FAC.getNewPassword(this.getUserAtributes()));
        this.changePassword(firstPwd); //Adiciona primeira a senha
      }
      catch (Exception exc) {
        throw new Exception("Falha na criao da senha inicial do novo usurio '"+this.getUserID()+"': "+exc.getMessage());
      }
      finally {
        newUser = false;
      }
      this.setPasswordMustChange(true);
      this.setPasswordExpirationDate();
      this.setPasswordAllowedChangeDate();
    }
  }

  public synchronized void removeUser() throws NamingException, Exception {
    if (!this.newUser) {
      if (!this.hasRole() && !this.hasOrg()) {
        ctx.value.destroySubcontext(this.getUserDN());
      }
      else {
        throw new Exception("Usurio '"+this.getUserID()+"' no pode ser removido,\n"+
                            "pois tem papel associado ou\n"+
                            "pertence a uma unidade organizacional.");
      }
    }
  }

  public synchronized boolean hasRole(String role) throws NamingException {
    Vector rolesNames = this.getRolesNames();
    Enumeration enumStr = rolesNames.elements();
    while (enumStr.hasMoreElements()) {
      String roleName = (String) enumStr.nextElement();
      if (roleName.equalsIgnoreCase(role)) {
        return true;
      }
    }
    return false;
  }

  static public void ldapAttributesToXML(Attributes attributes,
                                         Writer result, boolean tabbed) throws NamingException,
                                                                         IOException {
    String[] names = UserSchema.getTabNames();
    Hashtable writers = new Hashtable();
    for (int n = 0; n < names.length; n++) {
      StringWriter writer = new StringWriter(512);
      if (!names[n].equalsIgnoreCase("") && tabbed) {
        writer.write("<tab name=\""+names[n]+"\">" + LINE_END);
      }
      writers.put(names[n], writer);
    }
    Vector attrsNames = UserSchema.getAttrsNames();
    for (int i = 0; i < attrsNames.size(); i++) {
      Attribute  attr = attributes.get((String) attrsNames.get(i));
      if ((attr != null) && (!attr.getID().equalsIgnoreCase("userPassword"))) {
        String tabName = UserSchema.getAttributeTabName(attr.getID());
        StringWriter writer = (StringWriter) writers.get(tabName);
        writer.write("<attribute name=\""+attr.getID()+"\" ");
        Attributes attrSchema = (Attributes) UserSchema.getAttributeSchema(attr.getID());
        if (attrSchema != null) {
          NamingEnumeration ldapAttrsAttrs = attrSchema.getAll();
          while (ldapAttrsAttrs.hasMoreElements()) {
            Attribute ldapAttrAttr = (Attribute) ldapAttrsAttrs.nextElement();
            if (!"default".equalsIgnoreCase(ldapAttrAttr.getID()) &&
                !"name".equalsIgnoreCase(ldapAttrAttr.getID())) {
              writer.write(ldapAttrAttr.getID()+"=\""+ldapAttrAttr.get()+"\" ");
            }
          }
        }
        writer.write(">" + LINE_END);
        Enumeration enum = attr.getAll();
        while (enum.hasMoreElements()) {
          Object value = enum.nextElement();
          if (value != null) {
            writer.write("<value><![CDATA["+value+"]]></value>" + LINE_END);
          }
        }
        writer.write("</attribute>" + LINE_END);
      }
    }
    for (int n = 0; n < names.length; n++) {
      StringWriter writer = (StringWriter) writers.get(names[n]);
      if (!names[n].equalsIgnoreCase("")  && tabbed) {
        writer.write("</tab>" + LINE_END);
      }
      result.write(writer.toString());
    }
  }

  public synchronized void ldapEntryToXML(String returningFields,
                                          Writer result, boolean tabbed) throws NamingException,
                                                                                IOException {
    if (returningFields == null) {
      returningFields = "";
    }
    result.write("<ldapentry dn=\""+this.getUserDN()+"\">" + LINE_END);
    Enumeration enumAttrs;
    Attributes  attrsList = new BasicAttributes(true);
    if (returningFields.equalsIgnoreCase("*")) {
      attrsList = (Attributes) this.getUserAtributes();
    }
    else {
      Attributes      attrs = this.getUserAtributes();
      StringTokenizer strTk = new StringTokenizer(returningFields, ",");
      while (strTk.hasMoreTokens()) {
        Attribute attr = attrs.get(strTk.nextToken().trim());
        if (attr != null) {
          attrsList.put(attr);
        }
      }
    }
    this.ldapAttributesToXML(attrsList, result, tabbed);
    result.write("</ldapentry>" + LINE_END);
  }


  public synchronized  void rolesToXML(String[] returningFields, Writer result) throws NamingException,
                                                                                       IOException {
    result.write("<roles type=\"active\">" + LINE_END);
    for (int n = 0; n < activeRoles.size(); n++) {
      ((Role) activeRoles.get(n)).ldapEntryToXML(returningFields, "objectscope", false, false, result);
    }
    result.write("</roles>" + LINE_END);
    result.write("<roles type=\"available\">" + LINE_END);
    Enumeration enum = this.activableRolesHash.elements();
    while (enum.hasMoreElements()) {
      ((Role) enum.nextElement()).ldapEntryToXML(returningFields, "objectscope", false, false, result);
    }
    result.write("</roles>" + LINE_END);
  }

  public boolean isNewUser() {
    return newUser;
  }

  public synchronized boolean hasRole() throws NamingException {
    return this.getRolesNames().size() > 0;
  }

  public synchronized boolean hasOrg(String orgName) throws NamingException {
    Vector orgNames = this.getOrgNamesMembership();
    return orgNames.contains(orgName);
  }

  public boolean hasOrg() throws NamingException {
    return this.getOrgNamesMembership().size() > 0;
  }

  public synchronized String getUserDN() {
    String userRDN;
    userRDN = (_userRDN=="")?_userRDN:_userRDN+",";
    return userRDN+_userRoot;
  }


  public synchronized String getUserID() {
    String userID = _userRDN;
    int endPos    = userID.indexOf(",");
    if ( endPos == -1) {
      endPos = userID.length();
    }
    userID = userID.substring(userID.indexOf("=")+1,endPos);
    return userID;
  }

  public synchronized void resetCache() throws Exception {
  }

  public synchronized boolean hasAccess(String resName, String accessMode)  {
    return this.hasAccess(resName, "", accessMode);
  }

  public synchronized boolean hasAccess(String resName, String params, String accessMode)  {
    try {
      //1. Caso no haja papis disponveis para ativar, o acesso  negado e
      //e o algoritmo pra.
      if (this.activableRolesHash.isEmpty()) {
        return false;
      }
      //2. Verifica se h alguma autorizao forte, negativa, que proba o acesso.
      //2.1 Primeiro verifica as autorizaes estticas, i.e.,  sem regras. Caso
      //haja, o acesso  negado e o algoritmo pra.
      AuthorizationResult res = this.activableRolesHaveAuthorization(true, false, accessMode, resName); //true = forte; false = negative
      if (!res.isUndecided() && res.isFalse()) {
        return false;
      }
      //2.2 Depois verifica todas as autorizaes dinmicas, i.e.,  sem regras
      //Caso alguma seja negativa, o acesso  negado e o algoritmo pra.
      //Caso contrrio, no havendo autorizaes fortes e negativas, verifica-se se
      //h alguma autorizao forte, positiva, que conceda o acesso.
      //Primeiro verifica se houve alguma regra que foi avaliada 'positiva',
      //Caso tenha havido,  porque no houve
      //nenhuma autorizao, com ou sem regra, proibindo o acesso. Neste caso,
      //o acesso  concedido, o papel correspondente  ativado e o algoritmo pra.
      res = this.activableRolesHaveAuthorization(true ,accessMode, resName, params, inter);
      if (!res.isUndecided() && res.isFalse()) {
        return false;
      }
      else if (!res.isUndecided() && res.isTrue() && res.getRole() != null) {
        this.setActiveRole(res.getRole().getName());
        return true;
      }
      //3 Verifica se h alguma autorizaao esttica, forte, positiva, que concede o acesso.
      //Caso haja, o acesso  concedido, o papel correspondente  ativado
      // e o algoritmo pra.
      res = this.activableRolesHaveAuthorization(true, true, accessMode, resName); //true = forte; true = positive
      if (!res.isUndecided() && res.isTrue() && res.getRole() != null) {
        this.setActiveRole(res.getRole().getName());
        return true;
      }
      //4. Verifica se h alguma autorizao fraca, positiva, que conceda o acesso.
      //4.1 Primeiro procura por autorizaes estticas (i.e., sem regras), positivas.
      //Caso haja, o acesso  concedido e o algoritmo pra.
      res = this.activableRolesHaveAuthorization(false, true, accessMode, resName); //false = weak; true = positive
      if (!res.isUndecided() && res.isTrue() && res.getRole() != null) {
        this.setActiveRole(res.getRole().getName());
        return true;
      }
      //4.2 Depois procura por autorizaes fracas dinamicas (i.e., com regras) que
      //sejam avaliadas positivamente. Caso haja, o acesso  concedido
      //e o algoritmo pra.
      res = this.activableRolesHaveAuthorization(false ,accessMode, resName, params, inter);
      if (!res.isUndecided() && res.isTrue() && res.getRole() != null) {
        this.setActiveRole(res.getRole().getName());
        return true;
      }
      //Caso o algoritmo no tenha parado at este ponto, o acesso 
      //negado e o algoritmo pra.
    }
    catch (Exception exc) {
      if (this._log != null) {
        this._log.println(java.util.logging.Level.SEVERE, this.getUserID() + " :: "+this.getActiveRolesName(),
                          "maca_ca.User", "hasAccess",
                          exc.toString());
      }
      else {
        System.out.println(exc);
        exc.printStackTrace();
      }
    }
    return false;
  }

  public User(LogWriter aLog, String serverAndPort, String userRoot,
              String userRDN, String ldap_pwd, contexts.ContextExpressionInterpreter anInterpreter) throws NamingException {
    this(serverAndPort, userRoot, userRDN, ldap_pwd);
    this.inter = anInterpreter;
    this._log = aLog;
  }

  public User(DirContextHolder aContext, LogWriter aLog, String userRoot, String userRDN) throws NamingException,
                                                                                                 java.security.NoSuchAlgorithmException {
    this(aContext, userRoot, userRDN);
    _log = aLog;
  }

  public User(DirContextHolder aContext, String userRoot, String userRDN) throws NamingException,
                                                                                 java.security.NoSuchAlgorithmException {
    this._userRoot        = userRoot;
    this._userRDN         = userRDN;
    this.ctx              = aContext;
    try {
      newUser = false;
      attrs = ctx.value.getAttributes(this.getUserDN(), UserSchema.getAttributesNames());
    }
    catch (javax.naming.NameNotFoundException exc) {
      newUser = true;
      this.newUserSetup(_userRDN);
    }
    ActivableRolesSetup();
  }

  public User(String serverAndPort, String userRoot,
              String userRDN, String ldap_pwd) throws NamingException {

    this._userRoot          = userRoot;
    this._userRDN           = userRDN;
    String LDAP_LOGIN       = this.getUserDN();
    String LDAP_CREDENTIALS = ldap_pwd;
    Hashtable env = new Hashtable();
    env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
    if (!System.getProperty("maca_ca.ldap.ssl").equalsIgnoreCase("false")) {
      env.put(Context.SECURITY_PROTOCOL, "ssl");
      env.put("java.naming.ldap.factory.socket", System.getProperty("maca_ca.ldap.ssl.factory.socket"));
    }
    env.put(Context.SECURITY_AUTHENTICATION, "simple");
    env.put(Context.SECURITY_PRINCIPAL, LDAP_LOGIN);
    env.put(Context.SECURITY_CREDENTIALS, LDAP_CREDENTIALS);
    env.put(Context.PROVIDER_URL,"ldap://"+serverAndPort+"/");
    this.ctx = new DirContextHolder(new InitialDirContext(env));
    newUser  = false;
    attrs    = ctx.value.getAttributes(LDAP_LOGIN, UserSchema.getAttributesNames());
    ActivableRolesSetup();
  }
  
}