package maca_ca;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.sql.*;
import java.util.*;
import javax.xml.parsers.*;
import javax.swing.table.*;
import org.w3c.dom.Document;
import javax.naming.directory.*;
import org.w3c.dom.DOMException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public abstract class UserSchema {
  private static final String LINE_END = System.getProperty("line.separator");

  private static Vector             attrs = new Vector();
  private static Vector         attrsKeys = new Vector();
  private static Vector          tabAttrs = new Vector();
  private static Vector           tabKeys = new Vector();
  private static Hashtable     ocReqAttrs = new Hashtable();
  private static Hashtable attrsAllowedBy = new Hashtable();
  private static Hashtable   databaseurls = new Hashtable();
  private static Hashtable        lookups = new Hashtable();

  static {
    try {
      synchronized (ocReqAttrs) {
        synchronized (attrsAllowedBy) {
          readUserSchema();
          ocReqAttrs     = getObjectClassNamesWithRequiredAttributesNames();
          attrsAllowedBy = getAttributesNamesAllowedBy();
          loadDatabasesClasses();
        }
      }
    }
    catch (Exception exc) {
      exc.printStackTrace();
    }
  }

  private static Connection getConnection(String databaseurlname) throws Exception {
    javax.naming.directory.Attributes attrs = (javax.naming.directory.Attributes) databaseurls.get(databaseurlname);
    if (attrs != null) {
      javax.naming.directory.Attribute url = attrs.get("url");
      if ((url != null) && (url.get() != null)) {
        javax.naming.directory.Attribute userName = attrs.get("username");
        javax.naming.directory.Attribute      pwd = attrs.get("password");
        if ((userName != null) && (userName.get() != null) &&
            (pwd != null) && (pwd.get() != null)) {
          return DriverManager.getConnection((String) url.get(),
                                             (String) userName.get(),
                                             (String) pwd.get());
        }
        else {
          return DriverManager.getConnection((String) url.get());
        }
      }
    }
    return null;
  }

  private static void releaseConnection(Connection conn) throws Exception {
    conn.close();
  }


  private static synchronized void loadDatabasesClasses() throws Exception  {
    Enumeration enum = databaseurls.elements();
    while (enum.hasMoreElements()) {
      javax.naming.directory.Attributes attrs = (javax.naming.directory.Attributes) enum.nextElement();
      if (attrs != null) {
        javax.naming.directory.Attribute attr = attrs.get("driver");
        if ((attr != null) && (attr.get() != null)) {
          Class.forName((String) attr.get());
        }
      }
    }
  }

  private static String formatNamesToLowerCase(String aName) {
    StringTokenizer strToken = new StringTokenizer(aName);
    String nome = "";
    while (strToken.hasMoreTokens()) {
      String token = strToken.nextToken();
      if (!token.equalsIgnoreCase("da") &&
          !token.equalsIgnoreCase("de") &&
          !token.equalsIgnoreCase("di") &&
          !token.equalsIgnoreCase("do") &&
          !token.equalsIgnoreCase("du") &&
          !token.equalsIgnoreCase("das") &&
          !token.equalsIgnoreCase("dos")) {
        if (token.length() > 1) {
          token = token.substring(0,1).toUpperCase() + token.substring(1, token.length()).toLowerCase();
        }
      }
      else {
        token = token.toLowerCase();
      }
      nome = nome + (nome.equalsIgnoreCase("") ? "" : " ") + token;
    }
    return nome;
  }

  private static synchronized Hashtable getAttributesNamesAllowedBy()  {
    Hashtable attrsAndOcNames = new Hashtable();
    for (int i = 0; i < attrs.size(); i++) {
      Attributes attrSchema = (Attributes) attrs.get(i);
      if (attrSchema != null) {
        Attribute attrType = attrSchema.get("allowedby");
        try {
          if ((attrType != null) && (attrType.get() != null)) {
            String ocName = ((String) attrType.get()).toLowerCase();
            attrsAndOcNames.put(attrsKeys.get(i), ocName);
          }
        }
        catch (javax.naming.NamingException exc) {
        }
      }
    }
    return attrsAndOcNames;
  }

  private static synchronized Hashtable getObjectClassNamesWithRequiredAttributesNames()  {
    Hashtable ocAndAttrs = new Hashtable();
    for (int i = 0; i < attrs.size(); i++) {
      Attributes attrSchema = (Attributes) attrs.get(i);
      if (attrSchema != null) {
        Attribute attrType = attrSchema.get("requiredby");
        try {
          if ((attrType != null) && (attrType.get() != null)) {
            String ocName = ((String) attrType.get()).toLowerCase();
            Vector attrsList =  (Vector) ocAndAttrs.get(ocName);
            if (attrsList == null) {
              attrsList = new Vector();
              ocAndAttrs.put(ocName, attrsList);
            }
            attrsList.add(attrsKeys.get(i));
          }
        }
        catch (javax.naming.NamingException exc) {
        }
      }
    }
    return ocAndAttrs;
  }

  private static synchronized boolean putAttributeSchema(String tabName, String attrName, Attributes attrSchema) {
    tabName  = tabName.toLowerCase();
    attrName = attrName.toLowerCase();
    if (attrsKeys.indexOf(attrName) == -1) {
      attrs.add(attrSchema);
      attrsKeys.add(attrName);
      int index = tabKeys.indexOf(tabName);
      if (index == -1) {
        tabKeys.add(tabName);
        tabAttrs.add(new Vector());
        index = tabKeys.size() - 1;
      }
      Vector attrList = (Vector) tabAttrs.get(index);
      attrList.add(attrSchema);
      return true;
    }
    return false;
  }

  private static org.w3c.dom.Node getFirstNodeByName(org.w3c.dom.Node node, String nodeName) throws Exception {
    org.w3c.dom.Node nodeChildren = null;
    org.w3c.dom.NodeList nodeList = node.getChildNodes();
    for (int i = 0; i < nodeList.getLength(); i++) {
      if (nodeName.equalsIgnoreCase(nodeList.item(i).getNodeName())) {
        nodeChildren = nodeList.item(i);
        break;
      }
    }
    return nodeChildren;
  }

  private static void readUserAttributeSchema(org.w3c.dom.Node ldapEntryNode, String tabName) {
    org.w3c.dom.NodeList attrsListNodes = ldapEntryNode.getChildNodes();
    for (int n = 0; n < attrsListNodes.getLength(); n++) {
      org.w3c.dom.Node attrNode = attrsListNodes.item(n);
      if ("attribute".equalsIgnoreCase(attrNode.getNodeName())) {
        org.w3c.dom.NamedNodeMap attrValues = attrNode.getAttributes();
        org.w3c.dom.Node attrVal = attrValues.getNamedItem("name");
        if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
          String attrName = attrVal.getNodeValue();
          if (getAttributeSchema(attrName) == null) {
            Attributes attrs = new BasicAttributes(true);
            for (int k = 0; k < attrValues.getLength(); k++) {
              attrVal = attrValues.item(k);
              attrs.put(attrVal.getNodeName(), attrVal.getNodeValue());
              org.w3c.dom.NodeList defaults = attrNode.getChildNodes();
              Attribute defaultAttrValues = new BasicAttribute("default");
              for (int j = 0; j < defaults.getLength(); j++) {
                org.w3c.dom.Node _default =  defaults.item(j);
                if ("value".equalsIgnoreCase(_default.getNodeName())) {
                  org.w3c.dom.NodeList defaultValues = _default.getChildNodes();
                  for (int u = 0; u < defaultValues.getLength(); u++) {
                    String value = defaultValues.item(u).getNodeValue().trim();
                    if ((value != null) &&
                        (!"".equalsIgnoreCase(value))) {
                      defaultAttrValues.add(value);
                    }
                  }
                }
              }
              if (defaultAttrValues.size() > 0) {
                attrs.put(defaultAttrValues);
              }
            }
            putAttributeSchema(tabName, attrName, attrs);
          }
        }
      }
      else if ("tab".equalsIgnoreCase(attrNode.getNodeName())) {
        org.w3c.dom.NamedNodeMap attrValues = attrNode.getAttributes();
        org.w3c.dom.Node attrVal = attrValues.getNamedItem("name");
        if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
          String attrName = attrVal.getNodeValue();
          readUserAttributeSchema(attrNode, attrName);
        }
      }
    }
  }

  private static void readDatabaseUrls(org.w3c.dom.Node urlNode) {
    org.w3c.dom.NodeList attrsListNodes = urlNode.getChildNodes();
    for (int n = 0; n < attrsListNodes.getLength(); n++) {
      org.w3c.dom.Node attrNode = attrsListNodes.item(n);
      if ("databaseurl".equalsIgnoreCase(attrNode.getNodeName())) {
        org.w3c.dom.NamedNodeMap attrValues = attrNode.getAttributes();
        org.w3c.dom.Node attrVal = attrValues.getNamedItem("name");
        if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
          String databaseurlName = attrVal.getNodeValue();
          attrVal = attrValues.getNamedItem("url");
          if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
            String databaseurlUrl = attrVal.getNodeValue();
            attrVal = attrValues.getNamedItem("driver");
            if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
              String databaseurlDriver = attrVal.getNodeValue();
              javax.naming.directory.Attributes attrs = new javax.naming.directory.BasicAttributes(true);
              attrs.put("name", databaseurlName);
              attrs.put("url", databaseurlUrl);
              attrs.put("driver", databaseurlDriver);
              attrVal = attrValues.getNamedItem("username");
              if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
                attrs.put("username", attrVal.getNodeValue());
              }
              attrVal = attrValues.getNamedItem("password");
              if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
                attrs.put("password", attrVal.getNodeValue());
              }
              databaseurls.put(databaseurlName, attrs);
            }
          }
        }
      }
    }
  }

  private static void readLookups(org.w3c.dom.Node lookupsNode) {
    org.w3c.dom.NodeList attrsListNodes = lookupsNode.getChildNodes();
    for (int n = 0; n < attrsListNodes.getLength(); n++) {
      org.w3c.dom.Node attrNode = attrsListNodes.item(n);
      if ("lookup".equalsIgnoreCase(attrNode.getNodeName())) {
        org.w3c.dom.NamedNodeMap attrValues = attrNode.getAttributes();
        org.w3c.dom.Node attrVal = attrValues.getNamedItem("name");
        if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
          String lookupName = attrVal.getNodeValue();
          attrVal = attrValues.getNamedItem("databaseurlname");
          if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
            String lookupDatabaseurlname = attrVal.getNodeValue();
            attrVal = attrValues.getNamedItem("query");
            if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
              String lookupQuery = attrVal.getNodeValue();
              attrVal = attrValues.getNamedItem("case");
              String case_ = "normal";
              if ((attrVal != null) && (!"".equalsIgnoreCase(attrVal.getNodeValue()))) {
                case_ = attrVal.getNodeValue();
              }
              case_ = case_.trim().toLowerCase();
              javax.naming.directory.Attributes attrs = new javax.naming.directory.BasicAttributes(true);
              attrs.put("name", lookupName);
              attrs.put("databaseurlname", lookupDatabaseurlname);
              attrs.put("query", lookupQuery);
              attrs.put("case", case_);
              lookups.put(lookupName, attrs);
            }
          }
        }
      }
    }
  }

  private static synchronized void readUserSchema() throws Exception {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);
    DocumentBuilder builder = factory.newDocumentBuilder();
    InputStream st = null;
    try {
      st = new FileInputStream(System.getProperty("maca_ca.user.schema"));
      System.out.println("Esquema de usuarios '"+System.getProperty("maca_ca.user.schema")+"' carregado");
    }
    catch (FileNotFoundException exc) {
      System.out.println("Arquivo com esquema de usuarios '"+System.getProperty("maca_ca.user.schema")+"' nao localizado ou inexistente: esquema default carregado");
      st = LoadProperties.class.getResourceAsStream("/templates/esquema_usuario.xml");
    }
    Document doc = builder.parse(st);
    org.w3c.dom.Element rootElem = doc.getDocumentElement();
    if (!"maca_ca".equalsIgnoreCase(rootElem.getNodeName())) {
      throw new Exception("'esquema_usuario.xml' deve ter a 'tag' 'maca_ca' como elemento raiz");
    }
    org.w3c.dom.Node schemaNode = getFirstNodeByName(rootElem, "schema");
    if (schemaNode == null) {
      throw new Exception("'esquema_usuario.xml' deve ter o elemento 'schema' imediatamente abaixo do elemento raiz");
    }
    org.w3c.dom.Node userNode = getFirstNodeByName(schemaNode, "user");
    if (userNode == null) {
      throw new Exception("'esquema_usuario.xml' deve ter o elemento 'user' abaixo do elemento 'schema'");
    }
    org.w3c.dom.Node ldapEntryNode = getFirstNodeByName(userNode, "ldapentry");
    if (ldapEntryNode == null) {
      throw new Exception("'esquema_usuario.xml' deve ter o elemento 'ldapentry' abaixo do elemento 'user'");
    }
    readUserAttributeSchema(ldapEntryNode, "");
    org.w3c.dom.Node urlsNode = getFirstNodeByName(rootElem, "urls");
    if (urlsNode != null) {
       readDatabaseUrls(urlsNode);
    }
    org.w3c.dom.Node lookupsNode = getFirstNodeByName(rootElem, "lookups");
    if (urlsNode != null) {
       readLookups(lookupsNode);
    }
  }

  public static synchronized void init() {
    //Usuado para inicialiar a clusula static
  }

  public static synchronized Attributes getAttributeSchema(String attrName) {
    attrName = attrName.toLowerCase();
    int index = attrsKeys.indexOf(attrName);
    if (index >= 0) {
      return (Attributes) attrs.get(index);
    }
    return null;
  }

  public static synchronized Attribute getDefaultObjectClassAttribute() {
    Attribute oc = new BasicAttribute("objectclass");
    Attributes ocSchema = getAttributeSchema("objectclass");
    if (ocSchema != null) {
      Attribute attr = ocSchema.get("default");
      if (attr != null) {
        try {
          javax.naming.NamingEnumeration enum = attr.getAll();
          while (enum.hasMoreElements()) {
            Object value = enum.nextElement();
            if (value != null) {
              oc.add(value);
            }
          }
        }
        catch (javax.naming.NamingException exc) {}
      }
    }
    return oc;
  }

  public static synchronized Attribute[] getDefaultAttributes() {
    Vector defAttrs = new Vector();
    for (int i = 0; i < attrs.size(); i++) {
      Attributes attrSchema = (Attributes) attrs.get(i);
      if ((attrSchema != null) && (!"objectclass".equalsIgnoreCase((String)attrsKeys.get(i)))) {
        Attribute attr = attrSchema.get("default");
        if (attr != null) {
          Attribute newAttr = new BasicAttribute((String) attrsKeys.get(i));
          try {
            javax.naming.NamingEnumeration enum = attr.getAll();
            while (enum.hasMoreElements()) {
              Object value = enum.nextElement();
              if (value != null) {
                newAttr.add(value);
              }
            }
          }
          catch (javax.naming.NamingException exc) {}
          defAttrs.add(newAttr);
        }
      }
    }
    Attribute[] defaults = new Attribute[defAttrs.size()];
    for (int i = 0; i < defAttrs.size(); i++) defaults[i] = (Attribute) defAttrs.get(i);
    return defaults;
  }

  public static void setCharCase(Attributes attrs) throws javax.naming.NamingException {
    if (attrs != null) {
      Enumeration enum = attrs.getAll();
      while (enum.hasMoreElements()) {
        javax.naming.directory.Attribute attr = (javax.naming.directory.Attribute) enum.nextElement();
        if ((attr != null) && (!attr.contains(null))) {
          String case_ = UserSchema.getAttributeParameter(attr.getID(), "case");
          if ("upper".equalsIgnoreCase(case_)) {
            Attribute oldAttr = (Attribute) attr.clone();
            attr.clear();
            Enumeration values = oldAttr.getAll();
            while (values.hasMoreElements()) {
              attr.add(((String)values.nextElement()).toUpperCase());
            }
          }
          else if ("lower".equalsIgnoreCase(case_)) {
            javax.naming.directory.Attribute oldAttr = (javax.naming.directory.Attribute) attr.clone();
            attr.clear();
            Enumeration values = oldAttr.getAll();
            while (values.hasMoreElements()) {
              attr.add(((String)values.nextElement()).toLowerCase());
            }
          }
          else if ("firstupper".equalsIgnoreCase(case_)) {
            javax.naming.directory.Attribute oldAttr = (javax.naming.directory.Attribute) attr.clone();
            attr.clear();
            Enumeration values = oldAttr.getAll();
            while (values.hasMoreElements()) {
              attr.add(formatNamesToLowerCase(((String)values.nextElement())));
            }
          }
        }
      }
    }
  }

  public static void processReplacementRegularExpression(Attributes attrs) throws Exception {
    if (attrs != null) {
      Enumeration enum = attrs.getAll();
      while (enum.hasMoreElements()) {
        javax.naming.directory.Attribute attr = (javax.naming.directory.Attribute) enum.nextElement();
        if ((attr != null) && (!attr.contains(null))) {
          String regex = UserSchema.getAttributeParameter(attr.getID(), "replacementregex");
          if (!"".equalsIgnoreCase(regex)) {
            String repValue = UserSchema.getAttributeParameter(attr.getID(), "replacementvalue");
            String repType = UserSchema.getAttributeParameter(attr.getID(), "replacementtype");
            javax.naming.directory.Attribute newAttr = new BasicAttribute(attr.getID());
            javax.naming.NamingEnumeration    values = attr.getAll();
            while (values.hasMoreElements()) {
              Object attrVal = values.nextElement();
              if (attrVal instanceof String) {
                if ("first".equalsIgnoreCase(repType)) {
                  newAttr.add(((String)attrVal).replaceFirst(regex, repValue));
                }
                else {
                  newAttr.add(((String)attrVal).replaceAll(regex, repValue));
                }
              }
            }
            attrs.put(newAttr);
          }
        }
      }
    }
  }

  public static synchronized String getAttributeParameter(String attrName, String parameter) {
    try {
      attrName  = attrName.toLowerCase();
      parameter = parameter.toLowerCase();
      Attributes attrSchema = getAttributeSchema(attrName);
      if (attrSchema != null) {
        Attribute attr = attrSchema.get(parameter);
        if (attr != null) {
          String value = "";
          javax.naming.NamingEnumeration enum = attr.getAll();
          while (enum.hasMoreElements()) {
            value = value + enum.nextElement() + (enum.hasMoreElements()?",":"");
          }
          return value;
        }
      }
    }
    catch (javax.naming.NamingException exc) {
    }
    return "";
  }

  public static synchronized String[] getRequiredAttributesName()  {
    Vector reqNames = new Vector(attrs.size());
    for (int i = 0; i < attrs.size(); i++) {
      Attributes attrSchema = (Attributes) attrs.get(i);
      if (attrSchema != null) {
        Attribute reqAttr = attrSchema.get("required");
        try {
          if ((reqAttr != null) &&
              "true".equalsIgnoreCase((String) reqAttr.get())) {
            reqNames.add((String) attrsKeys.get(i));
          }
        }
        catch (javax.naming.NamingException exc) {
        }
      }
    }
    String[] names = new String[reqNames.size()];
    for (int i = 0; i < reqNames.size(); i++) names[i] = (String) reqNames.get(i);
    return names;
  }

  public static synchronized String[] getSameValueofAttributesName()  {
    Vector samevalueofNames = new Vector(attrs.size());
    for (int i = 0; i < attrs.size(); i++) {
      Attributes attrSchema = (Attributes) attrs.get(i);
      if (attrSchema != null) {
        Attribute samevalueofAttr = attrSchema.get("samevalueof");
          if (samevalueofAttr != null) {
            samevalueofNames.add((String) attrsKeys.get(i));
          }
      }
    }
    String[] names = new String[samevalueofNames.size()];
    for (int i = 0; i < samevalueofNames.size(); i++) names[i] = ((String) samevalueofNames.get(i)).trim().toLowerCase();
    return names;
  }

  public static synchronized String[] getLookupsAttributesName(boolean checked)  {
    Vector lookupNames = new Vector(attrs.size());
    for (int i = 0; i < attrs.size(); i++) {
      Attributes attrSchema = (Attributes) attrs.get(i);
      if (attrSchema != null) {
        Attribute reqAttr = attrSchema.get("lookupname");
        try {
          if ((reqAttr != null) &&
              (lookups.get((String) reqAttr.get()) != null)) {
            if (checked) {
              Attribute lookupcheckedAttr = attrSchema.get("lookupchecked");
              if ((lookupcheckedAttr != null) &&
                  "true".equalsIgnoreCase((String) lookupcheckedAttr.get())) {
                lookupNames.add((String) attrsKeys.get(i));
              }
            }
            else {
              lookupNames.add((String) attrsKeys.get(i));
            }
          }
        }
        catch (javax.naming.NamingException exc) {
        }
      }
    }
    String[] names = new String[lookupNames.size()];
    for (int i = 0; i < lookupNames.size(); i++) names[i] = (String) lookupNames.get(i);
    return names;
  }

  public static synchronized Attributes[] getTabbedAttributesSchema(String tabName) {
    tabName = tabName.toLowerCase();
    Attributes[] tabbedAttrs = new Attributes[0];
    int index = tabKeys.indexOf(tabName);
    if (index >= 0) {
      Vector attrList = (Vector) tabAttrs.get(index);
      tabbedAttrs = new Attributes[attrList.size()];
      for (int i = 0; i < attrList.size(); i++) {
        tabbedAttrs[i] = (Attributes) attrList.get(i);
      }
    }
    return tabbedAttrs;
  }


  public static synchronized String[] getAttributesNames() {
    String[] names = new String[attrsKeys.size()];
    for (int i = 0; i < attrsKeys.size(); i++) {
      names[i] = (String) attrsKeys.get(i);
    }
    return names;
  }

  public static synchronized String getAttributeTabName(String attrName) {
    attrName = attrName.toLowerCase();
    int index = attrsKeys.indexOf(attrName);
    if (index >= 0) {
      Attributes attrSchema = (Attributes) attrs.get(index);
      for (int i = 0; i < tabAttrs.size(); i++) {
        Vector attrsList = (Vector) tabAttrs.get(i);
        index = attrsList.indexOf(attrSchema);
        if (index >= 0) {
          return (String) tabKeys.get(i);
        }
      }
    }
    return "";
  }


  public static synchronized String[] getTabNames() {
    String[] names = new String[tabKeys.size()];
    for (int i = 0; i < tabKeys.size(); i++) {
      names[i] = (String) tabKeys.get(i);
    }
    return names;
  }

  public static synchronized int getTabSize() {
    return tabKeys.size();
  }

  public static synchronized boolean isMultivalue(String attrName) {
    attrName = attrName.toLowerCase();
    int index = attrsKeys.indexOf(attrName);
    if (index >= 0) {
      Attributes attrSchema = (Attributes) attrs.get(index);
      if (attrSchema != null) {
        Attribute multiValueAttr = attrSchema.get("multivalue");
        try {
          if ((multiValueAttr != null) &&
              "true".equalsIgnoreCase((String) multiValueAttr.get())) {
            return true;
          }
        }
        catch (javax.naming.NamingException exc) {
        }
      }
    }
   return false;
  }

  public static synchronized boolean areAttributesConsistent(Attributes actualAtributes) throws javax.naming.NamingException {
    Enumeration keys = attrsAllowedBy.keys();
    while (keys.hasMoreElements()) {
      String    attrName   = (String) keys.nextElement();
      Attribute actualAttr = actualAtributes.get(attrName);
      String    ocName     = (String) attrsAllowedBy.get(attrName);
      if ((ocName != null) && (actualAttr != null)) {
        Vector reqAttrs = (Vector) ocReqAttrs.get(ocName);
        if (reqAttrs != null) {
          Vector reqAttrsNames = new Vector();
          for (int k = 0; k < reqAttrs.size(); k++) {
            actualAttr = (Attribute) actualAtributes.get((String) reqAttrs.get(k));
            if (actualAttr == null) {
              String displayName = UserSchema.getAttributeParameter((String) reqAttrs.get(k), "displayName");
              if ("".equalsIgnoreCase(displayName)) displayName = (String) reqAttrs.get(k);
              reqAttrsNames.add(displayName);
            }
          }
          if (reqAttrsNames.size() > 0) {
            String displayName = UserSchema.getAttributeParameter(attrName, "displayName");
            if ("".equalsIgnoreCase(displayName)) displayName = attrName;
            throw new javax.naming.NamingException("Para preencher o campo '"+displayName+"' voc tambm deve preencher "+
                                                   (reqAttrsNames.size() == 1?"o seguinte campo: "+reqAttrsNames.lastElement():"os seguintes campos: "+reqAttrsNames));
          }
        }
      }
    }
    return true;
  }


  public static synchronized boolean isObjectClassConsistent(String ocName, Attributes actualAtributes) throws javax.naming.NamingException {
    ocName = ocName.toLowerCase();
    Vector reqAttrs = (Vector) ocReqAttrs.get(ocName);
    if (reqAttrs != null) {
       for (int i = 0; i < reqAttrs.size(); i++) {
         Attribute actualAttr = (Attribute) actualAtributes.get((String) reqAttrs.get(i));
         if (actualAttr == null) {
           String displayName = UserSchema.getAttributeParameter((String) reqAttrs.get(i), "displayName");
           if ("".equalsIgnoreCase(displayName)) displayName = (String) reqAttrs.get(i);
           throw new javax.naming.NamingException("O campo '"+displayName+"' deve ser preenchido");
         }
       }
    }
    return true;
  }

  public static synchronized Vector getAttrsNames() {
    return (Vector) attrsKeys.clone();
  }

  public static TableModel getLookup(String lookupname) throws Exception {
    return getLookup(lookupname, "");
  }

  public static TableModel getLookup(String lookupname, String whereclause) throws Exception {
    javax.naming.directory.Attributes attrs = (javax.naming.directory.Attributes) lookups.get(lookupname);
    if (attrs != null) {
      javax.naming.directory.Attribute urlName = attrs.get("databaseurlname");
      if ((urlName != null) && (urlName.get() != null)) {
        Connection conn = getConnection((String) urlName.get());
        try {
          if (conn != null) {
            javax.naming.directory.Attribute query = attrs.get("query");
            if ((query != null) && (query.get() != null)) {
              java.sql.Statement stmt = conn.createStatement();
              String       finalQuery = (String) query.get();
              if (!"".equalsIgnoreCase(whereclause)) {
                javax.naming.directory.Attribute case_ = attrs.get("case");
                if (case_ != null) {
                  if ("upper".equalsIgnoreCase((String)case_.get())) {
                    whereclause = whereclause.toUpperCase();
                  }
                  else if ("lower".equalsIgnoreCase((String)case_.get())) {
                    whereclause = whereclause.toLowerCase();
                  }
                }
                finalQuery = "select * "+
                             "from ("+ finalQuery + ") "+
                             "where "+whereclause;
              }
              java.sql.ResultSet result = stmt.executeQuery(finalQuery);
              java.sql.ResultSetMetaData metadata = result.getMetaData();
              Vector rows = new Vector(), colunms = new Vector();
              for (int n = 1; n <= metadata.getColumnCount(); n++) {
                try {
                  colunms.add(metadata.getColumnLabel(n));
                }
                catch (java.sql.SQLException exc) {
                  colunms.add(Integer.toString(n));
                }
              }
              while (result.next()) {
                Vector row = new Vector();
                for (int n = 1; n <= metadata.getColumnCount(); n++) {
                  try {
                    row.add(result.getString(n));
                  }
                  catch (java.sql.SQLException exc) {}
                }
                rows.add(row);
              }
              TableModel table = new DefaultTableModel(rows, colunms);
              return table;
            }
          }
        }
        finally {
          releaseConnection(conn);
        }
      }
    }
    return new DefaultTableModel();
  }

}