package maca_ad;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import maca_ca.*;
import java.awt.*;
import java.util.*;
import tablesorter.*;
import javax.swing.*;
import javax.swing.table.*;


public class ConflictingAuthorizationsTableModel extends AbstractTableModel {

  private int[] _colsWidth = {0, 0, 0, 0, 0, 0};
  private static final String[] _columnNames = {"Papel", "Recurso", "Privilgio", "Permisso", "TA", "Status"};
  private static final ImageIcon authoOn  = new ImageIcon(maca_ad.ConflictingAuthorizationsTableModel.class.getResource("green-on.gif"));
  private static final ImageIcon authoOff = new ImageIcon(maca_ad.ConflictingAuthorizationsTableModel.class.getResource("red-on.gif"));
  private static Color lightGreen = new Color(0, 50, 0, 50);

  private Vector _authos = new Vector();
  private JTable _table  = null;

  class RowRenderer extends DefaultTableCellRenderer {
      public Component getTableCellRendererComponent(JTable table,
                                                     Object value, boolean isSelected,
                                                     boolean hasFocus,
                                                     int row, int column) {
        int realRow = (_table.getModel() instanceof TableSorter)?
                       ((TableSorter)_table.getModel()).indexes[row] : row;
        if(row % 2 == 0) {
          this.setBackground(lightGreen);
        }
        else {
          setBackground(Color.white);
        }
        if (column == 3) {
          if (((row >= 0) && getValueAt(realRow, column).equals("+"))) {
            this.setToolTipText("CONCEDIDA");
          }
          else if (((row >= 0)  && getValueAt(realRow, column).equals("-"))) {
            this.setToolTipText("NEGADA");
          }
          else if (row >= 0) {
            Authorization autho = (Authorization) _authos.get(realRow);
            try {
              this.setToolTipText(autho.getPrivilegeType());
            }
            catch (Exception exc) {
              this.setToolTipText(exc.getMessage());
            }
          }
        }
        if ((row >= 0) && (column == 5)) {
          this.setIcon((ImageIcon)getValueAt(realRow, column));
          Authorization autho = (Authorization) _authos.get(realRow);
          try {
            if (autho.getStatus() == autho.ACTIVE) {
              this.setToolTipText("ATIVA NO PAPEL");
            }
            else {
              this.setToolTipText("DESATIVADA NO PAPEL");
            }
          }
          catch (Exception exc) {
              this.setToolTipText(exc.getMessage());
          }
        }
        if (column == 4) {
          this.setToolTipText("Tipo Autorizao");
        }
        return super.getTableCellRendererComponent(table,
                                                    value, isSelected, hasFocus,
                                                    row, column);
      }
  }

  private void adjustColumnsWidth() throws Exception {
    FontMetrics fm =  this._table.getFontMetrics(this._table.getFont());
    for (int n = 0; n < this._colsWidth.length; n++) this._colsWidth[n] = fm.stringWidth(this._columnNames[n]);
    this._colsWidth[3] = Math.max(fm.stringWidth(this._columnNames[3]), fm.stringWidth("+"));
    this._colsWidth[4] = Math.max(fm.stringWidth(this._columnNames[4]), Math.max(fm.stringWidth("Fraca"), fm.stringWidth("Forte")));
    this._colsWidth[5] = Math.max(authoOn.getIconWidth(), authoOff.getIconWidth());
    this._table.setRowHeight(Math.max(authoOn.getIconHeight(), authoOff.getIconHeight()));
    for (int n = 0; n < _authos.size(); n++) {
      Authorization autho = (Authorization) _authos.get(n);
      this._colsWidth[0] = Math.max(this._colsWidth[0], fm.stringWidth(autho.getRole().getName()));
      this._colsWidth[1] = Math.max(this._colsWidth[1], fm.stringWidth(autho.ldapResourceDNToResourceName(true)));
      this._colsWidth[2] = Math.max(this._colsWidth[2], fm.stringWidth(autho.getPrivilege()));
    }
    for (int n = 0; n < this.getColumnCount(); n++) {
        TableColumn column = this._table.getColumn(this._table.getColumnName(n));
        column.setMinWidth(Math.round((float)1.1*(float)this._colsWidth[n]));
    }
    this._table.doLayout();
  }



  public ConflictingAuthorizationsTableModel(Vector authos, JTable aTable) throws Exception {
    this._table  = aTable;
    this._authos = authos;
    this.authoOff.setDescription("");
    this.authoOn.setDescription("");
    this._table.setModel(new TableSorter(this));
    ((TableSorter)this._table.getModel()).addMouseListenerToHeaderInTable(this._table);
    for (int n = 0; n < this.getColumnCount(); n++) {
      TableColumn col = this._table.getColumn(this._table.getColumnName(n));
      RowRenderer rowRender = new RowRenderer();
      col.setCellRenderer(rowRender);
      if ((n == 2) || (n == 3) || (n == 4) || (n == 5)) {
         rowRender.setHorizontalAlignment(SwingConstants.CENTER);
      }
    }
    this.setAuthorizations(authos);
  }

  public void clear() throws Exception {
    this._authos.clear();
    this.adjustColumnsWidth();
    this.fireTableDataChanged();
  }

  public void setAuthorizations(Vector authos) throws Exception {
    this._authos = authos;
    this.adjustColumnsWidth();
    this.fireTableDataChanged();
  }



  public int getRowCount() {
    return _authos.size();
  }

  public int getColumnCount() {
    return _columnNames.length;
  }

  public Class getColumnClass(int c) {
    return getValueAt(0, c).getClass();
  }

  public String getColumnName(int col) {
    return _columnNames[col];
  }



  public Object getValueAt(int row, int col) {
    try {
      Authorization autho = (Authorization) _authos.get(row);
      switch (col) {
        case 0 : return autho.getRoleName();                   // Papeis
        case 1 : return autho.ldapResourceDNToResourceName(true); // Recurso
        case 2 : return autho.getPrivilege();                  // Privilgios
        case 3 : return (autho.isPositive()?"+":(autho.isNegative()?"-":"Regra"));          // Tipo privilgios
        case 4 : return (autho.isStrong()?"Forte":"Fraca");    // Tipo autorizao
        case 5 : return (autho.getStatus().equalsIgnoreCase(Authorization.ACTIVE)?authoOn:authoOff); // Tipo autorizao
      }
    }
    catch (Exception exc) {
      System.out.println(exc);
    }
    return "";
  }



}