package maca_ad;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import maca_ca.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.naming.*;
import java.awt.event.*;
import java.awt.event.ActionListener;
import javax.swing.event.*;
import javax.swing.border.*;

public class ResourceLookupComboBoxModel extends DefaultComboBoxModel {

  private Group  _resGroup    = null;
  private JComboBox _cb       = null;
  private ComboBoxEditor _cbe = null;
  private JTextField _tf      = null;

  private void updateItems() throws Exception {
    if (this._resGroup != null) {
        this.removeAllElements();
        Enumeration enum = _resGroup.children();
        while (enum.hasMoreElements()) {
          Group gp = (Group) enum.nextElement();
          this.addElement(gp.ldapResourceDNToResourceName(true));
        }
        if (_resGroup.isRoot()) {
          this._tf.setText("");
        }
        else {
          this._tf.setText(_resGroup.ldapResourceDNToResourceName(true));
        }
    }
  }

  protected void comboBoxEditorActionPerformed(ActionEvent e) throws Exception {
    Group gp = (Group) this._resGroup.getChildAt(this.getIndexOf(this.getSelectedItem()));
    if ((gp != null) && (!gp.isRoot())) {
      _resGroup = gp;
      this.updateItems();
    }
  }

  private void textFieldKeyReleased(KeyEvent e) throws Exception {
    if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
      Group gp = (Group) this._resGroup.getParent();
      if ((gp != null) && (!gp.isRoot())) {
        this._resGroup = gp;
        this.updateItems();
      }
    }
    else {
      if (_resGroup.isRoot()) {
        this._tf.setText("");
      }
      else {
        this._tf.setText(_resGroup.ldapResourceDNToResourceName(true));
      }
    }
  }

  private void setCurrentResource(String res, boolean reverse) throws Exception  {
    if (!this._resGroup.getGroupDN().equalsIgnoreCase(_resGroup.resourceNameToLDAPresourceDN(res, reverse))) {
      Group gp = new Group(GlobalDefinitions.ctx, this._resGroup.getGroupRoot(),
                            this._resGroup.resourceNameToLDAPresourceRDN(res, reverse));
      if ((gp != null) && (!gp.isRoot())) {
        this._resGroup = gp;
        this.updateItems();
      }
    }
  }

  public String getSelectedResourceName() throws Exception {
    return (this._resGroup.isRoot()? "" : this._resGroup.ldapResourceDNToResourceName(true));
  }

  public Group getSelectedResource() throws NamingException {
    return (this._resGroup.isRoot()? null : this._resGroup);
  }

  public void setSelectedResourceName(String resourceName) throws Exception {
      this.setCurrentResource(resourceName, true);
  }

  public ResourceLookupComboBoxModel(JComboBox aComboBox, Group aResGroup) throws Exception {
    this._resGroup = aResGroup;
    if (this._resGroup.isRoot()) {
      Enumeration enum = this._resGroup.children();
      if (enum.hasMoreElements()) {
        this._resGroup = (Group) enum.nextElement();
      }
    }
    this._cb       = aComboBox;
    this._cb.setModel(this);
    this._cb.setEditable(true);
    this._cbe      = this._cb.getEditor();
    this._tf       = (JTextField) this._cbe.getEditorComponent();
    this._tf.setEditable(false);
    this._cbe.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          comboBoxEditorActionPerformed(e);
        }
        catch (Exception exc) {
          JOptionPane.showMessageDialog(null, exc.getMessage(),
                                        "Erro na lendo o recurso",
                                        JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    this._tf.addKeyListener(new java.awt.event.KeyAdapter() {
      public void keyReleased(KeyEvent e) {
        try {
          textFieldKeyReleased(e);
        }
        catch (Exception exc) {
          JOptionPane.showMessageDialog(null, exc.getMessage(),
                                        "Erro lendo o recurso",
                                        JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    this.updateItems();
  }

  public JComboBox getComboBox() {
    return this._cb;
  }

}
