package maca_ad;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

public class SimpleAuthenticatorCS {

  private static final int PRIVILEGE_ATTRIBUTE_CREDENTIALS = 10;

  private org.omg.maca_cs.SecurityLevel2.Credentials creds = null;
  private org.omg.maca_cs.DfResourceAccessDecision.AccessDecision ado = null;
  private org.omg.maca_cs.Security.AttributeType[] attrTypeList = null;
  private org.omg.CORBA.ORB orb = null;
  private String paIOR = "", adoIOR = "";
  private String userID = "";

  private String changePassword(String passwordStr, String newPasswordStr, String newPasswordAgainStr) {
    String result = "Obrigatrio o preenchimento de todos os campos!";
    try {
      if ((creds != null) && (passwordStr != null) && (newPasswordStr != null) && (newPasswordAgainStr != null)) {
        if ((!newPasswordStr.equalsIgnoreCase("")) &&
            (newPasswordStr.equals(newPasswordAgainStr))) {
          if (!passwordStr.equals(newPasswordStr)) {
            if (!userID.equalsIgnoreCase(newPasswordStr)) {
              org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
              org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, PRIVILEGE_ATTRIBUTE_CREDENTIALS);
              org.omg.maca_cs.Security.SecAttribute[] reqstedPriv = {new org.omg.maca_cs.Security.SecAttribute(attrType, new byte [0], (passwordStr+"\n"+newPasswordStr).getBytes())};
              org.omg.maca_cs.Security.AttributeListHolder actualPriv = new org.omg.maca_cs.Security.AttributeListHolder();
              if (creds.set_privileges(true, reqstedPriv, actualPriv)) {
                result = "";
              }
              else {
                for (int n = 0; n < actualPriv.value.length; n++) {
                  if ((actualPriv.value[n].attribute_type.attribute_family.family_definer == 0) &&
                      (actualPriv.value[n].attribute_type.attribute_family.family == 1) &&
                      (actualPriv.value[n].attribute_type.attribute_type == PRIVILEGE_ATTRIBUTE_CREDENTIALS)) {
                    result = new String(actualPriv.value[n].value);
                  }
                }
              }
            }
            else {
              result = "Nova senha deve ser diferente do nome de login do usurio!";
            }
          }
          else {
            result = "Nova senha deve ser diferente da atual!";
          }
        }
        else if (newPasswordStr.length() > 0) {
          result = "Nova senha e confirmao da nova senha devem ser iguais!";
        }
        else {
          result = "Nova senha no pode ficar em branco!";
        }
      }
    }
    catch (Exception exc) {
      System.out.println(exc.toString());
    }
    return result;
  }

  private void changePassword(JPasswordField    password) {
   javax.swing.JTextField curPwdField = new JPasswordField(30);
   javax.swing.JTextField newPwdField = new JPasswordField(30);
   javax.swing.JTextField newPwdFieldAgain = new JPasswordField(30);
   Object comp[] = new Object[6];
   comp[0] = "Senha atual: ";
   comp[1] = curPwdField;
   comp[2] = "Nova senha: ";
   comp[3] = newPwdField;
   comp[4] = "Repetico da nova senha: ";
   comp[5] = newPwdFieldAgain;
   Object options[] = { "OK", "Cancela" };
   boolean done = false;
   while (!done) {
     int option = JOptionPane.showOptionDialog(null, ((Object) (comp)), "Troca de senha obrigatria", -1, 3, null, options, options[0]);
     if (option == 0) {
       String result = this.changePassword(curPwdField.getText(), newPwdField.getText(), newPwdFieldAgain.getText());
       if ("".equals(result)) {
         result = "Senha alterada com sucesso!";
         password.setText(newPwdField.getText());
         done = true;
       }
       JOptionPane.showMessageDialog(null, result,
                                     "Advertncia",
                                      JOptionPane.WARNING_MESSAGE);
     }
     else {
       CloseSession();
       System.exit(0);
     }
   }

  }

 private void readIORs() throws Exception {
   URL iorURL = new URL(System.getProperty("maca_ca.maca_cs.url")+"/maca_cs.iors");
   BufferedReader iorStream = new BufferedReader(new InputStreamReader(iorURL.openStream()));
   String line = iorStream.readLine();
   StringTokenizer strTk = new StringTokenizer(line, ";");
   while (strTk.hasMoreTokens()) {
     String ior = strTk.nextToken();
     if (ior.startsWith("paIOR")) {
       paIOR = ior.substring(2);
     }
     else if (ior.startsWith("adoIOR")) {
       adoIOR = ior.substring(3);
     }
   }
   iorStream.close();
 }

  public SimpleAuthenticatorCS() {
    String[] args = null;
    try {
      InputStream st = SimpleAuthenticatorCS.class.getResourceAsStream("/orb.properties");
      if (st != null) {
        System.getProperties().load(st);
        st.close();
        System.out.println("Recurso /orb.properties carregado");
      }
      try {
        st = new FileInputStream("./orb.properties");
        Properties fileProps = new Properties();
        fileProps.load(st);
        System.getProperties().putAll(fileProps);
        st.close();
        System.out.println("Arquivo ./orb.properties carregado");
      }
      catch (java.io.FileNotFoundException exc) {
      }

      orb = org.omg.CORBA.ORB.init(args, System.getProperties());

      org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
      org.omg.maca_cs.Security.AttributeType[] auxAttrType = {new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value),
                                                      new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.AccessId.value)};
      attrTypeList = auxAttrType;

      st = SimpleAuthenticatorCS.class.getResourceAsStream("/maca_ad.properties");
      if (st != null) {
        System.getProperties().load(st);
        st.close();
        System.out.println("Recurso /maca_ad.properties carregado");
      }
      try {
        st = new FileInputStream("./maca_ad.properties");
        Properties fileProps = new Properties();
        fileProps.load(st);
        System.getProperties().putAll(fileProps);
        st.close();
        System.out.println("Arquivo ./maca_ad.properties carregado");
      }
      catch (java.io.FileNotFoundException exc) {
      }
    }
    catch (Exception exc) {
      JOptionPane.showMessageDialog(null, exc.toString(),
                                    "Erro",
                                     JOptionPane.ERROR_MESSAGE);
    }
  }

  public synchronized void CloseSession () {
    if ((creds != null) /*&& creds._is_bound()*/) {
      try {
        creds.destroy();
      }
      catch (org.omg.CORBA.COMM_FAILURE exc) {
        JOptionPane.showMessageDialog(null, "Falha de comunicao com o servidor CORBA",
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
      catch (Exception exc) {
        JOptionPane.showMessageDialog(null, exc.getMessage(),
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
    }
    creds = null;
    ado = null;
  }

  public boolean AccessAllowed (java.lang.String resName,
                                java.lang.String accessMode) {
    if ((creds != null) && (ado != null)) {
      try {
        org.omg.maca_cs.Security.SecAttribute[] attrs = creds.get_attributes(attrTypeList);
        org.omg.maca_cs.DfResourceAccessDecision.ResourceNameComponent[] resNameCompts = {new org.omg.maca_cs.DfResourceAccessDecision.ResourceNameComponent("Aplicacoes", resName)};
        org.omg.maca_cs.DfResourceAccessDecision.ResourceName resourceName = new org.omg.maca_cs.DfResourceAccessDecision.ResourceName("", resNameCompts);
        return ado.access_allowed(resourceName, accessMode, attrs);
     }
      catch (org.omg.maca_cs.DfResourceAccessDecision.InternalError exc) {
        JOptionPane.showMessageDialog(null, exc.getMessage(),
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
      catch (org.omg.CORBA.COMM_FAILURE exc) {
        JOptionPane.showMessageDialog(null, "Falha de comunicao com o servidor CORBA",
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
      catch (Exception exc) {
        JOptionPane.showMessageDialog(null, exc.getMessage(),
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
    }
    return false;
  }

  public String getUserID() {
    return userID;
  }

  public String getUserRole() {
    if ((creds != null) && (ado != null)) {
      try {
        org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
        org.omg.maca_cs.Security.AttributeType[] attrTypeRoleList = {new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value)};
        org.omg.maca_cs.Security.SecAttribute[] attrs = creds.get_attributes(attrTypeRoleList);
        for (int n = 0; n < attrs.length; n++) {
          if ((attrs[n].attribute_type.attribute_family.family_definer == 0) && // OMG - family_definer = 0
              (attrs[n].attribute_type.attribute_family.family == 1) &&         // privilege attributes; family = 1
              (attrs[n].attribute_type.attribute_type == org.omg.maca_cs.Security.Role.value)) {
            return new String(attrs[n].value);
          }
        }
      }
      catch (org.omg.CORBA.COMM_FAILURE exc) {
        JOptionPane.showMessageDialog(null, "Falha de comunicao com o servidor CORBA",
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
      catch (Exception exc) {
        JOptionPane.showMessageDialog(null, exc.getMessage(),
                                      "Erro",
                                       JOptionPane.ERROR_MESSAGE);
      }
    }
    return "";
  }

  public synchronized boolean Authenticate(java.lang.String loginName,
                                           JPasswordField    password,
                                           java.lang.String userRole) {

    org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
    org.omg.maca_cs.Security.AttributeType     attrType = new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value);
    org.omg.maca_cs.Security.SecAttribute[]       attrs = {new org.omg.maca_cs.Security.SecAttribute(attrType, new byte [0], userRole.getBytes())};
    org.omg.maca_cs.SecurityLevel2.CredentialsHolder credHolder = new org.omg.maca_cs.SecurityLevel2.CredentialsHolder();
    org.omg.maca_cs.Security.OpaqueHolder  contData = new org.omg.maca_cs.Security.OpaqueHolder();
    org.omg.maca_cs.Security.OpaqueHolder authEspec = new org.omg.maca_cs.Security.OpaqueHolder();
    org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticator pa = null;
    userID = "";
    try {
      System.out.println("maca_ca.maca_cs.url: "+System.getProperty("maca_ca.maca_cs.url"));
      this.readIORs();
      org.omg.CORBA.Object corbaObj = orb.string_to_object(paIOR);
      pa = org.omg.maca_cs.SecurityLevel2.PrincipalAuthenticatorHelper.narrow(corbaObj);
    }
    catch (java.net.MalformedURLException exc) {
      JOptionPane.showMessageDialog(null, exc.getMessage(),
                                    "Advertncia",
                                     JOptionPane.WARNING_MESSAGE);
    }
    catch (java.io.IOException exc) {
      JOptionPane.showMessageDialog(null, exc.getMessage(),
                                    "Advertncia",
                                     JOptionPane.WARNING_MESSAGE);
    }
    catch (Exception exc) {
      JOptionPane.showMessageDialog(null, exc.getMessage(),
                                    "Erro",
                                     JOptionPane.ERROR_MESSAGE);
    }
    try {
      String authMessage = "";
      org.omg.maca_cs.Security.AuthenticationStatus authStatus = pa.authenticate(0, loginName, new String(password.getPassword()).getBytes(), attrs, credHolder, contData, authEspec);
      if ((pa != null) && (authStatus.value() == org.omg.maca_cs.Security.AuthenticationStatus.SecAuthSuccess.value() ||
                           authStatus.value() == org.omg.maca_cs.Security.AuthenticationStatus.SecAuthContinue.value())) {
        creds = credHolder.value;
        org.omg.CORBA.Object corbaObj = orb.string_to_object(adoIOR);
        ado = org.omg.maca_cs.DfResourceAccessDecision.AccessDecisionHelper.narrow(corbaObj);
        userID = loginName;
        authMessage = new String(authEspec.value);
        if (!"".equals(authMessage)) {
          JOptionPane.showMessageDialog(null, authMessage,
                                        "Advertncia",
                                         JOptionPane.WARNING_MESSAGE);
        }
        if (authStatus.value() == org.omg.maca_cs.Security.AuthenticationStatus.SecAuthContinue.value()) {
          this.changePassword(password);
        }
        return true;
      }
      authMessage = new String(authEspec.value);
      if (!"".equals(authMessage)) {
        JOptionPane.showMessageDialog(null, authMessage,
                                      "Advertncia",
                                       JOptionPane.WARNING_MESSAGE);
      }
    }
    catch (org.omg.CORBA.COMM_FAILURE exc) {
      JOptionPane.showMessageDialog(null, "Falha de comunicao com o servidor CORBA",
                                    "Erro",
                                     JOptionPane.ERROR_MESSAGE);
    }
    catch (Exception exc) {
      JOptionPane.showMessageDialog(null, exc.getMessage(),
                                    "Erro",
                                     JOptionPane.ERROR_MESSAGE);
    }
    return false;
  }

}