package maca_ads;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import maca_ca.*;
import java.io.*;
import java.net.*;
import java.util.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.maca_cs.Security.*;
import org.omg.maca_cs.TimeBase.*;
import org.omg.maca_cs.SecurityLevel2.*;
import org.omg.maca_cs.DfResourceAccessDecision.*;

public class MacaADServer {
  //Variveis estticas privadas
  static private NamingContextExt rootContext = null;

  //ADO - Access Decision Object
  private AccessDecision   ado = null;
  private DirContextHolder ctx = new DirContextHolder(null);


  private javax.naming.directory.InitialDirContext conectaToLDAPServer(String login, String credentials) throws Exception {
    Hashtable env = new Hashtable();
    env.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
    if ("true".equalsIgnoreCase(System.getProperty("maca_ca.ldap.ssl"))) {
      env.put(javax.naming.Context.SECURITY_PROTOCOL, "ssl");
      env.put("java.naming.ldap.factory.socket", System.getProperty("maca_ca.ldap.ssl.factory.socket"));
    }
    env.put(javax.naming.Context.SECURITY_AUTHENTICATION, "simple");
    env.put(javax.naming.Context.SECURITY_PRINCIPAL, "uid="+login+","+System.getProperty("maca_ca.ldap.peopleRDN")+","+System.getProperty("maca_ca.ldap.rootDN"));
    env.put(javax.naming.Context.SECURITY_CREDENTIALS, credentials);
    env.put(javax.naming.Context.PROVIDER_URL,"ldap://"+System.getProperty("maca_ca.ldap.server")+"/");
    return new javax.naming.directory.InitialDirContext(env);
  }

  private boolean hasAccess(org.omg.maca_cs.SecurityLevel2.Credentials creds,
                            String resourceName, String operation) throws Exception {
    if ((ado != null) && (creds != null)) {
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      AttributeType[] attrTypes = {new AttributeType(extFamily, AccessId.value)};
      SecAttribute[] attrs = creds.get_attributes(attrTypes);
      ResourceNameComponent[] resNameCompts = {new ResourceNameComponent("Aplicacoes", resourceName)};
      ResourceName resName = new ResourceName("127.0.0.1", resNameCompts);
      return ado.access_allowed(resName, operation, attrs);
    }
    return false;
  }

  private static synchronized void initProperties(String propertiesPath) throws Exception {
    maca_ca.LoadProperties.init(null);
    maca_ca.LoadProperties.loadPropertiesFile(null, propertiesPath + "/maca_ca.properties");
    maca_ca.LoadProperties.loadPropertiesFile(null, propertiesPath + "/orb.properties");
    maca_ca.UserSchema.init();
  }

  public MacaADServer() throws Exception {
  }

  public MacaADServer(String login, String credentials,
                      org.omg.maca_cs.DfResourceAccessDecision.AccessDecision anADO,
                      String propertiesPath) throws Exception {
    initMacaADServer(login, credentials, anADO, propertiesPath);
  }

  public synchronized void initMacaADServer(String login, String credentials,
                                            org.omg.maca_cs.DfResourceAccessDecision.AccessDecision anADO,
                                            String propertiesPath) throws Exception {
    initProperties(propertiesPath);
    //Inicaliza o servidor de autorizao de acesso "ado"
    ado = anADO;
    //Conecta-se com o servidor LDAP
    if (ctx.value == null) {
      ctx.value = conectaToLDAPServer(login, credentials);
    }
  }

  public String executeRequest (org.omg.maca_cs.SecurityLevel2.Credentials creds, String request) {
    try {
      if (this.hasAccess(creds, "MACA", "execuo")) {
        MacaADServerSaxParser parser = new MacaADServerSaxParser(ado, ctx);
        return parser.parser(creds, request);
      }
      else {
        String str = "";
        str += "<?xml version='1.0' encoding='ISO-8859-1'?>";
        str += "<maca_ads version=\"3.1\">";
        str += "<response action=\"MACA\"";
        str += "          status=\"not_ok\"";
        str += "          errormessage=\"Acesso no autorizado!\">";
        str += "</response>";
        str += "</maca_ads>";
        return str;
      }
    }
    catch (Exception exc) {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      exc.printStackTrace(pw);
      pw.close();
      String str = "";
      str += "<?xml version='1.0' encoding='ISO-8859-1'?>";
      str += "<maca_ads version=\"3.1\">";
      str += "<response action=\"MACA\"";
      str += "          status=\"error\"";
      str += "          errormessage=\""+sw.toString().replace('"', '\'')+"\">";
      str += "</response>";
      str += "</maca_ads>";
      return str;
    }
  }


  public static void main(String[] args) {
    try {
//      System.setProperty("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
//      System.setProperty("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");
      String login = "gustavo.motta";
      String pwd   = "cartola";
      MacaADServer macaADServer = new MacaADServer(login, pwd, null, ".");
      //Inicializa o ORB
      ORB orb = ORB.init(args, System.getProperties());

      String paIOR = "", adoIOR = "";
      PrincipalAuthenticator pa = null;
      AccessDecision        ado = null;
      URL iorURL = new URL(System.getProperty("maca_ca.maca_cs.url")+"/maca_cs.iors");
      BufferedReader iorStream = new BufferedReader(new InputStreamReader(iorURL.openStream()));
      String line = iorStream.readLine();
      StringTokenizer strTk = new StringTokenizer(line, ";");
      while (strTk.hasMoreTokens()) {
        String ior = strTk.nextToken();
        if (ior.startsWith("paIOR")) {
          paIOR = ior.substring("pa".length());
        }
        else if (ior.startsWith("adoIOR")) {
          adoIOR = ior.substring("ado".length());
        }
      }
      iorStream.close();
      org.omg.CORBA.Object corbaObj = orb.string_to_object(paIOR);
      pa = PrincipalAuthenticatorHelper.narrow(corbaObj);
      corbaObj = orb.string_to_object(adoIOR);
      ado = AccessDecisionHelper.narrow(corbaObj);

      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      AttributeType     attrType = new AttributeType(extFamily,  org.omg.maca_cs.Security.Role.value);
      SecAttribute[]       attrs = {new SecAttribute(attrType, new byte [0], "Administrador de Contas".getBytes())};
      CredentialsHolder credHolder = new CredentialsHolder();
      OpaqueHolder  contData = new OpaqueHolder();
      OpaqueHolder authEspec = new OpaqueHolder();
      if (pa.authenticate(0, login, pwd.getBytes(), attrs, credHolder, contData,
                            authEspec).value() == AuthenticationStatus.SecAuthSuccess.value()) {
          BufferedReader in = new BufferedReader(new FileReader("./requests/request_BloquearContaUsuario.xml"));
          String str = "";
          while (true) {
            line = in.readLine();
            if (line != null)
              str += line;
            else
              break;
          }
          macaADServer.initMacaADServer(login, pwd, ado, ".");
          PrintWriter out = new PrintWriter(new BufferedOutputStream(new FileOutputStream("./requests/text.xml", false)), true);
          out.write(macaADServer.executeRequest(credHolder.value, str));
          out.close();
          credHolder.value.destroy();
          macaADServer.ctx.value.close();
        }
        else {
          System.out.println("Falha no login");
        }
    }
    catch (Exception  exc) {
      exc.printStackTrace();
    }
  }
}



