package maca_ads;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import maca_ca.*;
import java.text.*;
import java.util.*;
import org.xml.sax.*;
import javax.naming.*;
import java.util.zip.*;
import javax.xml.parsers.*;
import javax.swing.table.*;
import org.xml.sax.helpers.*;
import org.omg.maca_cs.Security.*;
import org.omg.maca_cs.TimeBase.*;
import org.omg.maca_cs.SecurityLevel2.*;
import org.omg.maca_cs.DfResourceAccessDecision.*;

public class MacaADServerSaxParser extends DefaultHandler {
  //Constantes
  static private final String LINE_END = System.getProperty("line.separator");
  static private final DateFormat   DF = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss", new Locale("pt", "BR"));

  //Variaveis estticas privadas
  static private                  Writer  out = new OutputStreamWriter(System.out);
  static private  SAXParserFactory    factory = SAXParserFactory.newInstance();

  //Variveis privadas
  private String indentString = "    "; // Amount to indent
  private int     indentLevel = 0;
  private Stack  elementStack = new Stack();
  private StringWriter result = new StringWriter();
  private DirContextHolder ctx  = new DirContextHolder(null);
  private Credentials   creds = null;
  private AccessDecision  ado = null;
  private Group userGroup     = null;

  private boolean hasAccess(String resourceName, String operation) throws Exception {
    if ((ado != null) && (creds != null)) {
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      AttributeType[] attrTypes = {new AttributeType(extFamily, AccessId.value)};
      SecAttribute[] attrs = creds.get_attributes(attrTypes);
      ResourceNameComponent[] resNameCompts = {new ResourceNameComponent("Aplicacoes", resourceName)};
      ResourceName resName = new ResourceName("127.0.0.1", resNameCompts);
      return ado.access_allowed(resName, operation, attrs);
    }
    return false;
  }

  private Hashtable getUserAttributes () {
    Hashtable atributos = new Hashtable();
    //Verifica se a credencial foi criada
    if (creds != null) {
      //Define a famlia dos atributos de segurana
      ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
      //Define o tipo de atributo a recuperar, o conjunto de atributos da conta do usurio, neste caso
      AttributeType[]   attrType = {new AttributeType(extFamily, AttributeSet.value)};
      //Recupera o valor do tipo de atributo a partir da credencial
      //Cada elemento da lista retorna um atributo, do tipo string, com o seguinte formato:
      //<nome do atributo>: <valor_1 do atributo>, <valor_2 do atributo>, ..., <valor_n do atributo>
      SecAttribute[]       attrs = creds.get_attributes(attrType);
      //Examina a lista de valores em busca do tipo de atributo requisitado
      for (int n = 0; n < attrs.length; n++) {
        if ((attrs[n].attribute_type.attribute_family.family_definer == 0) &&
            (attrs[n].attribute_type.attribute_family.family == 1) &&
            (attrs[n].attribute_type.attribute_type == AttributeSet.value)) {
          StringTokenizer strTk = new StringTokenizer(new String(attrs[n].value), ":");
          if (strTk.hasMoreTokens()) {
            String       attrName = strTk.nextToken().trim().toLowerCase();
            Vector         values = new Vector();
            while (strTk.hasMoreTokens()) {
              String value = strTk.nextToken(",");
              if (value.startsWith(":")) value = value.substring(1);
              values.add(value.trim());
            }
            atributos.put(attrName, values);
          }
        }
      }
    }
    return atributos;
  }

  private boolean[] hasAccess (String[] nomeRecursos, String[] operacoes) throws Exception {
    if ((creds != null) && (ado != null) && (nomeRecursos.length == operacoes.length)) {
        ExtensibleFamily extFamily = new ExtensibleFamily((short) 0, (short) 1);
        AttributeType[] attrTypes = {new AttributeType(extFamily, AccessId.value)};
        SecAttribute[] attrs      = creds.get_attributes(attrTypes);
        //Define a lista de requisies de autorizao de acesso
        AccessDefinition[] access_requests = new AccessDefinition[nomeRecursos.length];
        //Para cada recurso, monta o componente com o nome do recurso e operao, e o inclui
        //na lista 'access_requests'
        for (int i = 0; i < access_requests.length; i++) {
          ResourceNameComponent[] resNameCompts =
                              {new ResourceNameComponent("Aplicacoes", nomeRecursos[i])};
          ResourceName resName = new ResourceName("127.0.0.1", resNameCompts);
          access_requests[i]   = new AccessDefinition(resName, operacoes[i]);
        };
        //Invoca o mtodo que retorna um array de booleanos, cada booleano correspondendo ao
        //resultado da solicitaao de acesso ao recurso correspondente a mesma posio do array
        return ado.multiple_access_allowed(access_requests, attrs);
    }
    return new boolean[0];
  }

  private class ElementNode {
    String             uri = "";
    String           lName = "";
    String           qName = "";
    Attributes       attrs = new AttributesImpl();

    public ElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      uri   = aUri;
      lName = aLName;
      qName = aQName;
      attrs = new AttributesImpl(anAttrs);
    }
  }

  private class RequestElementNode extends ElementNode {
    String   errorMessages = "";
    String warningMessages = "";
    String        messages = "";
    StringWriter    result = new StringWriter();
    Vector elementsToProcessing = new Vector();

    public RequestElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      super(aUri, aLName, aQName, anAttrs);
    }

    public void AddErrorMessage(String errorMessage) {
      errorMessages = errorMessages +
                      ((errorMessages.equalsIgnoreCase(""))?"":"\n") +
                      errorMessage.replace('"', '\'') + ".";
    }

    public void AddWarningMessage(String warningMessage) {
      warningMessages = warningMessages +
                        ((warningMessages.equalsIgnoreCase(""))?"":"\n") +
                        warningMessage.replace('"', '\'')  + ".";
    }

    public void AddMessage(String message) {
      messages = messages +
                 ((messages.equalsIgnoreCase(""))?"":"\n") +
                 message.replace('"', '\'')  + ".";
    }
  }

  private class LdapEntryElementNode extends ElementNode {
    javax.naming.directory.Attributes ldapattrs = new javax.naming.directory.BasicAttributes(true);
    String dn = "";

    public LdapEntryElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      super(aUri, aLName, aQName, anAttrs);
    }
  }

  private class AttributeElementNode extends ElementNode {
    javax.naming.directory.Attribute ldapattr = null;

    public AttributeElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      super(aUri, aLName, aQName, anAttrs);
    }
  }

  private class RolesElementNode extends ElementNode {
    Vector ldapentries = new Vector();

    public RolesElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      super(aUri, aLName, aQName, anAttrs);
    }
  }

  private class UserElementNode extends ElementNode {
    LdapEntryElementNode ldapentry = null;
    RolesElementNode         roles = null;

    public UserElementNode(String aUri, String aLName, String aQName, Attributes anAttrs) {
      super(aUri, aLName, aQName, anAttrs);
    }
  }

  //Mtodos privados
  private void endRoles() throws Exception {
    try {
      RolesElementNode elem = (RolesElementNode) elementStack.pop();
      if (!elementStack.isEmpty()) {
        ElementNode elemNode = (ElementNode) elementStack.peek();
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          userElemNode.roles = elem;
        }
      }
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void endValue() throws Exception {
    try {
      //Elemento final da rvore (folha) - no desempilhar
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void endAttribute() throws Exception {
    try {
      AttributeElementNode elem = (AttributeElementNode) elementStack.pop();
      if (!elementStack.isEmpty()) {
        ElementNode elemNode = (ElementNode) elementStack.peek();
        if (elemNode instanceof LdapEntryElementNode) {
          LdapEntryElementNode ldapEntryElemNode = (LdapEntryElementNode) elemNode;
          if ((ldapEntryElemNode.ldapattrs != null) && (elem.ldapattr != null)) {
            ldapEntryElemNode.ldapattrs.put(elem.ldapattr);
          }
        }
      }
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void endLdapEntry() throws Exception {
    try {
      LdapEntryElementNode elem = (LdapEntryElementNode) elementStack.pop();
      if (!elementStack.isEmpty()) {
        ElementNode elemNode = (ElementNode) elementStack.peek();
        if (elemNode instanceof RolesElementNode) {
          RolesElementNode rolesElemNode = (RolesElementNode) elemNode;
          if ((rolesElemNode.ldapentries != null) && (elem.ldapattrs != null)) {
            rolesElemNode.ldapentries.add(elem);
          }
        }
        else if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          userElemNode.ldapentry = elem;
        }
      }
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void endUser() throws Exception {
    try {
      UserElementNode elem = (UserElementNode) elementStack.pop();
      if (!elementStack.isEmpty()) {
        ElementNode elemNode = (ElementNode) elementStack.peek();
        if (elemNode instanceof RequestElementNode) {
          RequestElementNode requestElemNode = (RequestElementNode) elemNode;
          requestElemNode.elementsToProcessing.add(elem);
        }
      }
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void endRequest() throws Exception {
    try {
      RequestElementNode elem = (RequestElementNode) elementStack.pop();
      if ((elem != null) && (elem.attrs != null)) {
        result.write("<response ");
        if (elem.attrs.getLength() > 0) {
          String aName = elem.attrs.getLocalName(0); // Attr name
          if ("".equals(aName)) { aName = elem.attrs.getQName(0); }
          if ("action".equals(aName)) {
            String actionName = elem.attrs.getValue(aName);
            result.write("action=\""+actionName+"\"");
            if ("ConsultarUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              if (this.hasAccess(actionName, "execuo")) {
                //Aumenta o valor inicial do buffer, proporcional ao nmero mximo de usurios retornados numa consulta
                elem.result = new StringWriter(3000 *  1024);
                searchUsers(elem);
              }
              else {
                elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
              }
            }
            else if ("PegarSubgrupos;Grupos;MACA".equalsIgnoreCase(actionName)) {
              if (this.hasAccess(actionName, "execuo")) {
                getGroupTree(elem);
              }
              else {
                elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
              }
            }
            else if ("GravarUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              saveUser(elem);
            }
            else if ("NovoUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              addUser(elem);
            }
            else if ("NovoUsuarioTemplate;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              if (this.hasAccess(actionName, "execuo")) {
                getNewUserTemplate(elem);
              }
              else {
                elem.AddErrorMessage("Acesso ao recurso '"+actionName+"' no autorizado");
              }
            }
            else if ("AtribuirPapel;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso realizado internamente
                assignRolesToUser(elem);
            }
            else if ("RemoverPapel;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso realizado internamente
                removeRolesFromUser(elem);
            }
            else if ("RemoverUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              removeUser(elem);
            }
            else if ("BloquearContaUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso realizado internamente
                lockUserAccount(elem);
            }
            else if ("DesbloquearContaUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso realizado internamente
                unlockUserAccount(elem);
            }
            else if ("AlterarLoginUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso realizado internamente
                renameUserID(elem);
            }
            else if ("AtributosConsulta;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso no  realizado
                getUserAttributesWithDisplayName(elem, "maca_ca.ldap.searchableFields");
            }
            else if ("AtributosRetornoConsulta;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso no  realizado
                getUserAttributesWithDisplayName(elem, "maca_ca.ldap.returningFields");
            }
            else if ("PesquisarTabelaValidacao;Usuarios;MACA".equalsIgnoreCase(actionName)) {
                //Controle de acesso no  realizado
                lookupForValidationFields(elem);
            }
            else if ("PegarAtributosPoliticaSenha;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              getUserPasswordPolicyAttributes(elem);
            }
            else if ("AlterarAtributosPoliticaSenha;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              ChangeUserPasswordPolicyAttributes(elem);
            }
            else if ("AlterarSenhaUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              ChangeUserPassword(elem);
            }
            else if ("NovaSenhaUsuario;Usuarios;MACA".equalsIgnoreCase(actionName)) {
              //Controle de acesso realizado internamente
              NewUserPassword(elem);
            }
            else {
              elem.AddErrorMessage("Ao '"+actionName+"' invlida!");
            }
          }
          else {
            elem.AddErrorMessage("'action' deve ser o primeiro parmetro da requisio");
          }
        }
        else {
          elem.AddErrorMessage("Parmetro 'action' obrigatrio numa requisio");
        }
      }
      else {
        throw new SAXException("Erro fatal processando requisio");
      }

      //Trata mensagens geradas duranto o processamento da requisio
      if (!"".equalsIgnoreCase(elem.errorMessages)) {
        result.write(" status=\"not_ok\"" +
                     " errormessage=\""+elem.errorMessages.replace('"', '\'')+"\"");
      }
      else {
        result.write(" status=\"ok\"" +
                     " message=\""+elem.messages.replace('"', '\'')+"\"");
      }
      if (!"".equalsIgnoreCase(elem.warningMessages)) {
        result.write(" warnings=\""+elem.warningMessages.replace('"', '\'')+"\"");
      }

      //Finaliza o incio do elemento 'response' e seus atributos
      result.write(">" + LINE_END);

      //Escreve o contedo da resposta
      result.getBuffer().append(elem.result.getBuffer());

      //Finaliza a resposta e com o fim do elemento 'response
      result.write("</response>" + LINE_END);
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void startRoles() throws Exception {
    try {
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void startValue() throws Exception {
    try {
      //elementStack.peek() refere-se ao elementro superior
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void startAttribute() throws Exception {
    try {
      AttributeElementNode elem = (AttributeElementNode) elementStack.peek();
      if ((elem != null) && (elem.attrs != null)) {
        if (elem.attrs.getLength() > 0) {
          String aName = elem.attrs.getLocalName(0); // Attr name
          if ("".equals(aName)) { aName = elem.attrs.getQName(0); }
          if ("name".equals(aName)) {
            String name = elem.attrs.getValue(aName);
            if (name != null) {
              elem.ldapattr = new javax.naming.directory.BasicAttribute(name);
            }
          }
        }
      }
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void startLdapEntry() throws Exception {
    try {
      LdapEntryElementNode elem = (LdapEntryElementNode) elementStack.peek();
      if ( (elem != null) && (elem.attrs != null)) {
        if (elem.attrs.getLength() > 0) {
          String aName = elem.attrs.getLocalName(0); // Attr name
          if ("".equals(aName)) { aName = elem.attrs.getQName(0); }
          if ("dn".equals(aName)) {
            String dn = elem.attrs.getValue(aName);
            if (dn != null) {
              elem.dn = dn;
            }
          }
        }
      }
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void startUser() throws Exception {
    try {
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private void startRequest() throws Exception {
    try {
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  private boolean checkUserIDwithUserDN(String uid, String dn) {
    String dnStr = dn.trim().toLowerCase();
    if (dn.startsWith("uid="+uid.trim().toLowerCase()+",") &&
        dn.endsWith(System.getProperty("maca_ca.ldap.peopleRDN")+","+
                    System.getProperty("maca_ca.ldap.rootDN"))) {
      return true;
    }
    return false;
  }

  private void getUserAttributesWithDisplayName(RequestElementNode elem, String propertyName) {
    try {
      StringTokenizer strTk = new StringTokenizer(System.getProperty(propertyName), ",");
      while (strTk.hasMoreTokens()) {
        String attrName = strTk.nextToken().trim();
        String displayStr = attrName;
        elem.result.write("<attribute name=\""+attrName+"\" ");
        javax.naming.directory.Attributes attrSchema = maca_ca.UserSchema.getAttributeSchema(attrName);
        if (attrSchema != null) {
          javax.naming.directory.Attribute displayAttr = attrSchema.get("displayname");
          if ((displayAttr != null) && (displayAttr.get() != null)) {
            displayStr = (String) displayAttr.get();
          }
          Enumeration enum = attrSchema.getAll();
          while (enum.hasMoreElements()) {
            javax.naming.directory.Attribute attr = (javax.naming.directory.Attribute) enum.nextElement();
            if ((attr != null) && (attr.get() != null) &&
                (!attr.getID().equalsIgnoreCase("name")) &&
                (!attr.getID().equalsIgnoreCase("displayname"))) {
              elem.result.write(attr.getID()+"=\""+attr.get()+"\" ");
            }
          }
        }
        elem.result.write("displayname=\""+displayStr+"\"></attribute>" + LINE_END);
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void renameUserID(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String newUserID = (String) uidAttr.get();
                if ((newUserID != null) && (!"".equalsIgnoreCase(newUserID))) {
                  String userID = userLdapEntryNode.dn.substring(userLdapEntryNode.dn.indexOf("=") + 1, userLdapEntryNode.dn.indexOf(","));
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("AlterarLoginUsuario;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        user.changeLoginName(newUserID);
                        elem.AddMessage("Login '"+userID+"' foi alterado para '"+newUserID+"' com sucesso");
                      }
                      else {
                        elem.AddMessage("Alterao do login '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' no est cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Usurio para alterar o login no especificado");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void removeRolesFromUser(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      RolesElementNode rolesElemNode = (RolesElementNode) userElemNode.roles;
                      if (rolesElemNode != null) {
                        for (int k = 0; k < rolesElemNode.ldapentries.size(); k++) {
                          LdapEntryElementNode ldapEntryNode = (LdapEntryElementNode) rolesElemNode.ldapentries.get(k);
                          if (ldapEntryNode != null) {
                            maca_ca.Role role = new maca_ca.Role(ctx,
                                                                 System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                                                 System.getProperty("maca_ca.ldap.rootDN"),
                                                                 ldapEntryNode.dn.substring(0, ldapEntryNode.dn.indexOf(System.getProperty("maca_ca.ldap.groupsRDN")) - 1));
                            if (this.hasAccess("RemoverPapel;Usuarios;MACA(\""+user.getUserID()+"\",\""+role.getGroupDN()+"\")", "execuo")) {
                              if (role.removeMemberFromGroup(user.getUserDN())) {
                                elem.AddMessage("Papel '"+role.getName()+"' removido com sucesso do usurio '"+userID+"'");
                              }
                              else {
                                elem.AddMessage("Usurio '"+userID+"' no possui papel '"+role.getName()+"'");
                              }
                            }
                            else {
                              elem.AddMessage("Remoo do papel '"+role.getName()+"' para o usurio '"+userID+"' no autorizada");
                            }
                          }
                        }
                      }
                      else {
                        elem.AddMessage("No houve papis para atribuir ao usurio '"+userID+"'");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' no est cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Usurio para remover papis no especificado");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void assignRolesToUser(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      RolesElementNode rolesElemNode = (RolesElementNode) userElemNode.roles;
                      if (rolesElemNode != null) {
                        for (int k = 0; k < rolesElemNode.ldapentries.size(); k++) {
                          LdapEntryElementNode ldapEntryNode = (LdapEntryElementNode) rolesElemNode.ldapentries.get(k);
                          if (ldapEntryNode != null) {
                            maca_ca.Role role = new maca_ca.Role(ctx,
                                                                 System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                                                 System.getProperty("maca_ca.ldap.rootDN"),
                                                                 ldapEntryNode.dn.substring(0, ldapEntryNode.dn.indexOf(System.getProperty("maca_ca.ldap.groupsRDN")) - 1));
                            if (this.hasAccess("AtribuirPapel;Usuarios;MACA(\""+user.getUserID()+"\",\""+role.getGroupDN()+"\")", "execuo")) {
                              if (role.addMemberToGroup(user.getUserDN())) {
                                elem.AddMessage("Papel '"+role.getName()+"' atribuido com sucesso para o usurio '"+userID+"'");
                              }
                              else {
                                elem.AddMessage("Usurio '"+userID+"' j possui papel '"+role.getName()+"'");
                              }
                            }
                            else {
                              elem.AddMessage("Atribuio do papel '"+role.getName()+"' para o usurio '"+userID+"' no autorizada");
                            }
                          }
                        }
                      }
                      else {
                        elem.AddMessage("No houve papis para atribuir ao usurio '"+userID+"'");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' no est cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Usurio para atribuir papel no especificado");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void addUser(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (user.isNewUser()) {
                      user.setUserAttributes(userElemNode.ldapentry.ldapattrs);
                      ByteArrayOutputStream buffer = new ByteArrayOutputStream(1024);
                      ObjectOutputStream output = new ObjectOutputStream(new GZIPOutputStream(buffer));
                      output.writeObject(user.getUserAtributes());
                      output.flush();
                      output.close();
                      String userAttrsStr = Conversion.byteArrayToBase64String(buffer.toByteArray());
                      userAttrsStr = userAttrsStr.replace('\n', ' ');
                      if (this.hasAccess("NovoUsuario;Usuarios;MACA(\""+userAttrsStr+"\")", "execuo")) {
                        user.saveUserAtributes();
                        elem.AddMessage("Novo usurio '"+userID+"' gravado com sucesso");
                      }
                      else {
                        elem.AddErrorMessage("Gravao do novo usurio '"+userID+"' no autorizada");
                      }
                      RolesElementNode rolesElemNode = (RolesElementNode) userElemNode.roles;
                      if (rolesElemNode != null) {
                        for (int k = 0; k < rolesElemNode.ldapentries.size(); k++) {
                          LdapEntryElementNode ldapEntryNode = (LdapEntryElementNode) rolesElemNode.ldapentries.get(k);
                          if (ldapEntryNode != null) {
                            maca_ca.Role role = new maca_ca.Role(ctx,
                                                                 System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                                                 System.getProperty("maca_ca.ldap.rootDN"),
                                                                 ldapEntryNode.dn.substring(0, ldapEntryNode.dn.indexOf(System.getProperty("maca_ca.ldap.groupsRDN")) - 1));
                            if (this.hasAccess("AtribuirPapel;Usuarios;MACA(\""+user.getUserID()+"\",\""+role.getGroupDN()+"\")", "execuo")) {
                              if (role.addMemberToGroup(user.getUserDN())) {
                                elem.AddMessage("Papel '"+role.getName()+"' atribuido com sucesso para o usurio '"+userID+"'");
                              }
                              else {
                                elem.AddMessage("Usurio '"+userID+"' j possui papel '"+role.getName()+"'");
                              }
                            }
                            else {
                              elem.AddErrorMessage("Atribuio do papel '"+role.getName()+"' no autorizada");
                            }
                          }
                        }
                      }
                    }
                    else {
                      elem.AddErrorMessage("Usurio '"+userID+"' j est cadastrado");
                    }
                  }
                  else {
                    elem.AddErrorMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddErrorMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddErrorMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddErrorMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddErrorMessage("Entrada LDAP do novo usurio est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private String formatNamesToLowerCase(String aName) {
    StringTokenizer strToken = new StringTokenizer(aName);
    String nome = "";
    while (strToken.hasMoreTokens()) {
      String token = strToken.nextToken();
      if (!token.equalsIgnoreCase("da") &&
          !token.equalsIgnoreCase("de") &&
          !token.equalsIgnoreCase("di") &&
          !token.equalsIgnoreCase("do") &&
          !token.equalsIgnoreCase("du") &&
          !token.equalsIgnoreCase("das") &&
          !token.equalsIgnoreCase("dos")) {
        if (token.length() > 1) {
          token = token.substring(0,1) + token.substring(1, token.length()).toLowerCase();
        }
      }
      else {
        token = token.toLowerCase();
      }
      nome = nome + (nome.equalsIgnoreCase("") ? "" : " ") + token;
    }
    return nome;
  }

  private String composeUidFromGivenName(String givenName) {
    String uid = givenName;
    StringTokenizer strToken = new StringTokenizer(uid);
    if (strToken.countTokens() >= 2) {
      uid = strToken.nextToken();
      String lastStr = "";
      while (strToken.hasMoreTokens()) {
        lastStr = strToken.nextToken();
      }
      uid = (uid + "." + lastStr).toLowerCase();
    }
    return uid;
  }

  private String getLastName(String givenName) {
    String sn = givenName;
    StringTokenizer strToken = new StringTokenizer(sn);
    if (strToken.countTokens() >= 2) {
      while (strToken.hasMoreTokens()) {
        sn = strToken.nextToken();
      }
    }
    return sn;
  }

  private String getCommonName(String givenName) {
    String cn = givenName;
    StringTokenizer strToken = new StringTokenizer(cn);
    if (strToken.hasMoreTokens()) {
      cn = strToken.nextToken();
    }
    return cn;
  }

  private void getNewUserTemplate(RequestElementNode elem) {
    try {
      String employeenumber = elem.attrs.getValue("employeenumber");
      if ((employeenumber == null) || ("".equalsIgnoreCase((employeenumber = employeenumber.trim())))) {
        throw new SAXException("Attributo 'employeenumber' no especificado");
      }
      String givenname = elem.attrs.getValue("givenname");
      if ((givenname == null) || ("".equalsIgnoreCase((givenname = formatNamesToLowerCase(givenname.trim()))))) {
        throw new SAXException("Attributo 'givenname' no especificado");
      }
      String uid = elem.attrs.getValue("uid");
      if ((uid == null) || ("".equalsIgnoreCase((uid = uid.trim())))) {
        uid = composeUidFromGivenName(givenname);
      }
      User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                     System.getProperty("maca_ca.ldap.rootDN"),
                                     "uid="+uid);
      if (user.isNewUser()) {
        String        lookupname = UserSchema.getAttributeParameter("employeenumber", "lookupname");
        String lookupupdatefield = UserSchema.getAttributeParameter("employeenumber", "lookupupdatefield");
        String     lookupchecked = UserSchema.getAttributeParameter("employeenumber", "lookupchecked");
        if (!"".equalsIgnoreCase(lookupname) &&
            !"".equalsIgnoreCase(lookupupdatefield) &&
            "true".equalsIgnoreCase(lookupchecked)) {
          TableModel result = UserSchema.getLookup(lookupname, lookupupdatefield + "='" + employeenumber +"'");
          if (result.getRowCount() == 0) {
            String displayName = UserSchema.getAttributeParameter("employeenumber", "displayName");
            if ("".equalsIgnoreCase(displayName)) displayName = "employeenumber";
            elem.AddErrorMessage("O valor do campo '"+displayName+"' (" + employeenumber +")  invlido");
            return;
          }
        }
        javax.naming.directory.Attributes attrs = user.getUserAtributes();
        attrs.put("employeenumber", employeenumber);
        attrs.put("givenname", givenname);
        attrs.put("cn", this.getCommonName(givenname));
        attrs.put("sn", this.getLastName(givenname));
        user.setUserAttributes(attrs);
        elem.result.write("<user>" + LINE_END);
        user.ldapEntryToXML("*", elem.result, true);
        elem.result.write("</user>" + LINE_END);
      }
      else {
        String displayName = UserSchema.getAttributeParameter("uid", "displayName");
        if ("".equalsIgnoreCase(displayName)) displayName = "uid";
        elem.AddErrorMessage("J existe um usurio com "+displayName+" '"+uid+"' no cadastro");
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void NewUserPassword(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("NovaSenhaUsuario;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        javax.naming.directory.Attribute adminpwd = userLdapEntryNode.ldapattrs.get("adminpassword");
                        if ((adminpwd != null) && (adminpwd.get() != null)) {

                          if (user.setNewPassword((String) adminpwd.get(), new String(User.PWD_FAC.getNewPassword(user.getUserAtributes())))) {
                            user.setPasswordMustChange(true);
                            elem.AddMessage("Nova senha para o usurio '"+userID+"' definida com sucesso");
                          }
                          else {
                            elem.AddMessage("Nova senha no definida");
                          }
                        }
                        else {
                          elem.AddMessage("Nova senha no definida: valor da senha do administrador no especificado");
                        }
                      }
                      else {
                        elem.AddMessage("Definio de nova senha do usurio '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' deve estar cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP onde definir nova senha est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void ChangeUserPassword(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("AlterarSenhaUsuario;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        javax.naming.directory.Attribute userpwd = userLdapEntryNode.ldapattrs.get("userpassword");
                        if ((userpwd != null) && (userpwd.get() != null)) {
                          javax.naming.directory.Attribute adminpwd = userLdapEntryNode.ldapattrs.get("adminpassword");
                          if ((adminpwd != null) && (adminpwd.get() != null)) {
                            if (user.setNewPassword((String) adminpwd.get(), (String) userpwd.get())) {
                              elem.AddMessage("Alterao da senha do usurio '"+userID+"' realizada com sucesso");
                            }
                            else {
                              elem.AddMessage("Senha no alterada");
                            }
                          }
                          else {
                            elem.AddMessage("Senha no alterada: valor da senha do administrador no especificado");
                          }
                        }
                        else {
                          elem.AddMessage("Senha no alterada: valor da nova senha no especificado");
                        }
                      }
                      else {
                        elem.AddMessage("Alterao da senha do usurio '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' deve estar cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP onde alterar senha est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void ChangeUserPasswordPolicyAttributes(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("AlterarAtributosPoliticaSenha;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        javax.naming.directory.Attribute attr = userLdapEntryNode.ldapattrs.get("passwordmustchange");
                        if (attr != null) {
                          String attrVal = (String) attr.get();
                          if ("true".equalsIgnoreCase((String)attr.get())) {
                            user.setPasswordMustChange(true);
                          }
                          else if ("false".equalsIgnoreCase((String)attr.get())) {
                            user.setPasswordMustChange(false);
                          }
                        }
                        attr = userLdapEntryNode.ldapattrs.get("passwordexpwarned");
                        if (attr != null) {
                          String attrVal = (String) attr.get();
                          if ("true".equalsIgnoreCase((String)attr.get())) {
                            user.setPasswordExpirationWarned(true);
                          }
                          else if ("false".equalsIgnoreCase((String)attr.get())) {
                            user.setPasswordExpirationWarned(false);
                          }
                        }
                        attr = userLdapEntryNode.ldapattrs.get("passwordexpirationdate");
                        if (attr != null) {
                          String attrVal = (String) attr.get();
                          if (attrVal == null) {
                            user.setPasswordExpirationDate();
                          }
                          else {
                            user.setPasswordExpirationDate(DF.parse(attrVal));
                          }
                        }
                        attr = userLdapEntryNode.ldapattrs.get("passwordallowchangedate");
                        if (attr != null) {
                          String attrVal = (String) attr.get();
                          if (attrVal == null) {
                            user.setPasswordAllowedChangeDate();
                          }
                          else {
                            user.setPasswordAllowedChangeDate(DF.parse(attrVal));
                          }
                        }
                        elem.AddMessage("Atributos de segurana disponveis para o usurio '"+userID+"' alterados com sucesso");
                      }
                      else {
                        elem.AddMessage("Alterao dos atributos de segurana do usurio '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' deve estar cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP onde alterar atributos de poltica de segurana est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void getUserPasswordPolicyAttributes(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("PegarAtributosPoliticaSenha;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        elem.result.write("<user>" + LINE_END);
                        elem.result.write("<ldapentry dn=\""+user.getUserDN()+"\">" + LINE_END);
                        elem.result.write("<attribute name=\"uid\">" + LINE_END);
                        elem.result.write("<value><![CDATA["+user.getUserID()+"]]></value>" + LINE_END);
                        elem.result.write("</attribute>" + LINE_END);
                        if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordMustChange"))) {
                          elem.result.write("<attribute name=\"passwordmustchange\">" + LINE_END);
                          elem.result.write("<value><![CDATA["+user.passwordMustChange()+"]]></value>" + LINE_END);
                          elem.result.write("</attribute>" + LINE_END);
                        }
                        if (!"false".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordExp"))) {
                          String passwordExpTime = "";
                          try {
                            passwordExpTime = DF.format(user.getPasswordAttributeDate("passwordExpirationTime"));
                            elem.result.write("<attribute name=\"passwordexpirationdate\" active=\"true\" passwordexpwarned=\""+user.passwordExpirationDateWarned()+"\">" + LINE_END);
                          }
                          catch (Exception exc) {
                          	passwordExpTime = DF.format(user.getPasswordDefaultExpirationDate()); 
                            elem.result.write("<attribute name=\"passwordexpirationdate\" active=\"false\" passwordexpwarned=\""+user.passwordExpirationDateWarned()+"\">" + LINE_END);
                          }
                          elem.result.write("<value><![CDATA["+passwordExpTime+"]]></value>" + LINE_END);
                          elem.result.write("</attribute>" + LINE_END);
                        }
                        if (!"0".equalsIgnoreCase(System.getProperty("maca_ca.passwordPolicy.passwordMinAge"))) {
                          String passwordAllowChangeTime = "";
                          try {
                          	passwordAllowChangeTime = DF.format(user.getPasswordAttributeDate("passwordAllowChangeTime"));
                            elem.result.write("<attribute name=\"passwordallowchangedate\" active=\"true\">" + LINE_END);
                          }
                          catch (Exception exc) {
                          	passwordAllowChangeTime = DF.format(user.getPasswordDefaultMinAgeDate());
                            elem.result.write("<attribute name=\"passwordallowchangedate\" active=\"false\">" + LINE_END);
                          }
                          elem.result.write("<value><![CDATA["+passwordAllowChangeTime+"]]></value>" + LINE_END);
                          elem.result.write("</attribute>" + LINE_END);
                        }
                        elem.result.write("</ldapentry>" + LINE_END);
                        elem.result.write("</user>" + LINE_END);
                        elem.AddMessage("Atributos de segurana disponveis para o usurio '"+userID+"' recuperados com sucesso");
                      }
                      else {
                        elem.AddMessage("Recuperao dos atributos de segurana do usurio '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' deve estar cadastrado");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP para recuperar atributos de poltica de segurana est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void removeUser(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("RemoverUsuario;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        user.removeUser();
                        elem.AddMessage("Usurio '"+userID+"' removido com sucesso");
                      }
                      else {
                        elem.AddMessage("Remoo do usurio '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddMessage("Usurio '"+userID+"' deve estar cadastrado para ser removido");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP do usurio para remover est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void lockUserAccount(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("BloquearContaUsuario;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        user.lockAccount();
                        elem.AddMessage("Conta do usurio '"+userID+"' bloqueada com sucesso");
                      }
                      else {
                        elem.AddMessage("Bloqueio da conta do usurio '"+userID+"' no autorizado");
                      }
                    }
                    else {
                      elem.AddMessage("Conta do usurio '"+userID+"' inexistente");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP do usurio para bloquear conta est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void unlockUserAccount(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      if (this.hasAccess("DesbloquearContaUsuario;Usuarios;MACA(\""+user.getUserID()+"\")", "execuo")) {
                        user.unLockAccount();
                        elem.AddMessage("Conta do usurio '"+userID+"' desbloqueada com sucesso");
                      }
                      else {
                        elem.AddMessage("Desbloqueio da conta do usurio '"+userID+"' no autorizado");
                      }
                    }
                    else {
                      elem.AddMessage("Conta do usurio '"+userID+"' inexistente");
                    }
                  }
                  else {
                    elem.AddMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddMessage("Entrada LDAP do usurio para desbloquear conta est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void saveUser(RequestElementNode elem) {
    try {
      for (int n = 0; n < elem.elementsToProcessing.size(); n++) {
        ElementNode elemNode = (ElementNode) elem.elementsToProcessing.get(n);
        if (elemNode instanceof UserElementNode) {
          UserElementNode userElemNode = (UserElementNode) elemNode;
          LdapEntryElementNode userLdapEntryNode = (LdapEntryElementNode) userElemNode.ldapentry;
          if (userLdapEntryNode != null) {
            javax.naming.directory.Attribute uidAttr = userLdapEntryNode.ldapattrs.get("uid");
            if (uidAttr != null) {
              if (uidAttr.size() <= 1) {
                String userID = (String) uidAttr.get();
                if ((userID != null) && (!"".equalsIgnoreCase(userID))) {
                  if (checkUserIDwithUserDN(userID, userLdapEntryNode.dn)) {
                    User user = new User(this.ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                                                   System.getProperty("maca_ca.ldap.rootDN"),
                                                   "uid="+userID);
                    if (!user.isNewUser()) {
                      user.setUserAttributes(userElemNode.ldapentry.ldapattrs);
                      ByteArrayOutputStream buffer = new ByteArrayOutputStream(1024);
                      ObjectOutputStream output = new ObjectOutputStream(new GZIPOutputStream(buffer));
                      output.writeObject(user.getUserAtributes());
                      output.flush();
                      output.close();
                      String userAttrsStr = Conversion.byteArrayToBase64String(buffer.toByteArray());
                      userAttrsStr = userAttrsStr.replace('\n', ' ');
                      if (this.hasAccess("GravarUsuario;Usuarios;MACA(\""+userAttrsStr+"\")", "execuo")) {
                        user.saveUserAtributes();
                        elem.AddMessage("Usurio '"+userID+"' gravado com sucesso");
                      }
                      else {
                        elem.AddErrorMessage("Gravao do usurio '"+userID+"' no autorizada");
                      }
                    }
                    else {
                      elem.AddErrorMessage("Usurio '"+userID+"' deve estar cadastrado para ser gravado");
                    }
                  }
                  else {
                    elem.AddErrorMessage("Identificao do usurio '"+userID+"' e seu LDAP DN '"+userLdapEntryNode.dn+"' esto incompatveis");
                  }
                }
                else {
                  elem.AddErrorMessage("Atributo 'uid' no pode ser nulo");
                }
              }
              else {
                elem.AddErrorMessage("Atributo 'uid' deve ter um nico valor");
              }
            }
            else {
              elem.AddErrorMessage("Atributo 'uid'  obrigatrio");
            }
          }
          else {
            elem.AddErrorMessage("Entrada LDAP para gravar do usurio est vazia");
          }
        }
      }
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private void getGroupTree(RequestElementNode elem) {
    try {
      String baseDN = elem.attrs.getValue("baseDN");
      if (baseDN == null) {
        throw new SAXException("Attributo 'baseDN' no especificado");
      }
      String returningFieldsStr = elem.attrs.getValue("returningFields");
      returningFieldsStr = (returningFieldsStr == null)?"":returningFieldsStr;
      StringTokenizer strTk = new StringTokenizer(returningFieldsStr, ",");
      String[] returningFields = new String[strTk.countTokens()];
      for (int n = 0; strTk.hasMoreTokens(); n++)  returningFields[n] = strTk.nextToken();
      String subTreeScope = "objectscope";
      if ("onelevelscope".equalsIgnoreCase(elem.attrs.getValue("subTreeScope"))) {
        subTreeScope = "onelevelscope";
      }
      else if ("subtreescope".equalsIgnoreCase(elem.attrs.getValue("subTreeScope"))) {
        subTreeScope = "subtreescope";
      }
      boolean members = false;
      if ("true".equalsIgnoreCase(elem.attrs.getValue("members"))) {
        members = true;
      }
      boolean indirectMembers = false;
      if ("true".equalsIgnoreCase(elem.attrs.getValue("indirectMembers"))) {
        indirectMembers = true;
      }
      maca_ca.Role roleRoot = new maca_ca.Role(ctx,
                                               System.getProperty("maca_ca.ldap.groupsRDN")+","+
                                               System.getProperty("maca_ca.ldap.rootDN"),
                                               baseDN);
      roleRoot.ldapEntryToXML(returningFields, subTreeScope, members, indirectMembers, elem.result);
      elem.AddMessage("");
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }

  private String getRestrictedFilter(String filter) throws Exception {
    StringTokenizer strTk = new StringTokenizer(System.getProperty("maca_ca.ldap.filterFields"), ",");
    String[] resources    = new String[strTk.countTokens()];
    String[] filterFields = new String[strTk.countTokens()];
    String[] operations   = new String[strTk.countTokens()];
    for (int i = 0; strTk.hasMoreTokens(); i++) {
      filterFields[i] = strTk.nextToken().trim().toLowerCase();
      resources[i]    = "RestringirPorAtributo;ConsultarUsuario;Usuarios;MACA(\""+filterFields[i]+"\")";
      operations[i]   = "execuo";
    }
    boolean[] authorized = this.hasAccess(resources, operations);
    Vector authoFilterFields = new Vector();
    for (int i = 0; i < filterFields.length; i++) if (authorized[i]) authoFilterFields.add(filterFields[i]);
    Hashtable userAttrs = getUserAttributes();
    String  complAndFilter = "";
    for (int i = 0; i < authoFilterFields.size(); i++) {
      String complOrFilter    = "";
      String authoFilterField = (String) authoFilterFields.get(i);
      if (userAttrs.containsKey(authoFilterField)) {
        Vector values = (Vector) userAttrs.get(authoFilterField);
        for (int k = 0; k < values.size(); k++) {
          if (k == values.size() - 1) {
            complOrFilter += "("+ authoFilterField +"="+values.get(k) + ")";
          }
          else if (k < (values.size() - 1)) {
            complOrFilter += "(|("+ authoFilterField +"="+values.get(k) + ")";
          }
        }
        for (int k = 0; k < values.size() - 1; k++) complOrFilter += ")";
      }
      if (complOrFilter.equalsIgnoreCase("")) complOrFilter = authoFilterField+"=";
      if (i == authoFilterFields.size() - 1) {
        complAndFilter += "("+ complOrFilter + ")";
      }
      else if (i < (authoFilterFields.size() - 1)) {
        complAndFilter += "(&("+ complOrFilter + ")";
      }
    }
    for (int k = 0; k < authoFilterFields.size() - 1; k++) complAndFilter += ")";
    return complAndFilter.equalsIgnoreCase("")?filter:"&("+filter+")"+complAndFilter;
  }

  private void lookupForValidationFields(RequestElementNode elem) {
    try {
      String lookupname = elem.attrs.getValue("lookupname");
      if (lookupname == null) {
        throw new SAXException("Attributo 'lookupname' no especificado pesquisando campos para validao");
      }
      lookupname = lookupname.toLowerCase().trim();
      String lookupfield = elem.attrs.getValue("lookupfield");
      if (lookupfield == null) {
        throw new SAXException("Attributo 'lookupfield' no especificado pesquisando campos para validao");
      }
      lookupfield = lookupfield.trim();
      String lookupfieldvalue = elem.attrs.getValue("lookupfieldvalue");
      if (lookupfieldvalue == null) {
        throw new SAXException("Attributo 'lookupfieldvalue' no especificado pesquisando campos para validao");
      }
      lookupfieldvalue = lookupfieldvalue.trim();
      boolean exactmatch = false;
      if ("true".equalsIgnoreCase(elem.attrs.getValue("exactmatch"))) {
        exactmatch = true;
      }
      String query = "";
      if (exactmatch) {
        query = lookupfield + "= '" + lookupfieldvalue + "'";
      }
      else {
        query = lookupfield + " like '%" + lookupfieldvalue + "%'";
      }
      TableModel result = UserSchema.getLookup(lookupname, query);
      for (int row = 0; row < result.getRowCount(); row++) {
        elem.result.write("<row number=\""+(row+1)+"\">" + LINE_END);
        for (int col = 0; col < result.getColumnCount(); col++) {
          elem.result.write("<attribute name=\""+result.getColumnName(col)+"\">" + LINE_END);
          elem.result.write("<value><![CDATA["+(String)result.getValueAt(row, col)+"]]></value>" + LINE_END);
          elem.result.write("</attribute>" + LINE_END);
        }
        elem.result.write("</row>" + LINE_END);
      }
      elem.AddMessage("A pesquisa retornou "+ result.getRowCount() + " resultados");
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }


  private void searchUsers(RequestElementNode elem) {
    try {
      String filter          = elem.attrs.getValue("filter");
      if (filter == null) {
        throw new SAXException("Attributo 'filter' no especificado pesquisando usurios no servidor LDAP");
      }
      else if (filter.length() == 0) {
      	filter = "uid=*";
      }
      else {
      	filter = filter.trim();
      }
      filter = getRestrictedFilter(filter);
      boolean getRoles = false;
      if ((elem.attrs.getValue("roles") != null) && 
      	  "true".equalsIgnoreCase(elem.attrs.getValue("roles").trim())) {
        getRoles = true;
      }
      boolean tabbed = false;
      if ((elem.attrs.getValue("tabbed") != null) &&  
      	  "true".equalsIgnoreCase(elem.attrs.getValue("tabbed").trim())) {
        tabbed = true;
      }
      String accountType = elem.attrs.getValue("accountType");
      if ((accountType == null) || !(accountType.trim().equalsIgnoreCase("locked") || 
      		                         accountType.trim().equalsIgnoreCase("unlocked"))) {
      	accountType = "*";
      }
      else {
      	accountType = accountType.trim();
      }
      
      String returningFields = elem.attrs.getValue("returningFields");
      StringTokenizer returningFieldsStrTk = new StringTokenizer((returningFields == null)?"":returningFields, ",");
      String[] returningAttrs = new String[returningFieldsStrTk.countTokens()];
      for (int n = 0; returningFieldsStrTk.hasMoreTokens(); n++) {
        returningAttrs[n] = returningFieldsStrTk.nextToken().trim();
      }
      if (accountType.equalsIgnoreCase("locked")) {
      	filter = "&(!(userPassword=*))" + "(" + filter + ")";
      }
      else if (accountType.equalsIgnoreCase("unlocked")) {
      	filter = "&(userPassword=*)" + "(" + filter + ")";
      }
      int size = userGroup.getMembersToXML(filter, returningAttrs, getRoles, tabbed, elem.result);
      elem.AddMessage(((size == 0)?"Nenhum usurio":((size == 1)?"1 usurio":size+" usurios")));
    }
    catch (Exception exc) {
      elem.AddErrorMessage(exc.getMessage());
    }
  }


  //Mtodos pblicos
  public MacaADServerSaxParser(AccessDecision  anAdo, DirContextHolder aCtx) throws Exception {
    ado = anAdo;
    ctx = aCtx;
    userGroup = new Group(ctx, System.getProperty("maca_ca.ldap.peopleRDN")+","+
                               System.getProperty("maca_ca.ldap.rootDN"), "");
  }

  public String parser(org.omg.maca_cs.SecurityLevel2.Credentials aCreds, String request) throws Exception {
    this.creds = aCreds;
    try {
      SAXParser parser = factory.newSAXParser();
      parser.parse(new InputSource(new StringReader(request)), this);
      return result.toString();
    }
    finally {
      this.creds = null;
    }
  }

  public void startDocument() throws SAXException {
    result = new StringWriter(3000 *  1024);
    result.write("<?xml version='1.0' encoding='ISO-8859-1'?>" + LINE_END);
    result.write("<maca_ads version=\"3.1\">" + LINE_END);

    nl();
    nl();
    emit("START DOCUMENT");
    nl();
    emit("<?xml version='1.0' encoding='UTF-8'?>");
  }

  public void endDocument() throws SAXException {
    result.write("</maca_ads>" + LINE_END);

    nl(); emit("END DOCUMENT");
    try {
        nl();
        out.flush();
    }
    catch (IOException e) {
        throw new SAXException("I/O error", e);
    }
  }

  public void characters(char[] buf, int offset, int len) throws SAXException {
    try {
      String s = (new String(buf, offset, len)).trim();
      if (!s.equals("")) { nl(); emit("CHARS:   "+s); }

      ElementNode elem = (ElementNode) elementStack.peek();

      if (elem instanceof AttributeElementNode) { //Se est abaixo de atributo,  um valor de attributo
        AttributeElementNode attrelem = (AttributeElementNode) elem;
        if (attrelem.ldapattr != null) {
          if (!s.equals("")) {
            attrelem.ldapattr.add(s);
          }
        }
      }
    }
    catch (Exception exc) {
      throw new SAXException(exc);
    }
  }

  public void endElement(String uri, String lName, String qName) throws SAXException {
    try {
      String eName = lName; // element name
      if ("".equals(eName)) eName = qName; // namespaceAware = false

      nl(); emit("END_ELM: </"+eName+">");

      if (eName.equals("request")) {
        endRequest();
      }
      else if (eName.equals("user")) {
        endUser();
      }
      else if (eName.equals("ldapentry")) {
        endLdapEntry();
      }
      else if (eName.equals("attribute")) {
        endAttribute();
      }
      else if (eName.equals("value")) {
        endValue();
      }
      else if (eName.equals("roles")) {
        endRoles();
      }

      indentLevel--;
    }
    catch (Exception e) {
        throw new SAXException(e.getMessage(), e);
    }
  }

  public void startElement(String uri, String lName, String qName, Attributes attrs) throws SAXException {
    try {
        indentLevel++;

        String eName = lName; // element name
        if ("".equals(eName)) eName = qName; // namespaceAware = false

        if (eName.equals("request")) {
          elementStack.push(new RequestElementNode(uri, lName, qName, attrs));
          startRequest();
        }
        else if (eName.equals("user")) {
          elementStack.push(new UserElementNode(uri, lName, qName, attrs));
          startUser();
        }
        else if (eName.equals("ldapentry")) {
          elementStack.push(new LdapEntryElementNode(uri, lName, qName, attrs));
          startLdapEntry();
        }
        else if (eName.equals("attribute")) {
          elementStack.push(new AttributeElementNode(uri, lName, qName, attrs));
          startAttribute();
        }
        else if (eName.equals("value")) {
          //Elemento final da rvore (folha) - no empilhar
          //Se precisar dos atributos, passar como parmetro
          startValue();
        }
        else if (eName.equals("roles")) {
          elementStack.push(new RolesElementNode(uri, lName, qName, attrs));
          startRoles();
        }

        nl(); emit("ELEMENT: <"+eName);

        if (attrs != null) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String aName = attrs.getLocalName(i); // Attr name
                if ("".equals(aName)) aName = attrs.getQName(i);
                nl(); emit("   ATTR: "+aName+"\t\""+attrs.getValue(i)+"\"");
            }
        }
        if (attrs.getLength() > 0) nl(); emit(">");
    }
    catch (Exception exc) {
      throw new SAXException(exc);
    }
  }

  public void error(SAXParseException err) throws SAXException {
    try {
      RequestElementNode elem = (RequestElementNode) elementStack.peek();
      elem.AddErrorMessage("** Erro interpretando XML"
        + ", linha " + err.getLineNumber()
        + ", coluna " + err.getColumnNumber()
        + ", uri " + err.getSystemId());
    }
    catch (Exception exc) {
      throw err;
    }
  }

  public void fatalError(SAXParseException err) throws SAXException {
    try {
      RequestElementNode elem = (RequestElementNode) elementStack.peek();
      elem.AddErrorMessage("** Erro fatal interpretando XML"
        + ", linha " + err.getLineNumber()
        + ", coluna " + err.getColumnNumber()
        + ", uri " + err.getSystemId());
    }
    catch (Exception exc) {
      throw err;
    }
  }

  public void warning(SAXParseException err) throws SAXException {
    try {
      RequestElementNode elem = (RequestElementNode) elementStack.peek();
      elem.AddWarningMessage("** Advertncia interpretando XML"
        + ", linha " + err.getLineNumber()
        + ", coluna " + err.getColumnNumber()
        + ", uri " + err.getSystemId());
    }
    catch (Exception exc) {
      throw err;
    }
  }

  //===========================================================
  // Utility Methods ...
  //===========================================================

  // Wrap I/O exceptions in SAX exceptions, to
  // suit handler signature requirements
  private void emit(String s) throws SAXException { /*
    try {
        out.write(s);
        out.flush();
    }
    catch (IOException e) {
        throw new SAXException("I/O error", e);
    } */
  }

  // Start a new line
  // and indent the next line appropriately
  private void nl() throws SAXException { /*
    try {
        out.write(LINE_END);
        for (int i=0; i < indentLevel; i++) out.write(indentString);
    }
    catch (IOException e) {
        throw new SAXException("I/O error", e);
    }*/
  }

}