package maca_cs;

/**
 * <p>Title: MACA_CA</p>
 * <p>Description: Middleware de Autenticao e Controle de Acesso</p>
 * <p>Copyright: Copyright (c) 2000-2004</p>
 * <p>Company: Instituto do Corao - FM.USP</p>
 * @author Gustavo Motta
 * @version 3.2.1
 */

import java.io.*;
import maca_ca.*;
import java.util.*;
import javax.naming.*;
import org.omg.PortableServer.*;
import javax.naming.directory.*;

public class Credentials implements org.omg.maca_cs.SecurityLevel2.CredentialsOperations,
                                    maca_cs.CallbackCredentialsOperations {
  //Constantes globais
  public static final int PRIVILEGE_ATTRIBUTE_CREDENTIALS = 10;

  //Constantes locais
  private final long _creationTimeStamp = (new Date()).getTime();

  //Variveis estticas privadas
  protected static java.util.Timer callbackTimer = new java.util.Timer(true);
  protected static java.util.Timer       timeout = new java.util.Timer(true);


  //Variveis privadas
  private POA  _creds_POA;
  private POA  _poa;
  private byte [] thisServantRef = null;
  private String thisServantRefAsString = "";
  private org.omg.CORBA.ORB _orb = null;
  private UserHolder _userHolder = null;
  private String _userName;
  private boolean isValid = true;
  private LogWriter _log = new LogWriter(new PrintWriter(System.out, true));
  private java.util.TimerTask _timeoutTimerTask = new Timeout();
  private java.util.TimerTask _callbackTimerTask = null;
  private boolean _isUsed = true;

  //Classes locais privadas
  private class Timeout extends java.util.TimerTask {
    public void run() {
      try {
        checkIfCredentialsIsUsed();
      }
      catch (Exception exc) {
      }
    }
  }

  private class CallbackTimer extends java.util.TimerTask {
    private maca_cs.Callback callbackClient = null;

    CallbackTimer(maca_cs.Callback aCallbackClient) {
      callbackClient = aCallbackClient;
    }

    public void run() {
      try {
        checkIfClienteIsAlive(callbackClient);
      }
      catch (Exception exc) {
      }
    }
  }

  // Mtodos private definidos manualmente
  private String[] parsePasswordInfo(byte[] pwdInfo) {
    StringTokenizer strTk = new StringTokenizer(new String(pwdInfo), "\n");
    String[] pwds = new String[2]; //pwds[0] = senha atual, pwds[1] = nova senha
    int n = 0;
    while (strTk.hasMoreTokens()) {
      pwds[n] = (String) strTk.nextToken();
      n = n + 1;
    }
    return pwds;
  }


  private synchronized void checkIfClienteIsAlive(maca_cs.Callback aCallbackClient) {
    try {
      if (isValid) {
        if (aCallbackClient != null) {
          try {
            aCallbackClient.isClientAlive("Cliente "+_userName.toUpperCase()+" est vivo?");
          }
          catch (org.omg.CORBA.COMM_FAILURE exc) {
            _log.println(java.util.logging.Level.WARNING, _userName,
                         "maca_cs.Credentials", "checkIfClienteIsAlive",
                         "Cliente inativo: falha na comunicao CORBA no callback do cliente - "+exc.getLocalizedMessage());
            destroy();
          }
          catch (org.omg.CORBA.SystemException exc) {
            _log.println(java.util.logging.Level.WARNING, _userName,
                         "maca_cs.Credentials", "checkIfClienteIsAlive",
                         "Cliente inativo: falha de sistema CORBA no callback do cliente - "+exc.getLocalizedMessage());
            destroy();
          }
          catch (Exception exc) {
            _log.println(java.util.logging.Level.WARNING, _userName,
                         "maca_cs.Credentials", "checkIfClienteIsAlive",
                         "Cliente inativo: falha no callback do cliente"+exc.getMessage());
            destroy();
          }
        }
      }
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "checkIfClienteIsAlive",
                   "Falha no callback do cliente: "+e.getMessage());
    }
  }

  private synchronized void checkIfCredentialsIsUsed() {
    try {
      if (isValid) {
        if (!this.isUsed()) {
          _log.println(java.util.logging.Level.WARNING, _userName,
                       "maca_cs.Credentials", "checkIfCredentialsIsUsed",
                       "Timeout");
          destroy();
        }
        else {
          this.resetIsUsed(false);
        }
      }
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "checkIfCredentialsIsUsed",
                   "Falha no Timeout: "+e.getMessage());
    }
  }

  // Mtodos protected definidos manualmente
  protected synchronized void resetIsUsed(boolean value) {
    _isUsed = value;
  }

  protected synchronized boolean isUsed() {
    return _isUsed;
  }

  protected void finalize() throws Throwable {
    this.destroy();
//    System.out.println("Credenciais destrudas pelo gc: "+_userName);
    super.finalize();
  }

  //Mtodos pblicos
  public synchronized void setClientCallback (maca_cs.Callback aCallbackClient) {
    this.resetIsUsed(true);
    try {
      if (isValid && !this._userHolder.isLocked()) { 
        long period = 1000 * 60 * Integer.parseInt(System.getProperty("maca_cs.ClientCallbackRefreshTime"));
        try {
          if ((this._callbackTimerTask == null) && (aCallbackClient != null)) {
            this._callbackTimerTask = new CallbackTimer(aCallbackClient);
            callbackTimer.schedule(this._callbackTimerTask, period, period);
            _log.println(java.util.logging.Level.INFO, _userName,
                         "maca_cs.Credentials", "setClientCallback",
                         "Callback do cliente definido");
          }
        }
        catch (java.lang.IllegalStateException e) {
          try {
            _log.println(java.util.logging.Level.SEVERE, _userName,
                         "maca_cs.Credentials", "setClientCallback",
                         "Falha definindo callback do cliente: recriando callback timer - "+e.getMessage());
            synchronized(callbackTimer) {
              callbackTimer = new java.util.Timer(true);
            }
            callbackTimer.schedule(this._callbackTimerTask, period, period);
            _log.println(java.util.logging.Level.INFO, _userName,
                         "maca_cs.Credentials", "setClientCallback",
                         "Callback do cliente definido");
          }
          catch (Exception exc) {
            exc.printStackTrace();
            _log.println(java.util.logging.Level.SEVERE, _userName,
                         "maca_cs.Credentials", "setClientCallback",
                         exc.getMessage());
          }
        }
      }
    }
    catch (Exception exc) {
      exc.printStackTrace();
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "setClientCallback",
                   exc.getMessage());
    }
  }

  public Credentials(LogWriter aLog, org.omg.CORBA.ORB anORB, POA aPoa, String userName)
                               throws java.lang.Exception {
    this.resetIsUsed(true);
    this._userName          = userName.toLowerCase().trim();
    this._orb               = anORB;
    this._poa               = aPoa;
    if (aLog != null) {
      this._log = aLog;
    }
    this._creds_POA = _poa.find_POA("creds_POA", true);
    this._userHolder = (UserHolder) AuthenticatorServer.users.get(_userName);
    if (this._userHolder == null) {
      throw new Exception("Falha criando credenciais["+_userName+"]: usurio no existente");
    }
    try {
      timeout.schedule(this._timeoutTimerTask, AuthenticatorServer.CRED_TIMEOUT, AuthenticatorServer.CRED_TIMEOUT);
    }
    catch (java.lang.IllegalStateException e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "Credentials",
                   "Falha definido timeout do cliente: recriando timeout - "+e.getMessage());
      synchronized(timeout) {
        timeout = new java.util.Timer(true);
      }
      timeout.schedule(this._timeoutTimerTask, AuthenticatorServer.CRED_TIMEOUT, AuthenticatorServer.CRED_TIMEOUT);
    }
    _log.println(java.util.logging.Level.INFO, _userName+" :: "+this._userHolder.getUser().getActiveRolesName(),
                 "maca_cs.Credentials", "Credentials",
                 "Credenciais criadas");
  }

  public String getUniqueIdentifier() {
    return this.thisServantRefAsString;
  }

  public long getCreationTimeStamp () {
    return this._creationTimeStamp;
  }

  public org.omg.PortableServer.Servant createCredentialsPOATie(maca_cs.Credentials credsImpl) throws java.lang.Exception {
//    return new org.omg.maca_cs.SecurityLevel2.CredentialsPOATie(credsImpl);
    //Cria credenciais que implementa a interface de callback, no caso, a
    //interface maca_cs.CallbackCredentials
    return new maca_cs.CallbackCredentialsPOATie((maca_cs.CallbackCredentialsOperations) credsImpl);
  }

  public org.omg.maca_cs.SecurityLevel2.Credentials credentialsNarrow(org.omg.CORBA.Object creds)  throws java.lang.Exception  {
//    return org.omg.maca_cs.SecurityLevel2.CredentialsHelper.narrow(creds);
    //Faz o type casting para a interface de callback, no caso, a
    //interface org.omg.maca_cs.CallbackCredentials
    return maca_cs.CallbackCredentialsHelper.narrow(creds);
  }


  public org.omg.maca_cs.SecurityLevel2.Credentials activateCredentials () throws Exception {
    if (thisServantRef == null) {
      org.omg.PortableServer.Servant cred_Tie = this.createCredentialsPOATie(this);
      thisServantRef = _creds_POA.activate_object(cred_Tie);
      thisServantRefAsString = "";
      for (int n = 0; n < thisServantRef.length; n++) {
        thisServantRefAsString = thisServantRefAsString + Byte.toString(thisServantRef[n]);
      }
      _userHolder.openNewSession(this);
      return this.credentialsNarrow(_creds_POA.servant_to_reference(cred_Tie));
    }
    return this.credentialsNarrow(_creds_POA.id_to_reference(thisServantRef));
  }

  public org.omg.maca_cs.SecurityLevel2.Credentials copy () {
    this.resetIsUsed(true);
    try {
      if (isValid && !this._userHolder.isLocked()) { 
        Credentials credImpl = new Credentials(_log, _orb, _poa, this._userHolder.getUser().getUserID());
        org.omg.maca_cs.SecurityLevel2.Credentials cred = credImpl.activateCredentials();
        _log.println(java.util.logging.Level.WARNING, _userName,
                     "maca_cs.Credentials", "copy",
                     "Credenciais copiadas");
        return cred;
      }
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "copy",
                   e.getMessage());
    }
    return null;
  }

  public synchronized void destroy () {
    try {
      if (isValid) {
        int session = _userHolder.closeSession(this);
        String userActiveRoles = _userHolder.getUser().getActiveRolesName();
        if (session == 0) {
          maca_cs.AuthenticatorServer.users.remove(_userName);
        }
        if (thisServantRef != null) {
          _creds_POA.deactivate_object(thisServantRef);
        }
        _log.println(java.util.logging.Level.INFO, _userName+" :: "+userActiveRoles,
                     "maca_cs.Credentials", "destroy",
                     "Logout - "+session+" sesses abertas remanescentes");
      }
    }
    catch (org.omg.PortableServer.POAPackage.WrongPolicy exc) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "destroy",
                   exc.getMessage());
    }
    catch (org.omg.PortableServer.POAPackage.ObjectNotActive exc) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "destroy",
                   exc.getMessage());
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "destroy",
                   e.getMessage());
    }
    finally {
      if (_timeoutTimerTask != null) {
        _timeoutTimerTask.cancel();
      }
      if (_callbackTimerTask != null) {
        _callbackTimerTask.cancel();
      }
      _userHolder  = null;
      isValid     = false;
    }
  }

  public void set_security_features (org.omg.maca_cs.Security.CommunicationDirection direction,
                                     org.omg.maca_cs.Security.SecurityFeatureValue[] security_features) {
    throw new org.omg.CORBA.NO_IMPLEMENT("Operao 'set_security_features' no disponvel");
  }

  public org.omg.maca_cs.Security.SecurityFeatureValue[] get_security_features (org.omg.maca_cs.Security.CommunicationDirection direction) {
    throw new org.omg.CORBA.NO_IMPLEMENT("Operao 'get_security_features' no disponvel");
  }

  public boolean set_privileges (boolean force_commit,
                                 org.omg.maca_cs.Security.SecAttribute[] requested_privileges,
                                 org.omg.maca_cs.Security.AttributeListHolder actual_privileges) {
    this.resetIsUsed(true);
    boolean result = false;
    Vector attrList = new Vector();
    try {
      for (int n = 0; n < requested_privileges.length; n++) {
        if ((isValid) &&
            (requested_privileges[n].attribute_type.attribute_family.family_definer == 0) && // OMG famlily
            (requested_privileges[n].attribute_type.attribute_family.family == 1)) { // privilege attributes; family = 1
          if ((requested_privileges[n].attribute_type.attribute_type != PRIVILEGE_ATTRIBUTE_CREDENTIALS) &&
          	  !this._userHolder.isLocked()) { //O nico privilgio que o usurio pode alterar quando
                                              //ele est bloquado  a prpria senha
          	
             if ((requested_privileges[n].attribute_type.attribute_type == org.omg.maca_cs.Security.Role.value)) { 
               if (_userHolder.getUser().setActiveRole(new String(requested_privileges[n].value))) {
                 String activeRoles = _userHolder.getUser().getActiveRolesName();
                 org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, requested_privileges[n].attribute_type.attribute_family.family);
                 org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, requested_privileges[n].attribute_type.attribute_type);
                 attrList.add(new org.omg.maca_cs.Security.SecAttribute(attrType, (byte []) requested_privileges[n].defining_authority.clone(), (byte []) requested_privileges[n].value.clone()));
                 _log.println(java.util.logging.Level.INFO, _userName+" :: "+activeRoles,
                              "maca_cs.Credentials", "set_privileges",
                              "Ativao do papel '"+new String(requested_privileges[n].value)+"'");
                 result = true;
               }
             }
          }
          else if (requested_privileges[n].attribute_type.attribute_type == PRIVILEGE_ATTRIBUTE_CREDENTIALS) {
              org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, requested_privileges[n].attribute_type.attribute_family.family);
              org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, requested_privileges[n].attribute_type.attribute_type);
              String[] pwds = this.parsePasswordInfo(requested_privileges[n].value);
              try {
                if (_userHolder.getUser().setNewPassword(pwds[0], // senha atual
                                         pwds[1] // nova senha
                                         )) {
                	_userHolder.unlock(); //Desbloqueia o usurio aps a mudana da senha
                  _log.println(java.util.logging.Level.WARNING, _userName,
                               "maca_cs.Credentials", "set_privileges",
                               "Nova senha definida");
                  result = true;
                }
                else {
                  attrList.add(new org.omg.maca_cs.Security.SecAttribute(attrType, (byte []) requested_privileges[n].defining_authority.clone(), "Senha atual invlida!".getBytes()));
                }
              }
              catch (PasswordErrorException exc) {
                _log.println(java.util.logging.Level.WARNING, _userName,
                             "maca_cs.Credentials", "set_privileges",
                             "Falha na definio nova senha: "+exc.getMessage());
                attrList.add(new org.omg.maca_cs.Security.SecAttribute(attrType, (byte []) requested_privileges[n].defining_authority.clone(), exc.getMessage().getBytes()));
              }
          }
        }
      }
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "set_privileges",
                   "Falha na definio de privilgios: "+e.getMessage());
      result = false;
    }
    org.omg.maca_cs.Security.SecAttribute[] attrSec = new org.omg.maca_cs.Security.SecAttribute[attrList.size()];
    attrList.copyInto(attrSec);
    actual_privileges.value = attrSec;
    return result;
  }

  public org.omg.maca_cs.Security.SecAttribute[] get_attributes (org.omg.maca_cs.Security.AttributeType[] attributes) {
    this.resetIsUsed(true);
    org.omg.maca_cs.Security.SecAttribute[] secAttr = {};
    try {
      Vector attrVec = new Vector();
      if (isValid && !this._userHolder.isLocked()) {
        if (attributes.length > 0) {
          for (int n = 0; n < attributes.length; n++) {
            if (attributes[n].attribute_family.family_definer == 0) { // OMG famlily
              if (attributes[n].attribute_family.family == 1) { // privilege attributes; family = 1
                switch (attributes[n].attribute_type){
                  case org.omg.maca_cs.Security.AccessId.value: {
                    org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, attributes[n].attribute_family.family);
                    org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, attributes[n].attribute_type);
                    attrVec.add(new org.omg.maca_cs.Security.SecAttribute(attrType, new byte[0], _userHolder.getUser().getUserID().getBytes()));
                  };
                  break;
                case org.omg.maca_cs.Security.AttributeSet.value: {
                  org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, attributes[n].attribute_family.family);
                  org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, attributes[n].attribute_type);
                  Attributes attrs = _userHolder.getUser().getUserAtributes();
                  for (int k = 0; k < AuthenticatorServer.USER_ACCOUNT_ATTR_LIST.length; k++) {
                    Attribute attr = attrs.get(AuthenticatorServer.USER_ACCOUNT_ATTR_LIST[k]);
                    if ((attr != null) && (!attr.getID().equalsIgnoreCase("userPassword"))){
                      attrVec.add(new org.omg.maca_cs.Security.SecAttribute(attrType, new byte[0], attr.toString().getBytes()));
                    }
                  }
                };
                break;
                  case org.omg.maca_cs.Security.Role.value: {
                    org.omg.maca_cs.Security.ExtensibleFamily extFamily = null;
                    org.omg.maca_cs.Security.AttributeType    attrType  = null;
                    String activeRoles = _userHolder.getUser().getActiveRolesName();
                    String[] userRoles = _userHolder.getUser().getRoles();
                    for (int i = 0; i < userRoles.length; i++) {
                      extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, attributes[n].attribute_family.family);
                      attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, attributes[n].attribute_type);
                      attrVec.add(new org.omg.maca_cs.Security.SecAttribute(attrType, new byte[0], userRoles[i].getBytes()));
                    }
                    //Especifica os papeis atuais ativos
                    extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, attributes[n].attribute_family.family);
                    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, attributes[n].attribute_type);
                    attrVec.insertElementAt(new org.omg.maca_cs.Security.SecAttribute(attrType, new byte[0], activeRoles.getBytes()), 0);
                  };
                  break;
                }
              }
            }
          }
        }
        else {
          org.omg.maca_cs.Security.ExtensibleFamily extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
          org.omg.maca_cs.Security.AttributeType    attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.AccessId.value);
          attrVec.add(new org.omg.maca_cs.Security.SecAttribute(attrType, new byte[0], _userHolder.getUser().getUserID().getBytes()));
          extFamily = new org.omg.maca_cs.Security.ExtensibleFamily((short) 0, (short) 1);
          attrType  = new org.omg.maca_cs.Security.AttributeType(extFamily, org.omg.maca_cs.Security.Role.value);
          attrVec.add(new org.omg.maca_cs.Security.SecAttribute(attrType, new byte[0], _userHolder.getUser().getActiveRolesName().getBytes()));
        }
      }
      secAttr = new org.omg.maca_cs.Security.SecAttribute[attrVec.size()];
      for (int n = 0; n < attrVec.size(); n++) {
        secAttr[n] = (org.omg.maca_cs.Security.SecAttribute) attrVec.get(n);
      }
    }
    catch (Exception e) {
      _log.println(java.util.logging.Level.SEVERE, _userName,
                   "maca_cs.Credentials", "get_attributes",
                   "Falha na requisio de atributos: "+e.getMessage());
      secAttr = new org.omg.maca_cs.Security.SecAttribute[0];
    }
    return secAttr;
  }

/*  
  public boolean is_valid (org.omg.maca_cs.Security.UtcTHolder expiry_time) {
    this.resetIsUsed(true);
    return isValid;
  }
*/
  public boolean is_valid (org.omg.maca_cs.TimeBase.UtcTHolder expiry_time) {
    this.resetIsUsed(true);
    return isValid;
  }

  public boolean refresh () {
    this.resetIsUsed(true);
    try {
      return this._userHolder.getUser().refresh();
    }
    catch (Exception exc) {
    }
    return false;
  }


}